#!/usr/bin/perl

# This script is a wrapper for Gene3D.
# It calls HMMER3, parses the output into SSF format and then calls DomainFinder3

# By S. Hunter 29/09/2010

BEGIN
{
#$ENV{IPRSCAN_HOME}='/opt/biogrid/usr/iprscan';
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool::Hmmer3;
use English;
use Getopt::Long;
use Sys::Hostname;
use strict;

# Get command-line options
# Need to get: inputfile, HMM file.  All others are optional (and have a default value)
my ($help,$infile, $hmmfile, $hmmscan, $df3, $tmpdir, $zval, $domE, $df_l, $df_e, $min_align);

if ($#ARGV > 0) {
	unless (&GetOptions("help"  => \$help,
			"evalue=f"  => \$df_e,
			"tmpdir=s"  => \$tmpdir,
			"zval=i"    => \$zval,
			"domE=i"    => \$domE,
			"df_l=i"    => \$df_l,
			"min=s"     => \$min_align,
			"df3=s"     => \$df3,
			"hmmscan=s"  => \$hmmscan,
			"i=s"       => \$infile,
			"hmmfile=s" => \$hmmfile)) {
		usage();
		exit 1;
	}

	if ($help) {
		usage();
		exit 0;
	}

	usage() unless($infile && -f $infile);
	usage() unless($hmmfile && -f $hmmfile);
} else {
	usage();
	exit 0;
}

# Set defaults for optional variables
$df_e = 0.001 unless ($df_e);
$tmpdir = $ENV{IPRSCAN_HOME}."/tmp" unless ($tmpdir);
$zval = 100000 unless ($zval);
$domE = 1 unless ($domE);
$df_l = 5900 unless ($df_l);
$min_align = 30 unless ($min_align);
$hmmscan = $ENV{IPRSCAN_HOME}."/bin/binaries/hmmer3/hmmscan" unless ($hmmscan);
$df3 = $ENV{IPRSCAN_HOME}."/bin/binaries/DomainFinder3" unless ($df3);

# Set some other variables
my $unique = hostname() . time() . $PID;
my $resfile = $tmpdir . "/" . $unique . ".res";
my $ssffile = $tmpdir . "/" . $unique . ".ssf";
my $outfile = $tmpdir . "/" . $unique . ".out";
my @files = (); 			# list of files to remove at the end (if successful)
my $debug = 0;


########## Run HMMER3 on input file #########

system("$hmmscan --domE $domE -Z $zval $hmmfile $infile > $resfile"); 

########## Parse HMMER3 output file #########
# Firstly, check if it exists

if ( not -e "$resfile" ) {
	die "Runs finished, but the result file $resfile doesn't exist!\n";
}

if ( -z "$resfile" ) {
	die "Runs finished, but the result file $resfile has zero size!\n";
}

unless(`tail -n 1 $resfile` =~ /\/\//){
	die "Runs finished, but the result file $resfile is incomplete!\n";
}

push (@files,$resfile);

# Next, parse the information from the results file 

my ($g3dres) = Dispatcher::Tool::Hmmer3->new();
my $result = { };
my ($res, $msg) = $g3dres->parseResult($result, $resfile);

die "Failure parsing Gene3D Hmmer result: $msg" unless ($res);

########## Output SSF format temporary file #########

# OUTPUT
#---------------------------------------------------------------------------------
# FILE FORMAT: Sam Summary File (SSF) Format 2.1
#
# Column 1:   Sequence accession code e.g. gi|159385
# Column 2:   HMM model name (not taken from stat or align file; set externally by user)
# Column 3:   Total sequence length
# Column 4:   Total HMM Length
# Column 5:   Length of alignment (upper case letters AND gaps that appear within segments)
# Column 6:   Number of matched residues (upper case letters)
# Column 7:   First residue in sequence to match model (first upper case letter)
# Column 8:   Last residue in sequence to match model (last upper case letter)
# Column 9:   First matched position in model
# Column 10:  Last matched position in model
# Column 11:  Evalue
# Column 12:  Simple score
# Column 13:  Reverse score
# Column 14:  Number of matched sequence segments (2+: discontinuous match)
# Column 15:  Colon-separated segment boundaries e.g. 1:51:99:163
#

open (SSF, ">$ssffile") || die "Cannot open $ssffile: $!\n";

my $nhits = $result->{nhits};

for (my $i = 1; $i <= $nhits; $i++){
	my $ndomains = $result->{$i}->{ndomains};

	# Go through each domain hit, one by one
	for(my $j = 1 ; $j <= $ndomains; $j++) {

		# First of all, sort out the alignments...
		my $alignment = $result->{$i}->{$j}->{alignment};

                # Set counters
                my $firstSeq = 0;	
                my $lastSeq = 0;
                my $firstHmm = 0;
                my $lastHmm = 0;
                my $alilen = 0;
                my $numUC = 0;
                my $numsegs = 1;
                my $bounds = "";

		my $seqpos = $result->{$i}->{$j}->{start} -1;
		my $hmmpos = $result->{$i}->{$j}->{hmmf} -1;

		my $lastCharisUC = 0;      # boolean
		my $discontig = 0;         # length counter

		my @residues = split (//,$alignment);

		$bounds = "$result->{$i}->{$j}->{start}";
		foreach my $a (@residues) {
			if ($a eq "-" && $lastCharisUC) { # gap in the sequences
				$hmmpos++;
				$alilen++; # increment the count of the alignment length
			}
			elsif ($a =~ /[a-z]/) { # deletion in the model compared to the sequence (a gap)
				# I don't think that a sequence can ever start with a LC letter in a match
				# i.e. you shouldn't see "------mAKC"
				$alilen++; # increment the count of the alignment length

				if ($lastCharisUC) { # gap in model match just started (discontiguous model match)
					$discontig = 1;
					$lastCharisUC = 0;
				} else {                # Or we're just continuing the discontiguous region
					$discontig++;
				}
				$seqpos++; # we're also one residue further along

			} 
			elsif ($a =~ /[A-Z]/) { # match in the sequence and model
				$numUC++; # increment the count of Uppercase letters
				$alilen++; # increment the count of the alignment length

				if (!$lastCharisUC && $discontig >= $min_align ) { # We've just finished a discontiguous region
					# NB discontiguous regions (LC letters) do not happen at the end of alignments,
					# but gaps (-) can (however seq-t value is same as last UC letter)
					# The current segment's last seq pos should be same as last UC (i.e. curr pos - discontig value)
					# And the current residue is the first of a new segment
					my $seqend = $seqpos - $discontig;
					$bounds .= ":".$seqend.":".$seqpos;
					$numsegs++;
				}
				$lastCharisUC = 1;
				$seqpos++;
				$hmmpos++;
			}
		}
		$bounds .= ":$result->{$i}->{$j}->{end}";


		# then print out the results
		print SSF "$result->{$i}->{seqid}";		# Sequence AC
		print SSF "\t$result->{$i}->{$j}->{methid}";	# Method AC
		print SSF "\t$result->{$i}->{seqlen}";		# Sequence length
		print SSF "\t0"; 				# The HMM length is always output as zero..
		print SSF "\t$alilen"; 				# Alignment length
		print SSF "\t$numUC"; 				# number of uppercase letters
		print SSF "\t$result->{$i}->{$j}->{start}"; 	# First reside in sequence to match the model
		print SSF "\t$result->{$i}->{$j}->{end}"; 	# Last residue in sequence to match the model
		print SSF "\t$result->{$i}->{$j}->{hmmf}"; 	# First matched position in model
		print SSF "\t$hmmpos"; 				# Last matched position in model
		print SSF "\t$result->{$i}->{$j}->{evalue}";	# Evalue
		print SSF "\t$result->{$i}->{$j}->{score}";	# Score
		print SSF "\t0.00"; 				# Reverse score (always set as zero)
		print SSF "\t$numsegs"; 			# Num segs per domain match
		print SSF "\t$bounds"; 				# Discontiguous boundaries
		print SSF "\n";
	}
}

close(SSF);

print STDERR "SSF FILE is $ssffile\n" if ($debug);
push (@files,$ssffile);


########## Call DomainFinder3 #########

system ("$df3 -p -l 5900 -e 0.001 -i $ssffile -o $outfile");

print STDERR "OUT FILE is $outfile\n" if (-e $outfile && $debug);

push (@files,$outfile);

########## Output results to STDOUT #########

open (OUT,$outfile) || die "Cannot open output file $outfile: $!\n";

while (<OUT>) { print; }

close (OUT);

########## Clean up files ############

foreach (@files) {
	unlink $_;
}

exit 0;

########## Usage subroutine ##########

sub usage {

	print "\nUsage: $0 [-help] -i <seqfile> -hmmfile <hmmfile> [-hmmscan <hmmscan_binary>] ";
	print "[-domE <domE_value>] [-zval <z_value>] [-df3 <df3_binary>] [-evalue <evalue>] ";
	print "[-df_l <l_value>] [-tmpdir <tmp_dir>] [-min <minmum_gapsize>] \n\n";
	print "  -help                      This message\n";
	print "  -i <seqfile>               Your sequence file (mandatory).\n";
	print "  -hmmfile <hmmfile>         Location of HMM library file (mandatory).\n";
	print "  -hmmscan <hmmscan_binary>  Location of hmmscan binary\n";
	print "  -domE <domE_value>         HMMER3 domain evalue cutoff\n";
	print "  -zval <z_value>            HMMER3 Z-value\n";
	print "  -df3 <df3_binary>          Location of DomainFinder3 binary\n";
	print "  -evalue <evalue>           E-value cut off for DomainFinder3\n";
	print "  -df_l <l_value>            Edge limit value for DomainFinder3\n";
	print "  -tmpdir <tmp_dir>          Path to temporary directory\n";
	print "  -min <minimum_gapsize>     Size of alignment gap for domain to be discontiguous\n";
	print "\n\n";

}
# END
