#!/usr/bin/perl
#
# Iterator reads fasta sequences from the input one at a time and
# executes the command for each sequence. Command must use stdout
# to print the output. Use tag %infile on the command line to specify
# the location of the input file.
#
# usage: iterator.pl [-h] -i <infile> -o <outfile> -c "<cmd>"
#

use English;
use File::Basename;
use File::Spec::Functions;
use Getopt::Long;
use Sys::Hostname;

my ($headers, $infile, $outfile, $cmd);

unless (&GetOptions("h"   => \$headers,
                    "i=s" => \$infile,
                    "o=s" => \$outfile,
                    "c=s" => \$cmd)) {
    usage();
    exit 1;
}

unless ($infile && $outfile && $cmd) {
    usage();
    exit 1;
}

unless (-f $infile && -r $infile) { 
    die "input file $infile does not exist\n";
}

unlink($outfile) if -e $outfile;

my ($name, $path, $ext) = fileparse($infile);

unless(open(INFILE, "< $infile")) {
    die "could not read file $infile: $!\n";
}

# FIXME: if the sequence id contains > this will fail
$/ = ">";   # read a one seq entry in one go

my ($tmpfile, $ncmd, $res, $msg, $id);
my $nseq = 1;

while (<INFILE>) {
    chomp $_;
    next unless $_;

    $tmpfile = catfile($path, "iterator-" . hostname() . time() . $PID . ".$nseq");

    unless (open(TMP, "> $tmpfile")) {
	close(INFILE);
	unlink($tmpfile);
	die "could not write file $tmpfile: $!\n";
    }

    print TMP ">$_";
    close(TMP);

    my $ncmd = $cmd;
    $ncmd =~ s/\%infile/$tmpfile/gi;

    if ($headers && $_ =~ /^([^\s]+)/) {
      $id = $1;
      
      # print fasta header to the output file
      ($res, $msg) = systemCall("echo \">>>$id\" >> $outfile");
      unless ($res) {
	close(INFILE);
	unlink($tmpfile);
	die "command '$cmd' failed: $msg\n";
      }
    }

    ($res, $msg) = systemCall("$ncmd >> $outfile");
    unless ($res) {
      close(INFILE);
      unlink($tmpfile);
      die "command '$cmd' failed: $msg\n";
    }	

    unlink($tmpfile);
    $nseq++;
}

close(INFILE);


sub usage() {
    print "usage: $0 -i <infile> -o <outfile> -c <cmd> [-h display header]\n";
}

sub cleanup($) {
    my $aref = shift;
    return unless $aref && @$aref > 0;

    for (@$aref) {
	unlink($_);
    }
}

sub systemCall($) {
    my $cmd = shift;
    return (0, "no command to execute") unless $cmd;

    my $rc = 0xffff & system($cmd);
    my $msg;

    if ($rc == 0) {
        # command exited normally
    }
    elsif ($rc == 0xff00) {
        $msg = "$cmd: $!";
    } 
    elsif (($rc & 0xff) == 0) {
        $rc >>= 8;
        $msg = "$cmd: exit status $rc";
    }  
    else {
        $msg = "$cmd: ";
        if ($rc &   0x80) {
            $rc &= ~0x80;
            $msg .= "coredump from ";
        } 
        $msg .= "signal $rc";
    }
    return ($rc == 0, $msg);
}

