#!/usr/bin/perl
#
# This script creates raw file from output file parsing them.
# It requieres the jobid to be able to get the correct paths etc..
#
# send comments to tuco@ebi.ac.uk
#

BEGIN {
$ENV{IPRSCAN_HOME} ='/opt/biogrid/usr/iprscan';
}

use lib "$ENV{IPRSCAN_HOME}/lib";

use Dispatcher::Tool;
use Dispatcher::Tool::InterProScan;
use strict;
use Getopt::Long;
use File::Basename;

my($help, $jobid, $out, $verbose);

unless(GetOptions('h' => \$help, 'j:s' => \$jobid, 'o:s' => \$out, 'v' => \$verbose)){
    usage();
}

usage() if $help;

print "Creating InterProScan object ... " if $verbose;
my $iprscan = new Dispatcher::Tool::InterProScan();
$iprscan->setToolName('iprscan');
print "DONE\n" if $verbose;

my $defaults = { };

if($jobid =~ /^([\w]+-([\d]{4})(\d\d)(\d\d)-(\d\d)(\d\d)(\d\d)(\d\d))/) {
    # set date and time because they may be used in configuration (paths and URLs)
    $defaults->{YYYY}  = $2;
    $defaults->{MM}    = sprintf("%.2d", $3);
    $defaults->{DD}    = sprintf("%.2d", $4);
    $defaults->{hh}    = sprintf("%.2d", $5);
    $defaults->{mm}    = sprintf("%.2d", $6);
    $defaults->{ss}    = sprintf("%.2d", $7);
    $defaults->{jobid} = $1;
}
else{
    usage("could not determine jobid from '$jobid'");
}

print "Parsing configuration file ... " if $verbose;
my ($res, $msg) = $iprscan->parseConfig(1, $defaults);
die "Parsing configuration file failed : $msg" unless $res;
print "DONE\n" if $verbose;

my $dir = $iprscan->getConfigValue('workdir');
die "Configuration error : 'workdir' not configured." unless $dir;
die "[$dir] not a directory : $!" unless -d $dir;

print "Opening directory $dir ... " if $verbose;
unless(opendir(DIR, "$dir")){
  closedir DIR;
  usage($!);
}
print "DONE\n" if $verbose;

my $pfile = $iprscan->getConfigValue('toolparams');;
die "Configuration error : 'toolparams' not configured" unless $pfile;
die "Unable to find params file : $!" unless -f $pfile;

my $params = { };

print "Reading params file ... " if $verbose;
($res, $msg) = $iprscan->readParamFile($pfile, $params);
die $msg unless $res;
print "DONE\n" if $verbose;

$iprscan->setParams($params);

#We need to parse it again to get everything.
print "Parsing configuration file once again ... " if $verbose;
($res, $msg) = $iprscan->parseConfig(1, $params);
die "Parsing configuration file failed : $msg" unless $res;
print "DONE\n" if $verbose;

my $toolraw = $iprscan->getConfigValue('toolraw');
die "A raw file [$toolraw] already exists in your [$dir] directory.\nPlease rename or remove it to avoid overwriting/duplicate." if (-f $toolraw);

my $h_chunk = { };
my ($chunk, $Ncnk);
my $seqs = { };
my $seqid;
my($cnk, $nseq);

#We need to fill the seqs hash table to get the crc and length for each seqid.
print "Getting keys from params ... " if $verbose;
foreach my $key (grep  { /^seq\.\d+\.\d+\.id/ } keys %$params){
    if($key =~ /^seq\.(\d+)\.(\d+)\.id/){
	$cnk = $1;
	$nseq = $2;
	$seqid = $params->{$key};
	$seqs->{$seqid}          = { } unless($seqs->{seqid});
	$seqs->{$seqid}->{len}   = $params->{"seq.$cnk.$nseq.len"};
	$seqs->{$seqid}->{crc64} = $params->{"seq.$cnk.$nseq.crc64"};
    }else{
	print STDERR "Problem reading params with $key\n";
    }
}
print "DONE\n" if $verbose;

print "Getting chunk directories\n" if $verbose;
foreach my $d (readdir(DIR)) {
    next unless($d =~ /^chunk_\d+/);
    print "\tOpeining dir $dir/$d ... " if $verbose;
    unless(opendir(CNK, "$dir/$d")){
	die "Could not open directory [$dir/$d] : $!";
    }
    print "DONE\n" if $verbose;
    if($d =~ /^chunk_(\d+)/){
	$chunk = $1;
	$Ncnk++;
    }else{
	die "Could not determine chunk number from $d";
    }


    my @files = sort grep { /output$/ } readdir(CNK);
    print STDERR "Chunk $d does not have output files\n" unless(scalar(@files));
    closedir CNK;

    $h_chunk->{$chunk}->{jobs} = { } unless($h_chunk->{$chunk}->{jobs});
    my $jobs = { };
    foreach my $file (@files){
	$file =~ s/\.output$//;
	$jobs->{$file} = 1 unless(exists($jobs->{$file}));
    }
    $h_chunk->{$chunk}->{jobs} = $jobs;
}
closedir(DIR);
print "DONE\n" if $verbose;

$h_chunk->{nbchunk} = $Ncnk;

print "Creating Raw Results ... " if $verbose;
($res, $msg) = $iprscan->createRawResult($seqs, $h_chunk);
die "Failed to create Raw results : $msg" unless $res;
print "DONE\n" if $verbose;

unless($out){
    $out = $iprscan->getConfigValue('toolraw');
}

print "Concatenating all raw files ... " if $verbose;
if(system("find $dir -name \\*.raw -exec cat {} > $out.tmp \\;")){
    die "system command failed : $!";
}
print "DONE\n" if $verbose;

print "Renaming $out.tmp to $out ... " if $verbose;
unless(rename("$out.tmp", $out)){
    die "Failed to rename $out.tmp to $out : $!";
}
print "DONE\n" if $verbose;

exit;

sub usage {
  my ($mess) = @_;
  print "\n**** ERROR: $mess ****\n\n" if($mess);
  print "out2raw.pl is used to parse application output files '*.output' and create a raw file 'merged.raw'.\n";
  print "It requieres the jobid (e.g. iprscan-20050401-13452162) to be able to find where your output files are located.\n\n";
  print "Usage: $0 -j <jobid> -o <output> -h <help> -v\n";
  print "          -j jobid\n";
  print "          -o output file redirection (optional).\n";
  print "          -v verbose mode\n";
  print "          -h Displays this help and exits.\n";
  exit 0;
}



