#!/usr/bin/perl
# *************************************************************
#
# Package:    SWISS-PROT/TREMBL bulkfile utilities
#
# Purpose:    Scan one or more fasta formatted proteins against
#             one or more PROSITE pattterns
#
# Usage:      scanregexpf.pl [options] fasta_file [SP_file]+ > result
#
#             -pattern prosite.patterns
#             -confirm  confirm.patterns
#
# *************************************************************

use Getopt::Long;
    
# * Global constants
$opt_debug = 0;
$opt_pattern = 'prosite.patterns';
$opt_confirm  = 'confirm.patterns';

&GetOptions("debug=i", "warn=i", "pattern=s", "confirm=s");

# * Main
$code = precompile_main_loop($opt_pattern, $opt_confirm);
eval($code);
warn $@ if $@;


#
# precompile main loop
# 
# for speed reasons, we construct the matching code of the main
# loop to be eval'ed later
#
# Input  patternfilename
# Input  confirmfilename
# Output perl code
sub precompile_main_loop {
  my $patternfile = shift;
  my $confirmfile  = shift;
  
  my %confirms  = read_confirm_patterns($confirmfile);
  my $patterncode = read_prosite_patterns($patternfile, \%confirms);

  my $code = <<'END_OF_CODE';
  $/ = ">";      # Read an entire fasta entry at a time
  while (<>){
    my $ac;           # AC of the entry
    my $seq;          # buffer for sequence
    my %prosite = (); # prosite_ac -> matchcount
    
  
    # Read the entry
    chomp; next unless $_;
    
    # Get accession
#    s/^([^\s;\|]+)(.*)$//m; 
    s/^([^\s;]+)(.*)$//m; #to allow ncbi stile ids
    $ac = $1;
    die "No accession number found." unless $ac;
    
    # Get sequence
    s/\s//mg;

    #study;
    
    # the PROSITE matching code will be inserted here
    DOPATTERNMATCH;
    
    print "\n"; #to separate the records
  }
END_OF_CODE

  $code =~ s/DOPATTERNMATCH/$patterncode/;

  print STDERR "$code\n" if $main::opt_debug > 2;
  return $code;
}


#
# Read a CONFIRMATIONAL *.pattern file and put it in a hash
# 
# Input     scalar path&filename of the .pattern file
# Output    scalar Perl-code for parsing the patterns
# Exception dies if file not readable
#
sub read_confirm_patterns {
  my $filename = shift;  # Filename of a *.pattern file to be read.
  my %confirms = ();
  
  # local variables
  my $code = "\n";       # Perl-code to be constructed

  open (PAT,"<$filename") || die "Cannot open $filename";
  while (<PAT>){
    my ($patternac,$pattern,$name,$taxonrange)=split(' ');
    push @{$confirms{$patternac}}, $pattern;
  }
  close (PAT);
  return %confirms;
}

#
# Read a PROSITE *.pattern file and convert it to Perl-code
# 
# The *.pattern files contain PROSITE patterns in Unix style, f.i.
# 'PS00017 [AG]....GK[ST] ATP_GTP_A ??E??'
# instead of the original PROSITE pattern '[AG]-x(4)-G-K-[ST]'.
#
# The code assumes the following when being executed
# $_        contains amino acid sequence
# $ac       accession number of the sequence
#
# Every line is converted to Perl-code that matches the pattern.
#
# Input     scalar path&filename of the .pattern file
# Output    scalar Perl-code for parsing the patterns
# Exception dies if file not readable
#
sub read_prosite_patterns {
  my $filename = shift;  # Filename of a *.pattern file to be read.
  my $confirms  = shift;  # Reference to a hash containing the confirm patterns

  # local variables
  my $code = "\n";       # Perl-code to be constructed
  my $patternac;         # PROSITE accession number
  my $pattern;           # PROSITE pattern
  my $name;              # PROSITE name
  my $taxonrange;        # PROSITE taxonomic range
  
  open (PAT,"<$filename") || die "Cannot open $filename";
  while (<PAT>){
    ($patternac,$pattern,$name,$taxonrange)=split(' ');
    $taxonrange =~ s/\?//g; 
    
    $code .= "    while(/$pattern/g){\n";
    $code .= "      my (\$match,\$start,\$end,\$confirmed) = (\$&,length(\$\`)+1,pos \$_,0);\n";
    
    if ($$confirms{$patternac}){
    	
      foreach my $pattern (@{$$confirms{$patternac}}){
        $code .= "      \$confirmed ||= 1 if \$match =~ /$pattern/;\n";
      }
      $code .= "      \$confirmed = (\$confirmed ? 'T' : '?');\n";
      $code .= "      print \"\$ac\t$patternac\t$name\t\$start\t\$end\t\$confirmed\\n\";\n";
    } else {
      $code .= "      print \"\$ac\t$patternac\t$name\t\$start\t\$end\t?\\n\";\n";
    }
    
    $code .= "    };\n";
  }
  close (PAT);
  return $code;
}



