=head1 NAME

Config - interface to configuration files.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Config;

  my $config = new Dispatcher::Config();

  # parse fasta configuration, expand tags
  my ($res, $msg) = $config->parse('fasta', 1);
  unless ($res) {
      # handle error, message is in $msg
      ...
  }

  # get value from configuration
  my $infile = $config->get('toolinput');

  # set value to configuration
  $config->set('tmpinput', "$infile.tmp");

  # get all values in a hash
  my $defaults = $config->getValues();

  # initialize second configuration by giving default values
  my $qconfig = new Dispatcher::Config();
  ($res, $msg) = $qconfig->parse('lsf42', 1, $defaults);
  unless ($res) {
      # handle error, message is in $msg
      ...
  }

=head1 DESCRIPTION

=head1 VERSIONS

$Id: Config.pm,v 1.2 2010/01/20 13:51:23 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

interhelp@ebi.ac.uk

=cut


package Dispatcher::Config;

use Data::Dumper;
use English;
use File::Spec::Functions;
use Sys::Hostname;
use strict;

use vars qw($VERSION);
$VERSION = '0.1';


=head1 CONSTRUCTORS

=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        Configuration name, e.g., 'fasta'.
              $expand      1: expand tags.
                           0: do not expand tags (default).
              $defaults    Default values, hash reference (optional).
 Returns:     $self object

=cut

sub new() {
    my ($class, $name, $expand, $defaults) = @_;
    my $self = { };
    bless $self, $class;

    if ($name) {
        my ($res, $msg) = $self->parse($name, $expand, $defaults);
        unless ($res) {
            print STDERR "$0: ERROR init $name configuration failed:\n$msg\n";
        }
    }

    return $self;
}


=head1 PUBLIC METHODS


=head2 parse

 Description: Parses configuration file and loads it to memory.
 Arguments:   $name        Configuration name, e.g., 'fasta'.
              $expand      1: expand tags.
                           0: do not expand tags (default).
              $defaults    Default values, hash reference (optional).
              $checkuf     1: check unique files (uniquefiles tag).
                           0: do not check unique files (default).
                           This can be used only with expand = 1.
                           Use it when the job is submitted to
                           guarantee unique file names containing
                           timestamps and random numbers.
              $file        Configuration file (optional). Default is
                           $ENV{IPRSCAN_HOME}/conf/$name.conf
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub parse($;$$$$) {
    my ($self, $name, $expand, $defaults, $checkuf, $file) = @_;

    unless ($file) {
      return (0, "environment variable IPRSCAN_HOME not set")
	unless $ENV{IPRSCAN_HOME};

        $file = catfile($ENV{IPRSCAN_HOME}, 'conf', "$name.conf");
    }

    return (0, "could not parse $file: no such file") unless -r $file;

    my $config = { };
    my $backup = { };  # keep backup to check unique jobid later
    my ($key, $value);

    if (!open(FH, "< $file")) {
        return (0, "could not parse $file: $!");
    }
    while (<FH>) {
        chomp;              # no newline
        s/^\#.*//;          # no comments 
        s/^\s+//;           # no leading white
        s/\s+$//;           # no trailing white
        next unless length; # anything left?
        ($key, $value) = split(/\s*=\s*/, $_, 2);
        $config->{$key} = $value;
        $backup->{$key} = $value;
    }

    close(FH);

    # cache the parsed configuration
    $self->{name}   = $name;
    $self->{config} = $config;
    $self->setDefaults($defaults) if $defaults;

    if ($expand && !$checkuf) {
        $self->setDefaults($defaults) if $defaults;
        $self->expandTags($defaults);
    }
    elsif ($expand && $checkuf) {
        my $tries = 10;
        my $unique;
        my $checkFiles;
        my $file;

        while ($tries > 0) {
            $self->setDefaults($defaults) if $defaults;
            $self->expandTags($defaults);

            $checkFiles = $self->get('uniquefiles');
            $unique = 1;

            my @files = split /,/, $checkFiles if $checkFiles;

            for (@files) {
                $file = $self->get($_);
                if (-e $file) {
                    $unique = 0;
                    last;  # exit for loop
                }
            }

            last if $unique;  # exit while loop

            # filename clashed, try to expand tags again
            $self->{config} = $backup;
            $tries--;
            sleep(1);
        }

        unless ($unique) {
            return (0, "could not guarantee unique filename in $name configuration, " .
                    "$file clashed with an existing file (possible configuration error)");
        }
    }

    return (1, '');
}


=head2 getName

 Description: Returns configuration name.
 Arguments:   -
 Returns:     name as string

=cut

sub getName() {
    my $self = shift;
    return $self->{name};
}


=head2 get

 Description: Get value from configuration.
 Arguments:   $key     Configuration key.
 Returns:     value as string

=cut

sub get($) {
    my ($self, $key) = @_;
    return '' unless $key;

    unless ($self->{config}) {
        # try to parse configuration
        my ($res, $msg) = $self->parse($self->{name});
        return '' unless $res;
    }

    my $value = $self->{config}->{$key};
    return '' unless $value;
    return $value;
}


=head2 getValues

 Description: Returns configuration values in a hash.
 Arguments:   -
 Returns:     Hash reference, empty if there is no configuration.

=cut

sub getValues($) {
    my $self = shift;

    unless ($self->{config}) {
        # try to parse configuration
        my ($res, $msg) = $self->parse($self->{name});
        return { } unless $res;
    }
    return $self->{config};
}


=head2 set

 Description: Sets value to configuration.
 Arguments:   $key      Configuration key.
              $value    Configuration value.
 Returns:     -

=cut

sub set($$) {
    my ($self, $key, $value) = @_;
    return unless $key;

    unless ($self->{config}) {
        # try to parse configuration
        my ($res, $msg) = $self->parse($self->{name});
        return unless $res;
    }

    $self->{config}->{$key} = $value;
}


=head2 remove

 Description: Removes key-value pair from configuration.
 Arguments:   $key    Configuration key.
 Returns:     -

=cut

sub remove($) {
    my ($self, $key) = @_;
    return unless $key;

    unless ($self->{config}) {
        # try to parse configuration
        my ($res, $msg) = $self->parse($self->{name});
        return unless $res;
    }

    delete $self->{config}->{$key};
}


=head2 setDefaults

 Description: Set default values to configuration.
 Arguments:   $defaults    Default values, hash reference.
 Returns:     -

=cut

sub setDefaults($) {
    my ($self, $defaults) = @_;
    return unless $defaults && ref($defaults) eq "HASH";

    unless ($self->{config}) {
        # try to parse configuration
        my ($res, $msg) = $self->parse($self->{name});
        return unless $res;
    }

    for (keys %$defaults) {
        $self->set($_, $defaults->{$_});
    }
}


=head1 PRIVATE METHODS


=head2 expandTags

 Description: Expands tags in configuration.
 Arguments:   $defaults    Default values, hash reference (optional).
 Returns:     -

=cut

sub expandTags(;$) {
    my ($self, $defaults) = @_;

    unless ($self->{config}) {
        # try to parse configuration
        my ($res, $msg) = $self->parse($self->{name}, 0, $defaults);
        return unless $res;
    }

    my ($key, $value);

    foreach $key (keys %{$self->{config}}) {
        $value = $self->expand($key, $self->{config}->{$key}, $defaults);
        $self->{config}->{$key} = $value;
    }
}


=head2 expand

 Description: Expands tags in a configuration value.
 Arguments:   $key         Configuration key.
              $value       Configuration value.
              $defaults    Default values, hash reference (optional).
 Returns:     -

=cut

sub expand($$;$) {
    my ($self, $key, $value, $defaults) = @_;
    return unless $key && defined $value;

    while ($value =~ /\[(%[\w\.]+(?:\s*[^\]]+)?)\]/) {
        my $tag = $1;
        my ($name, @params) = split / /, $tag;
        my $pstr = join ' ', @params;

        if ($name eq '%env' && $pstr) {
            my $env = $ENV{$pstr} || '';
            $value =~ s/\[$tag\]/$env/g;
        }
        elsif ($name eq '%if' && $pstr) {
            my ($lop, $rop, $op, $tval, $fval);
            my $opre = "lt|<|le|<=|=<|eq|==|ge|>=|=>|gt|>|ne|!=";

            if ($pstr =~ /%([\w\.]+)\s+($opre)\s+([^\?]+)\s+\?\s+([^\:]+)\s+\:\s+([^\s]+)/) {
                $lop  = $self->{config}->{"$1"};
                $op   = $2;
                $rop  = $3;
                $tval = $4;
                $fval = $5;
            }
            elsif ($pstr =~ /%([\w\.]+)\s+\?\s+([^\?]+)\s+\:\s+([^\:]+)/) {
                $lop = $self->{config}->{"$1"};
                $tval = $2;
                $fval = $3;
            }
            elsif ($pstr =~ /%([\w\.]+)\s+\?\s+([^\?]+)/) {
                $lop = $self->{config}->{"$1"};
                $tval = $2;
                $fval = '';
            }

            if ($rop && $rop =~ /^["'](.*)["']$/) {
                $rop = $1;
            }
            if ($rop =~ /%([\w\.]+)/) {
                $rop = $PREMATCH . $self->{config}->{"$1"} . $POSTMATCH;
            }

            if ($tval && $tval =~ /^["'](.*)["']$/) {
                $tval = $1;
            }
            if ($tval =~ /%([\w\.]+)/) {
                $tval = $PREMATCH . $self->{config}->{"$1"} . $POSTMATCH;
            }

            if ($fval && $fval =~ /^["'](.*)["']$/) {
                $fval = $1;
            }
            if ($fval =~ /%([\w\.]+)/) {
                $fval = $PREMATCH . $self->{config}->{"$1"} . $POSTMATCH;
            }

            #print STDERR "l:$lop o:$op r:$rop t:$tval f:$fval\n";

            if ($lop && $rop && $op) {
                if ($op eq 'eq' || $op eq '==') {
                    $value = $lop eq $rop ? $tval : $fval;
                }
                elsif ($op eq 'ne' || $op eq '!=') {
                    $value = $lop ne $rop ? $tval : $fval;
                }
                elsif ($op eq 'le' || $op eq '<=' || $op eq '=<') {
                    $value = $lop <= $rop ? $tval : $fval;
                }
                elsif ($op eq 'lt' || $op eq '<') {
                    $value = $lop < $rop ? $tval : $fval;
                }
                elsif ($op eq 'ge' || $op eq '>=' || $op eq '=>') {
                    $value = $lop >= $rop ? $tval : $fval;
                }
                elsif ($op eq 'gt' || $op eq '>') {
                    $value = $lop > $rop ? $tval : $fval;
                }
                else {
                    $value = '';
                }
            }
            elsif ($lop) {
                $value = $tval;
            }
            elsif (!$lop) {
                $value = $fval;
            }
            else {
                $value = '';
            }
        }
        elsif ($name eq '%switch' && $pstr) {
            my ($choice, @cases) = split / /, $pstr;
            return '' unless @cases > 0;

            if ($choice =~ /%([\w\.]+)/) {
                $choice = $self->{config}->{"$1"};
            }

            my ($case, $default, $cons, $expr, $svalue);

            foreach $case (@cases) {
                ($cons, $expr) = split /:/, $case;
                next unless $cons && $expr;

                if ($cons =~ /^["'](.*)["']$/) {
                    $cons = $1;
                }

                if ($cons && $cons eq 'default') {
                    $default = $expr;
                }
                elsif ($cons && $cons eq $choice) {
                    $svalue = $expr;
                    last;
                }
            }

            $default = '' unless $default;
            $svalue = $default unless $svalue;

            if ($svalue && $svalue =~ /%([\w\.]+)/) {
                $value = $self->{config}->{"$1"};
            }
            elsif ($svalue) {
                $value = $svalue;
            }
            else {
                $value = '';
            }
        }
        elsif ($name eq '%random' && $pstr) {
            srand($PID);
            my $n = int(rand($pstr));
            $n = sprintf('%.' . (length $pstr - 1)  . 'd', $n);
            $n = $defaults->{random} if $defaults && exists $defaults->{random};
            $value =~ s/\[$tag\]/$n/g;
        }
        elsif ($tag eq '%YYYY') {
            my $year = (localtime(time))[5];
            $year += 1900;
            $year = $defaults->{YYYY} if $defaults && exists $defaults->{YYYY};
            $value =~ s/\[$tag\]/$year/g;
        }
        elsif ($tag eq '%MM') {
            my $month = (localtime(time))[4];
            $month++;
            $month = $defaults->{MM} if $defaults && exists $defaults->{MM};
            $month = sprintf("%.2d", $month);
            $value =~ s/\[$tag\]/$month/g;
        }
        elsif ($tag eq '%DD') {
            my $day = (localtime(time))[3];
            $day = $defaults->{DD} if $defaults && exists $defaults->{DD};
            $day = sprintf("%.2d", $day);
            $value =~ s/\[$tag\]/$day/g;
        }
        elsif ($tag eq '%hh') {
            my $hour = (localtime(time))[2];
            $hour = $defaults->{hh} if $defaults && exists $defaults->{hh};
            $hour = sprintf("%.2d", $hour);
            $value =~ s/\[$tag\]/$hour/g;
        }
        elsif ($tag eq '%mm') {
            my $min = (localtime(time))[1];
            $min = $defaults->{mm} if $defaults && exists $defaults->{mm};
            $min = sprintf("%.2d", $min);
            $value =~ s/\[$tag\]/$min/g;
        }
        elsif ($tag eq '%ss') {
            my $sec = (localtime(time))[0];
            $sec = $defaults->{ss} if $defaults && exists $defaults->{ss};
            $sec = sprintf("%.2d", $sec);
            $value =~ s/\[$tag\]/$sec/g;
        }
        elsif ($tag eq '%hostname') {
            my $host = hostname;
            $host = $defaults->{hostname} if $defaults && exists $defaults->{hostname};
            $value =~ s/\[$tag\]/$host/g;
        }
        elsif ($tag eq '%pid') {
            my $pid = $PID;
            $pid = $defaults->{pid} if $defaults && exists $defaults->{pid};
            $value =~ s/\[$tag\]/$pid/g;
        }
        elsif ($tag eq '%uname'){
	    my $os = `uname`;
	    $os =~ s/\n//g;
            $os = $defaults->{arch} if $defaults && exists $defaults->{arch};
            $value =~ s/\[$tag\]/$os/g;
        }
        elsif ($tag =~ /^%[\w\.]+$/) {
            # this tag references another key-value pair
            my $k = substr($tag, 1);
            my $v = $self->{config}->{$k};
            $v = '' unless defined $v;

            # avoid infinite loop if keys are same
            return '' if $k eq $key;

            if ($v =~ /\[(%[\w\.]+(?:\s*[^\]]+)?)\]/) {
                # expand tags recursively
                $v = $self->expand($k, $v, $defaults);
            }

            # Set the value to configuration so it will be expanded only once.
            # This is necessary to avoid tag to be expanded multiple times,
            # for example, %random in %jobid must be evaluated only once, not
            # each time %jobid is encountered.
            $self->{config}->{$k} = $v;

            $value =~ s/\[$tag\]/$v/;
        }
        else {
            # tag is not recognized, delete it from string to avoid infinite loop
            $value = "$PREMATCH$POSTMATCH";
            last;
        }
    }

    return $value;
}


=head2 trim

 Description: Trims leading and trailing whitespace.
 Arguments:   $s    String.
 Returns:     trimmed string

=cut

sub trim($) {
    my ($self, $s) = @_;
    return '' unless defined $s;
    $s =~ s/^\s+//;
    $s =~ s/\s+$//;
    return $s;
}


1;
__END__
