=head1 NAME

Queue - interface to queuing systems.

=head1 SYNOPSIS

There will be a proper instructions later.

=head1 DESCRIPTION

=head1 VERSIONS

$Id: Queue.pm,v 1.2 2010/01/20 13:51:53 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

interhelp@ebi.ac.uk

=cut


package Dispatcher::Queue;

use Data::Dumper;
use English;
use File::Basename;
use File::Copy;
use File::Path;
use File::Spec::Functions;
use Sys::Hostname;

use Dispatcher::Config;

use strict;

use vars qw($VERSION);
$VERSION = '0.1';

my $Root;
my $Log;
my $DefaultTemplate;
my $Debug = 0;

use constant DEFAULT_DIR_MODE => '0755';

BEGIN {
    $Root = $ENV{IPRSCAN_HOME};

    unless (-d $Root) {
        my $error = "ERROR Dispatcher directory $Root does not exist\n";
        $error .= "Please set environment variable IPRSCAN_HOME\n";
        die($error);
    }

    $DefaultTemplate = catfile($Root, "conf", "default.sh");
    unless (-r $DefaultTemplate) {
        print STDERR "WARNING Dispatcher default template $DefaultTemplate does not exist\n";
    }
}

# job states
use constant JOB_PENDING   => 1;
use constant JOB_RUNNING   => 2;
use constant JOB_DONE      => 3;
use constant JOB_FAILED    => 4;
use constant JOB_NOT_FOUND => 5;

# file extensions
use constant EXT_SUBMIT    => ".dsub";
use constant EXT_TOOLCMD   => ".dcmd";
use constant EXT_KILL      => ".dkil";
use constant EXT_CHECK     => ".dchk";
use constant EXT_JOBID     => ".djob";

# state variables
use constant SHEBANG_NONE  => 0;
use constant SHEBANG_PERL  => 1;
use constant SHEBANG_CSH   => 2;
use constant SHEBANG_SH    => 3;

# cache for job identifiers
my $QueueJobIDs = { };


=head1 CONSTRUCTORS

=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   -
 Returns:     $self object

=cut

sub new() {
    my $class = shift;
    my $self = { };
    bless $self, $class;
    $self->_init();
    return $self;
}


=head1 PUBLIC METHODS


=head2 submit

 Description: Submits a new job to the queue.

              Tool configuration MUST define following values:
              'queue'   => batch queue system name
              'jobid'   => unique job id
              'cmdline' => tool command line to be executed

              If the queue configuration contains tags that are specific
              to the tool, they must be set in the tool configuration.
              For example, 'lsf42' uses 'resource' and 'jobname' tags,
              which are set in the tool configuration. When queue
              configuration is created, the tool configuration is used
              to override all key-value pairs in queue configuration.
              This is why you should be familiar with the keys in the
              queue configuration to avoid unwanted conflicts.

 Arguments:   $tool    Tool name or Dispatcher::Config object reference.
                       When tool name is given, this method creates
                       Dispatcher::Config object for the tool and parses
                       the tool configuration.

              $input   Reference to an array (optional). Each array element
                       can be a string (path to the file), a FileHandle
                       reference or a FileHandle glob. Please read
                       "perldoc File::Copy" for additional information.

              $sync    0: asynchronous submission (default).
                       1: synchronous submission.

 Returns:     1, job ID on success
              0, msg    on failure

=cut

sub submit($;$$) {
    my ($self, $tool, $input, $sync) = @_;
    return (0, "cannot submit without tool name") unless $tool;

    my ($res, $msg, $cmdline);

    ($res, $tool) = $self->_createConfig($tool);
    return (0, $tool) unless $res;

    # copy input file(s) to shared directory
    ($res, $msg) = $self->copyInputFiles($tool, $input) if $input;
    return (0, $msg) unless $res;

    my $tname = $tool->getName();
    $cmdline = $sync ? $tool->get('cmdline.syncsub') : $tool->get('cmdline.asyncsub');
    $cmdline = $tool->get('cmdline') unless $cmdline;
    return (0, "$tname cmdline not defined") unless $cmdline;

    my $qname = $tool->get('queue');
    return (0, "$tname queue not configured") unless $qname;

    my $jobid = $tool->get('jobid');
    return (0, "$tname jobid not configured") unless $jobid;

    # We need to parse the queue configuration at this point to resolve
    # the directory where command files will be written. Use tool config
    # values as defaults to set any tool dependent values.
    my $queue = new Dispatcher::Config();
    ($res, $msg) = $queue->parse($qname, 1, $tool->getValues());
    return (0, "failed to parse $qname configuration: $msg") unless $res;

    $Debug = $queue->get('debug');  # set debug flag

    my $dir = $queue->get('workdir');
    $dir = $queue->get('default.workdir') unless $dir && -d $dir;
    return (0, "$qname default.workdir not configured") unless $dir;
    return (0, "$qname default.workdir $dir does not exist") unless -d $dir;

    # create unique command file prefix
    my $unique = catfile($dir, $jobid);

    # get regular expression matching queue specific jobid
    my $jobidRE = $queue->get('jobid.regexp');
    return (0, "$qname jobid.regexp not configured") unless $jobidRE;

    # get template for tool command file (optional)
    my $ttemplate = $tool->get($sync ? 'template.syncsub' : 'template.asyncsub');
    $ttemplate = $tool->get('template') unless $ttemplate;
    $ttemplate = $DefaultTemplate unless $ttemplate;

    # create tool command file
    $tool->set('cmd', $cmdline);
    ($res, $msg) = $self->createCmdFile($unique . EXT_TOOLCMD, $ttemplate,
					$tool->getValues());
    return (0, "failed to create $tname command file: $msg") unless $res;

    # set the tool command file to tool configuration
    $tool->set('toolcmd', $unique . EXT_TOOLCMD);

    # reload queue configuration to set the toolcmd to submission command
    ($res, $msg) = $queue->parse($qname, 1, $tool->getValues());
    return (0, "failed to reload $qname configuration: $msg") unless $res;

    # get submission command
    my $subcmd = $self->getToolSpecificValue($queue,
					     $sync ? 'syncsub' : 'asyncsub',
					     $tname,
					     $tool->get('toolgroup'));
    return (0, "$qname " . ($sync ? 'syncsub' : 'asyncsub') .
	    " command not configured") unless $subcmd;

    # get template for submission command file
    my $stemplate = $self->getToolSpecificValue($queue,
						$sync ? 'syncsub.template' : 'asyncsub.template',
						$tname, $tool->get('toolgroup'));
    $stemplate = $queue->get('default.template') unless $stemplate;
    $stemplate = $DefaultTemplate unless $stemplate;

    # create submission command file
    $queue->set('cmd', $subcmd);
    ($res, $msg) = $self->createCmdFile($unique . EXT_SUBMIT, $stemplate,
					$queue->getValues());
    return (0, "failed to create $qname submission file: $msg") unless $res;

    # execute the submission command
    my ($stdout, $stderr);
    ($res, $msg) = $self->exec($unique . EXT_SUBMIT, \$stdout, \$stderr);
    if (!$res) {
	# PLEASE NOTE: LSF bsub -K option writes <<Job is finished>> to
        # stderr so don't interpret length(stderr) > 0 as an error!
	my $error = "failed to execute $qname submit: $msg";
	$error .= "\n$stderr" if length($stderr) > 0;
	return (0, $error);
    }

    # write queue specific jobid from stdout or stderr output to cache and file
    if (($stdout && $stdout =~ /$jobidRE/) || ($stderr && $stderr =~ /$jobidRE/)) {
	my $qjobid  = $1;
	$QueueJobIDs->{"$jobid"} = $qjobid;  # cache to memory

	my $jobfile = $unique . EXT_JOBID;
	unless (open(JOBFILE, "> $jobfile")) {
	    return (0, "failed to write $qname jobid to file $jobfile: $!");
	}
	print JOBFILE $qjobid;
	close JOBFILE;
    }
    else {
	$msg  = "failed to determine $qname jobid from submission output ";
	$msg .= $unique . EXT_SUBMIT;
	$msg .= ":\n\n" if $stdout || $stderr;
	$msg .= "$stdout\n\n" if $stdout;
	$msg .= "$stderr\n\n" if $stdout;
	return (0, $msg);
    }

    # PLEASE NOTE: don't delete command files yet, job may have not
    # even started yet. Even for synchronous jobs you don't know how
    # the queue works, so use Queue::cleanup to remove these files.

    return (1, $jobid);
}


=head2 submitWithArgs

 Description: Submits a new job to the queue. This method takes the
              arguments for the command line and creates the Config
              object by using the tool name. If the arguments contain
              paths to files, it is the responsibility of the caller
              to ensure that the submitting host and executing host
              can access the files in a shared directory.

              Tool configuration MUST define following values:
              'binary' => binary path
              'queue'  => batch queue system name
              'jobid'  => unique job identifier

              If the queue configuration contains tags that are specific
              to the tool, they must be set in the tool configuration.
              For example, 'lsf42' uses 'resource' and 'jobname' tags,
              which are set in the tool configuration. When queue
              configuration is created, the tool configuration is used
              to override all key-value pairs in queue configuration.
              This is why you should be familiar with the keys in the
              queue configuration to avoid unwanted conflicts.

 Arguments:   $tname    Tool name.

              $args     Reference to an array containing command
                        line arguments (optional).

              $sync     0: asynchronous submission (default).
                        1: synchronous submission.

              $std      Reference to an array or hash. When an array is
                        passed, it defines paths to the redirection files
                        for stdin, stdout, stderr and exit code in this
                        order. When a hash is passed, the files are
                        defined in a following way:

                        stdin    => '/path/to/my.stdin.file'
                        stdout   => '/path/to/my.stdout.file'
                        stderr   => '/path/to/my.stderr.file'
                        exitcode => '/path/to/my.exitcode.file'

                        Any one of the redirection files can be undefined,
                        in which case the redirection is not done. For example,
                        if the application does not write to stdout, the caller
                        must use undef value (or empty string) in the array or
                        not define the 'stdout' key in the hash. Caller is
                        responsible for using unique file names, any existing
                        files with same names will be overwritten.

              $env      Reference to an array or hash. When an array
                        is passed, it defines which environment
                        variables are set in the tool command template
                        using the current environment values (in the
                        submitting host). When a hash is passed, the
                        environment variable values will be set from
                        the hash.

              $jobid    Job identifier (optional). Submit will be done using
                        this job ID. This is useful in cases when the caller
                        wants to do a synhronous submission and check/kill
                        the job before it is finished. See also method
                        createJobID.

 Returns:     1, job ID on success
              0, msg    on failure

=cut

sub submitWithArgs($;$$$$$) {
    my ($self, $tname, $args, $sync, $std, $env, $jobid) = @_;
    return (0, "tool name not defined") unless $tname;

    if ($args && !(ref($args) eq 'ARRAY')) {
        return (0, "command line arguments must be passed using an array reference");
    }

    if ($env && !(ref($env) eq 'ARRAY' || ref($env) eq 'HASH')) {
        return (0, "environment variables must be passed using an array or hash reference");
    }

    if ($std && !(ref($std) eq 'ARRAY' || ref($std) eq 'HASH')) {
        return (0, "redirection files must be passed using an array or hash reference");
    }

    my ($res, $msg, $tool, $binary);

    ($res, $tool) = $self->_createConfig($tname, $jobid);
    return (0, $tool) unless $res;

    $binary = $tool->get('binary');
    return (0, "$tname binary not defined") unless $binary;

    my $qname = $tool->get('queue');
    return (0, "$tname queue not configured") unless $qname;

    $jobid = $tool->get('jobid') unless $jobid;
    return (0, "$tname jobid not configured") unless $jobid;

    # We need to parse the queue configuration at this point to resolve
    # the directory where command files will be written. Use tool config
    # values as defaults to set any tool dependent values.
    my $queue = new Dispatcher::Config();
    ($res, $msg) = $queue->parse($qname, 1, $tool->getValues());
    return (0, "failed to parse $qname configuration: $msg") unless $res;

    $Debug = $queue->get('debug');  # set debug flag

    my $dir = $queue->get('workdir');
    $dir = $queue->get('default.workdir') unless $dir && -d $dir;
    return (0, "$qname default.workdir not configured") unless $dir;
    return (0, "$qname default.workdir $dir does not exist") unless -d $dir;

    # create unique command file prefix, use job ID if defined
    my $unique = catfile($dir, $jobid);
    
    # get regular expression matching queue specific jobid
    my $jobidRE = $queue->get('jobid.regexp');
    return (0, "$qname jobid.regexp not configured") unless $jobidRE;

    # get template for tool command file (optional)
    my $ttemplate = $tool->get($sync ? 'template.syncsub' : 'template.asyncsub');
    $ttemplate = $tool->get('template') unless $ttemplate;
    $ttemplate = $DefaultTemplate unless $ttemplate;
    ($res, $msg) = $self->createToolCmdFile($unique . EXT_TOOLCMD,
					    $ttemplate, $binary,
					    $args, $std, $env);
    return (0, "failed to create $tname command file: $msg") unless $res;

    # set the tool command file to tool configuration
    $tool->set('toolcmd', $unique . EXT_TOOLCMD);

    # reload queue configuration to set the toolcmd to submission command
    ($res, $msg) = $queue->parse($qname, 1, $tool->getValues());
    return (0, "failed to reload $qname configuration: $msg") unless $res;

    # get submission command
    my $subcmd = $self->getToolSpecificValue($queue,
					     $sync ? 'syncsub' : 'asyncsub',
					     $tname,
					     $tool->get('toolgroup'));
    return (0, "$qname " . ($sync ? 'syncsub' : 'asyncsub') .
	    " command not configured") unless $subcmd;

    # get template for submission command file
    my $stemplate = $self->getToolSpecificValue($queue,
						$sync ? 'syncsub.template' : 'asyncsub.template',
						$tname, $tool->get('toolgroup'));
    $stemplate = $queue->get('default.template') unless $stemplate;
    $stemplate = $DefaultTemplate unless $stemplate;

    # create submission command file
    $queue->set('cmd', $subcmd);
    ($res, $msg) = $self->createCmdFile($unique . EXT_SUBMIT, $stemplate,
					$queue->getValues());
    return (0, "failed to create $qname submission file: $msg") unless $res;

    # execute the submission command
    my ($stdout, $stderr);
    ($res, $msg) = $self->exec($unique . EXT_SUBMIT, \$stdout, \$stderr);
    if (!$res) {
	# PLEASE NOTE: bsub -K option writes <<Job is finished>> to stderr
        # so don't interpret length(stderr) > 0 as an error!
	my $error = "failed to execute $qname submit: $msg";
	$error .= "\n$stderr" if length($stderr) > 0;
	return (0, $error);
    }

    # write queue specific jobid from stdout or stderr output to cache and file
    if (($stdout && $stdout =~ /$jobidRE/) || ($stderr && $stderr =~ /$jobidRE/)) {
      my $qjobid  = $1;
      $QueueJobIDs->{"$jobid"} = $qjobid;  # cache to memory
      my $jobfile = $unique . EXT_JOBID;
      unless (open(JOBFILE, "> $jobfile")) {
	return (0, "failed to write $qname jobid to file $jobfile: $!");
      }
      print JOBFILE $qjobid;
      close JOBFILE;
    }
    else {
	$msg  = "failed to determine $qname jobid from submission output ";
	$msg .= $unique . EXT_SUBMIT;
	$msg .= ":\n\n" if $stdout || $stderr;
	$msg .= "$stdout\n\n" if $stdout;
	$msg .= "$stderr\n\n" if $stdout;
	return (0, $msg);
    }

    # PLEASE NOTE: don't delete command files yet, job may have not
    # even started yet. Even for synchronous jobs you don't know how
    # the queue works, so use Queue::cleanup to remove these files.

    return (1, $jobid);
}


=head2 kill

 Description: Terminates a job.

 Arguments:   $tool     Tool name or Dispatcher::Config object reference.
                        When tool name is given, this method creates
                        Dispatcher::Config object for the tool and parses
                        the tool configuration.

                        Tool configuration MUST define following values:
                        'queue' => batch queue system name

              $jobid    Job ID returned by submit.

 Returns:     1, ''  on success
              0, msg on failure

=cut

sub kill($$) {
    my ($self, $tool, $jobid) = @_;
    return (0, "cannot kill the job without tool name") unless $tool;
    return (0, "cannot kill the job without job ID")    unless $jobid;

    my ($res, $msg, $qjobid, $cmdline);

    ($res, $tool) = $self->_createConfig($tool, $jobid);
    return (0, $tool) unless $res;

    my $tname = $tool->getName();
    my $qname = $tool->get('queue');
    return (0, "$tname queue not configured") unless $qname;

    # parse queue configuration, pass tool configuration values as defaults
    my $queue = new Dispatcher::Config();
    ($res, $msg) = $queue->parse($qname, 1, $tool->getValues());
    return (0, "failed to parse $qname configuration: $msg") unless $res;

    $Debug = $queue->get('debug');  # set debug flag

    my $dir = $queue->get('workdir');
    $dir = $queue->get('default.workdir') unless $dir && -d $dir;
    return (0, "$qname default.workdir not configured") unless $dir;
    return (0, "$qname default.workdir $dir does not exist") unless -d $dir;

    ($res, $qjobid) = $self->getQueueJobID($jobid, $qname, $queue, $dir);
    return (0, $qjobid) unless $res;

    # get kill command
    my $cmd = $queue->get('kill');
    return (0, "$qname kill command not configured") unless $cmd;

    # set queue jobid to the command line
    $cmd .= " $qjobid";

    # get template for command file
    my $template = $self->getToolSpecificValue($queue, 'kill.template',
					       $tname, $tool->get('toolgroup'));
    $template = $queue->get('default.template') unless $template;
    $template = $DefaultTemplate unless $template;

    # create command file
    my $cmdfile = catfile($dir, $jobid . EXT_KILL);
    $queue->set('cmd', $cmd);
    ($res, $msg) = $self->createCmdFile($cmdfile, $template, $queue->getValues());
    return (0, "failed to create $qname kill file: $msg") unless $res;

    # execute the command
    my ($stdout, $stderr);
    ($res, $msg) = $self->exec($cmdfile, \$stdout, \$stderr);
    if (!$res) {
	my $error = "failed to execute $qname kill: $msg";
	$error .= "\n$stderr" if length($stderr) > 0;
	return (0, $error);
    }

    unlink($cmdfile);
 
    return (1, '');
}


=head2 check

 Description: Returns job status.

 Arguments:   $tool     Tool name or Dispatcher::Config object reference.
                        When tool name is given, this method creates
                        Dispatcher::Config object for the tool and parses
                        the tool configuration.
                        Tool configuration MUST define following values:
                        'queue' => batch queue system name

              $jobid    Job ID returned by submit.

 Returns:     JOB_NOT_FOUND, ''  when job is not found
              JOB_PENDING,   ''  when job is pending
              JOB_RUNNING,   ''  when job is running
              JOB_DONE,      ''  when job finished ok
              JOB_FAILED,    ''  when job failed
              0,               msg on failure

=cut

sub check($$) {
    my ($self, $tool, $jobid) = @_;
    return (0, "cannot check job status without tool name") unless $tool;
    return (0, "cannot check job status without job ID")    unless $jobid;

    my ($res, $msg, $qjobid, $cmdline);

    ($res, $tool) = $self->_createConfig($tool, $jobid);
    return (0, $tool) unless $res;

    my $tname = $tool->getName();
    my $qname = $tool->get('queue');
    return (0, "$tool queue not configured") unless $qname;

    # parse queue configuration, pass tool configuration values as defaults
    my $queue = new Dispatcher::Config();
    ($res, $msg) = $queue->parse($qname, 1, $tool->getValues());
    return (0, "failed to parse $qname configuration: $msg") unless $res;

    $Debug = $queue->get('debug');  # set debug flag

    my $dir = $queue->get('workdir');
    $dir = $queue->get('default.workdir') unless $dir && -d $dir;
    return (0, "$qname default.workdir not configured") unless $dir;
    return (0, "$qname default.workdir $dir does not exist") unless -d $dir;

    ($res, $qjobid) = $self->getQueueJobID($jobid, $qname, $queue, $dir);
    return (0, $qjobid) unless $res;

    # get status command
    my $cmd = $queue->get('status');
    return (0, "$qname status command not configured") unless $cmd;

    # set queue jobid to the command line
    $cmd .= " $qjobid";

    # get template for command file
    my $template = $self->getToolSpecificValue($queue, 'status.template',
					       $tname,
					       $tool->get('toolgroup'));
    $template = $queue->get('default.template') unless $template;
    $template = $DefaultTemplate unless $template;

    # create command file
    my $cmdfile = catfile($dir, $jobid . EXT_CHECK);
    $queue->set('cmd', $cmd);
    ($res, $msg) = $self->createCmdFile($cmdfile, $template, $queue->getValues());
    return (0, "failed to create $qname check file: $msg") unless $res;

    # execute the command
    my ($stdout, $stderr);
    ($res, $msg) = $self->exec($cmdfile, \$stdout, \$stderr);
    if (!$stdout && !$stderr && !$res) {
	my $error = "failed to execute $qname check: $msg";
	$error .= "\n$stderr" if length($stderr) > 0;
	return (0, $error);
    }

    unlink($cmdfile);

    # get regular expressions matching job states
    my $pendRE = $queue->get('state.pend.regexp');
    my $runRE  = $queue->get('state.run.regexp');
    my $doneRE = $queue->get('state.done.regexp');
    my $failRE = $queue->get('state.fail.regexp');
    my $notfoundRE = $queue->get('state.notfound.regexp');

    # try to find job state from stdout or stderr output
    if ($pendRE && (($stdout && $stdout =~ /$pendRE/) ||
		    ($stderr && $stderr =~ /$pendRE/)))
    {
        return (JOB_PENDING, $stdout);
    }
    elsif ($runRE && (($stdout && $stdout =~ /$runRE/) ||
		      ($stderr && $stderr =~ /$runRE/))) {
        return (JOB_RUNNING, $stdout);
    }
    elsif ($doneRE && (($stdout && $stdout =~ /$doneRE/) ||
 	 	       ($stderr && $stderr =~ /$doneRE/))) {
        return (JOB_DONE, $stdout);
    }
    elsif ($failRE && (($stdout && $stdout =~ /$failRE/) ||
 	 	       ($stderr && $stderr =~ /$failRE/))) {
        return (JOB_FAILED, $stdout);
    }
    elsif ($notfoundRE && (($stdout && $stdout =~ /$notfoundRE/) ||
			   ($stderr && $stderr =~ /$notfoundRE/))) {
        return (JOB_NOT_FOUND, $stdout);
    }
    elsif (!$res) {
	return (0, $msg);
    }

    $msg = "failed to check $qname job $jobid:\n";
    $msg .= $stdout if $stdout;
    $msg .= $stderr if $stderr;
    $msg .= "no error message available\n" unless $stdout || $stderr;

    return (0, $msg);
}


=head2 createJobName

 Description: Creates job name.

              DEPRECATED Use createJobID method instead.

              Job name is same as job ID. Dispatcher uses solely job ID
              to identify the job. Job name is an LSF concept.

 Arguments:   $tname   Tool name.

 Returns:     1, job name on success
              0, msg      on failure

=cut

sub createJobName($) {
    my ($self, $tname) = @_;
    return $self->createJobID($tname);
}


=head2 createJobID

 Description: Creates job identifier.

 Arguments:   $tname    Tool name.

 Returns:     1, job ID on success
              0, msg    on failure

=cut

sub createJobID($) {
    my ($self, $tname) = @_;
    return (0, "cannot create job ID without tool name") unless $tname;

    my ($res, $tool) = $self->_createConfig($tname);
    return (0, $tool) unless $res;

    my $jobid = $tool->get('jobid');
    return (0, "$tname configuration does not define jobid") unless $jobid;
    return (1, $jobid);
}


=head2 cleanup

 Description: Removes any temporary files created during job submission.
              You should call this method AFTER the job has finished to
              remove temporary files created by Queue.

 Arguments:   $tname    Tool name.
              $jobid    Job ID.

 Returns:     1, number of deleted files on success
              0, msg on failure

=cut

sub cleanup($$) {
    my ($self, $tname, $jobid) = @_;
    return (0, "cannot cleanup without tool name") unless $tname;
    return (0, "cannot cleanup without job ID")    unless $jobid;

    my ($res, $tool) = $self->_createConfig($tname, $jobid);
    return (0, $tool) unless $res;	

    my $qname = $tool->get('queue');
    return (0, "$tname queue not configured") unless $qname;

    my ($dir, @dirs, $file, $msg);

    my $queue = new Dispatcher::Config();
    ($res, $msg) = $queue->parse($qname, 1, $tool->getValues());
    return (0, "failed to parse $qname configuration: $msg") unless $res;

    $Debug = $queue->get('debug');  # set debug flag

    $dir = $queue->get('default.workdir');
    push @dirs, $dir if $dir && -d $dir;

    $dir = $queue->get('workdir');
    push @dirs, $dir if $dir && -d $dir;

    $dir = $queue->get('sworkdir');
    push @dirs, $dir if $dir && -d $dir;

    $dir = $queue->get('aworkdir');
    push @dirs, $dir if $dir && -d $dir;

    my $count = 0;

    foreach $dir (@dirs) {
	$file = catfile($dir, $jobid . EXT_SUBMIT);
	$count += unlink($file) if -f $file;

	$file = catfile($dir, $jobid . EXT_TOOLCMD);
	$count += unlink($file) if -f $file;

	$file = catfile($dir, $jobid . EXT_JOBID);
	$count += unlink($file) if -f $file;
    }

    delete $QueueJobIDs->{"$jobid"};

    return (1, $count);
}


=head1 PRIVATE METHODS


=head2 getToolSpecificValue

 Description: Returns value from configuration that depends on tool or
              tool group.

 Arguments:   $queue     Dispatcher::Config object for queue.
              $key       Configuration key.
              $tname     Tool name.
              $tgroup    Tool group.

 Returns:     value as string

=cut

sub getToolSpecificValue($$$;$) {
    my ($self, $queue, $key, $tname, $tgroup) = @_;
    return '' unless $queue && $key && $tname;

    # first one encountered is used:
    #  tool.<tool>.<key>
    #  toolgroup.<toolgroup>.<key>
    #  <key>
    if ($queue->get("tool.$tname.$key")) {
        return $queue->get("tool.$tname.$key");
    }
    elsif ($tgroup && $queue->get("toolgroup.$tgroup.$key")) {
        return $queue->get("toolgroup.$tgroup.$key");
    }

    return $queue->get($key) || '';
}


=head2 getCmdSpecificValue

 Description: Returns value from configuration that depends on command.

 Arguments:   $queue    Dispatcher::Config object for queue.
              $key      Configuration key.
              $cmd      Command.

 Returns:     value as string

=cut

sub getCmdSpecificValue($$$) {
    my ($self, $queue, $key, $cmd) = @_;
    return '' unless $queue && $key;

    # first one encountered is used:
    #  <cmd>.<key>
    #  <key>
    if ($cmd && $queue->get("$cmd.$key")) {
        return $queue->get("$cmd.$key");
    }

    return $queue->get($key) || '';
}


=head2 getQueueJobID

 Description: Returns queue specific jobid matching Dispatcher jobid.

 Arguments:   $jobid    Dispatcher jobid.
              $qname    Queue name.
              $queue    Dispatcher::Config object for queue.
              $dir      Directory where queue jobid is stored.

 Returns:     $qjobid

=cut

sub getQueueJobID($$$$) {
    my ($self, $jobid, $qname, $queue, $dir) = @_;
    return (0, 'no jobid defined') unless $jobid;

    if (exists $QueueJobIDs->{"$jobid"}) {
	# use value stored in memory cache
	return (1, $QueueJobIDs->{"$jobid"});
    }

    return (0, 'no queue name defined')          unless $qname;
    return (0, 'no queue configuration defined') unless $queue;
    return (0, 'no jobid directory defined')     unless $dir;
    return (0, 'jobid directory does not exist') unless -d $dir;

    my $jobfile = catfile($dir . $jobid . EXT_JOBID);
    return (0, "$qname jobid file $jobfile does not exist") unless -e $jobfile;

    unless (open(JOBFILE, "< $jobfile")) {
	return (0, "failed to read $qname jobid from file $jobfile: $!");
    }

    local $INPUT_RECORD_SEPARATOR;
    my $qjobid = <JOBFILE>;
    close JOBFILE;
    $INPUT_RECORD_SEPARATOR = "\n";

    unless ($qjobid) {
	return (0, "could not determine $qname jobid from file $jobfile");
    }

    $QueueJobIDs->{"$jobid"} = $qjobid;  # cache to memory
    return (1, $qjobid);
}


=head2 createCmdFile

 Description: Creates a command file.

 Arguments:   $cmdfile     Path to command file to be created.
              $template    Path to command file template.
              $tags        Reference to a hash containing tags as
                           key-value pairs. For example, key 'cmd'
                           will match tag [%cmd] in the template
                           and be replaced with 'cmd' value in hash.

 Returns:     1, ''  on success
              0, msg on error

=cut

sub createCmdFile($$;$) {
    my($self, $cmdfile, $template, $tags) = @_;
    return (0, "command file not defined") unless $cmdfile;
    return (0, "command file template not defined") unless $template;
    return (0, "command file template $template does not exist") unless -r $template;

    if ($tags && !(ref($tags) eq 'HASH')) {
        return (0, "command file tags must be passed in a hash reference");
    }

    unless (open(TEMPLATE, "< $template")) {
        close TEMPLATE;
        return (0, "could not read command file template $template: $!");
    }

    unless (open(CMDFILE, "> $cmdfile")) {
        close TEMPLATE;
        close CMDFILE;
        return (0, "could not write command file $cmdfile: $!");
    }

    my ($key, $value);

    while(<TEMPLATE>) {
        if ($tags) {
            foreach $key (keys %$tags) {
                if ($_ =~ /\[\%$key\]/) {
                    $value = $tags->{$key};
                    s/\[\%$key\]/$value/g;
                }
            }
        }

        print CMDFILE $_;
    }

    close TEMPLATE;
    close CMDFILE;

    chmod 0755, $cmdfile;

    return (1, '');
}


=head2 createToolCmdFile

 Description: Creates a tool command file.

 Arguments:   $cmdfile     Path to command file to be created.
              $template    Path to command file template.
              $binary      Path to tool binary.
              $args        Reference to an array containing command line
                           arguments (optional).
              $std         Reference to an array or hash containing
                           redirection files (optional).
              $env         Reference to an array or hash containing
                           environment variables to be set (optional).

 Returns:     1, ''  on success
              0, msg on error

=cut

sub createToolCmdFile($$$;$$$) {
    my($self, $cmdfile, $template, $binary, $args, $std, $env) = @_;
    return (0, "command file not defined") unless $cmdfile;
    return (0, "command file template not defined") unless $template;
    return (0, "command file template $template does not exist") unless -r $template;
    return (0, "cannot create command file without binary") unless $binary;

    if ($args && !(ref($args) eq 'ARRAY')) {
        return (0, "command line arguments must be passed using an array reference");
    }

    if ($env && !(ref($env) eq 'ARRAY' || ref($env) eq 'HASH')) {
        return (0, "environment variables must be passed using an array or hash reference");
    }

    my ($stdin, $stdout, $stderr, $exitcode);

    if ($std && ref($std) eq 'ARRAY') {
	($stdin, $stdout, $stderr, $exitcode) = @$std;
    }
    elsif ($std && ref($std) eq 'HASH') {
	$stdin  = $std->{stdin}  || '';
	$stdout = $std->{stdout} || '';
	$stderr = $std->{stderr} || '';
	$exitcode = $std->{exitcode} || '';
    }
    else {
        return (0, "redirection files must be passed using an array or hash reference");
    }

    # suppress warnings about unitialized values in substitution
    $stdin = ''    unless $stdin;
    $stdout = ''   unless $stdout;
    $stderr = ''   unless $stderr;
    $exitcode = '' unless $exitcode;

    unless (open(TEMPLATE, "< $template")) {
        close TEMPLATE;
        return (0, "could not read command file template $template: $!");
    }

    unless (open(CMDFILE, "> $cmdfile")) {
        close TEMPLATE;
        close CMDFILE;
        return (0, "could not write command file $cmdfile: $!");
    }

    my $shebang = SHEBANG_NONE;

    while(<TEMPLATE>) {
        if ($_ =~ /^\#!.*perl/) {
	    $shebang = SHEBANG_PERL;  # perl script
        }
        elsif ($_ =~ /^\#!.*csh/) {
	    $shebang = SHEBANG_CSH;   # csh or tcsh script
        }
        elsif ($_ =~ /^\#!.*sh/) {
	    $shebang = SHEBANG_SH;    # sh or bash script
        }
        elsif ($_ =~ /\[%binary\]/) {
	    $_ =~ s/\[%binary\]/$binary/g;
        }
        elsif ($_ =~ /\[%argsline\]/) {
	    my $argsline = '';

	    if ($args && @$args > 0) {
		my $delim = '';

		foreach my $arg (@$args) {
		    $argsline .= "$delim$arg";
		    $delim = " ";
		}
	    }

	    $_ =~ s/\[%argsline\]/$argsline/;
        }
        elsif ($_ =~ /\[%argslist\]/) {
	    my $argslist = '';

	    if ($args && @$args > 0) {
		my $delim = '';

		foreach my $arg (@$args) {
		    $argslist .= "$delim$arg";
		    $delim = "\n";
		}
	    }

	    $_ =~ s/\[%argslist\]/$argslist/;
        }
        elsif ($_ =~ /\[%env\]/) {
	    # expand environment variables
	    my $envlist = '';

	    if ($shebang == SHEBANG_NONE) {
		close TEMPLATE;
		close CMDFILE;
		return (0, "could not determine how to set environment variables " .
			"to a command file using template $template");
	    }

	    my $evalues = { };
	    my $ekey;

	    if ($env && ref($env) eq 'ARRAY' && @$env > 0) {
		$evalues = { };

		foreach $ekey (@$env) {
		    $ekey =~ s/^\s+//;    # no leading white
		    $ekey =~ s/\s+$//;    # no trailing white
		    next unless length($ekey) > 0;
		    $evalues->{"$ekey"} = $ENV{"$ekey"};
		}
	    }
	    elsif ($env && ref($env) eq 'HASH') {
		$evalues = $env;
	    }

	    my $evalue;

	    foreach $ekey (sort keys %$evalues) {
		$evalue = $evalues->{$ekey} || '';

		if ($shebang == SHEBANG_PERL) {
		    $envlist .= "\$ENV\{\"$ekey\"\} = '$evalue';\n";
		}
		elsif ($shebang == SHEBANG_CSH) {
		    $envlist .= "setenv $ekey $evalue\n";
		}
		elsif ($shebang == SHEBANG_SH) {
		    $envlist .= "$ekey=$evalue\n";
		    $envlist .= "export $ekey\n";
		}
	    }

	    $_ =~ s/\[%env\]/$envlist/;
        }
        elsif ($_ =~ /\[%stdin\]/) {
	    $_ =~ s/\[%stdin\]/$stdin/g;
        }
        elsif ($_ =~ /\[%stdout\]/) {
	    $_ =~ s/\[%stdout\]/$stdout/g;
        }
        elsif ($_ =~ /\[%stderr\]/) {
	    $_ =~ s/\[%stderr\]/$stderr/g;
        }
        elsif ($_ =~ /\[%exitcode\]/) {
	    $_ =~ s/\[%exitcode\]/$exitcode/g;
        }

	print CMDFILE $_;
    }

    close TEMPLATE;
    close CMDFILE;

    chmod 0755, $cmdfile;

    return (1, '');
}


=head2 getUniqueID

 Description: Generates unique identifier.
 Arguments:   -
 Returns:     id string

=cut

sub getUniqueID() {
    my $self = shift;
    my ($sec, $min, $hour, $mday, $mon, $year, $wday) = localtime(time);
    $mon++;         # localtime starts months from 0
    $year += 1900;  # localtime starts years from 100
    my $time = sprintf("%d%.2d%.2d%.2d%.2d%.2d",
		       $year, $mon, $mday, $hour, $min, $sec);
    my $rand = sprintf("%.6d", int(rand(1000000)));
    return hostname() . "-$time-$rand";
}


=head2 copyInputFiles

 Description: Copies local input files to configured (shared) directory.
 Arguments:   $tool    Tool name or Dispatcher::Config object reference.
                       When tool name is given, this method creates
                       Dispatcher::Config object for the tool and parses
                       the tool configuration.

                       When tool accepts only one input file, use 'toolinput'
                       to define the path:
                       'toolinput' => path to input file

                       When tool accepts multiple input files, use
                       'toolinput.N' to define the paths:
                       'toolinput.1' => path to first input file
                       'toolinput.2' => path to second input file ...

              $input   Reference to an array. Each array element can be
                       a string (path to the file), a FileHandle reference or
                       a FileHandle glob. Please read "perldoc File::Copy" for
                       additional information.

 Returns:     1, ''  on success
              0, msg on failure

=cut

sub copyInputFiles($$) {
    my ($self, $tool, $input) = @_;
    return (1, '') unless $input;

    unless (ref($input) eq 'ARRAY') {
        return (0, "failed to copy input files, not an array reference");
    }

    return (1, '') unless @$input > 0;

    unless (ref($tool) eq "Dispatcher::Config") {
        my $config = new Dispatcher::Config();
        my ($res, $msg) = $config->parse($tool, 1);
        return (0, "failed to parse $tool configuration: $msg") unless $res;
        $tool = $config;
    }

    my $tname = $tool->getName();
    my ($src, $dst, $key);
    my $i = 1;

    foreach $src (@$input) {
        $key = @$input == 1 ? 'toolinput' : 'toolinput.' . $i;
	$dst = $tool->get($key);
        unless ($dst) {
            return (0, "wrong number of $tname input files, $key not configured");
        }

        my ($file, $path, $ext) = fileparse($dst);
        my ($res, $msg) = $self->_checkDirectory($path, $tool->get('dirmode'));
	return (0, "failed to copy $src to $dst: $msg") unless $res;

        unless (copy($src, $dst)) {
            return (0, "failed to copy $src to $dst: $!");
        }

        $i++;
    }

    return (1, '');
}


=head2 exec

 Description: Executes a command file and returns stdout and stderr.

              FIXME: IO::Select is not used because can_read() does not
              always return the file handles.

 Arguments:   $cmdfile    Command file to execute.
	      $outref     Scalar reference for output (stdout).
	      $errref     Scalar reference for errors (stderr).

 Returns:     1, ''  on success
	      0, msg on error

=cut

sub exec($$$) {
    my ($self, $cmdfile, $outref, $errref) = @_;
    return (0, "no file command defined") unless $cmdfile;
    return (0, "command file does not exist") unless -r $cmdfile;
 
    my $outfile = "$cmdfile.stdout";
    my $errfile = "$cmdfile.stderr";
    my $cmd = "$cmdfile 1> $outfile 2> $errfile";

    my $rc = system($cmd);

    #
    # vsi 2004-04-05: this seems to miss some non-zero exit statuses
    #
    #my $rc = 0xffff & system($cmd);
    #if ($rc == 0xff00) {
    #    $rc = $!;
    #}
    #elsif (($rc & 0xff) == 0) {
    #    $rc >>= 8;
    #}

    #
    # vsi 2004-05-05: check that the files exist. When an interactive
    # job runs for too long, the output files may have been deleted.
    #
    if (-e $outfile) {
	unless (open(CMDOUT, $outfile)) {
	    close CMDOUT;
	    return (0, "could not read $outfile");
	}

	local $INPUT_RECORD_SEPARATOR;  # undefine newline limit
	${$outref} = <CMDOUT>;
        $INPUT_RECORD_SEPARATOR = "\n";
        close(CMDOUT);
        unlink($outfile);
    }

    if (-e $errfile) {
        unless (open(CMDERR, $errfile)) {
	    close CMDERR;
	    return (0, "could not read $errfile");
	}

        local $INPUT_RECORD_SEPARATOR;  # undefine newline limit
        ${$errref} = <CMDERR>;
        $INPUT_RECORD_SEPARATOR = "\n";
        close(CMDERR);
        unlink($errfile);
    }

    return (0, "$cmdfile exited with status $rc") if $rc != 0;
    return (1, '');
}


=head2 _checkDirectory

 Description: Checks if given directory exists and tries to create it.
 Arguments:   $dir     Directory.
              $mode    Numeric mode (see DEFAULT_DIR_MODE).
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub _checkDirectory($;$) {
    my ($self, $dir, $mode) = @_;
    return (0, 'no directory given') unless $dir;

    $mode = DEFAULT_DIR_MODE unless $mode;

    unless (-d $dir) {
        unless (mkpath($dir, 0, oct(sprintf("%.4d", $mode)))) {
            return (0, "could not create directory $dir: $!");
        }
    }

    return (1, '');
}


=head2 _createConfig

 Description: Creates Dispatcher::Config object.
 Arguments:   $tool     Tool name or
                        Tool object or
                        Dispatcher::Config object
              $jobid    Job identifier (optional). Used to set defaults.
 Returns:     1, reference to Dispatcher::Config object on success
              0, msg on failure

=cut

sub _createConfig($;$) {
    my ($self, $tool, $jobid) = @_;
    my ($res, $msg, $type);

    $type = ref($tool);

    if ($type eq "Dispatcher::Config") {
        # we got it
    }
    elsif ($type =~ /^Dispatcher::Tool/) {
        my $config = $tool->getConfig();
        unless ($config && ref($config) eq "Dispatcher::Config") {
	    return (0, "tool $type does not contain Dispatcher::Config object");
        }
        $tool = $config;   
    }
    else {
	my $defaults = { };
	$defaults->{jobid} = $jobid if $jobid;

	if ($jobid && $jobid =~ /[^\d]([\d]{4})(\d\d)(\d\d)[^\d](\d\d)(\d\d)(\d\d)/) {
	    # set date and time because they may be used in configuration (paths and URLs)
	    $defaults->{YYYY} = $1;
	    $defaults->{MM} = sprintf("%.2d", $2);
	    $defaults->{DD} = sprintf("%.2d", $3);
	    $defaults->{hh} = sprintf("%.2d", $4);
	    $defaults->{mm} = sprintf("%.2d", $5);
	    $defaults->{ss} = sprintf("%.2d", $6);
	}

        my $config = new Dispatcher::Config();
        ($res, $msg) = $config->parse($tool, 1, $defaults);
        return (0, "failed to parse $type $tool configuration: $msg") unless $res;
        $tool = $config;
    }

    return (1, $tool);
}


=head2 _init

 Description: Initializes the object.
 Arguments:   -
 Returns:     -

=cut

sub _init () {
    srand($PID);

    my $config = catfile($Root, "conf", "logger.conf");
    $config = '' unless -r $config;
    #$Log = new Logger($config);
}


1;
__END__
