=head1 NAME

BlastProDom - BlastProDom utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::BlastProDom;

=head1 DESCRIPTION

=head1 VERSIONS

$Id: BlastProDom.pm,v 1.2 2010/01/20 13:52:06 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

interhelp@ebi.ac.uk
Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut

package Dispatcher::Tool::BlastProDom;

BEGIN {
  unless($ENV{IPRSCAN_HOME}){
    print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
    exit();
  }
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool;
@ISA = qw( Dispatcher::Tool );


use Data::Dumper;
use English;
use File::Spec::Functions;
use POSIX qw(strftime);
use Sys::Hostname;
use strict;


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        BlastProDom tool name (optional). If given, sets
                           the Dispatcher::Config object automatically inside
                           the object. If not given, the Dispatcher::Config
                           must be set later with setConfig.
              $defaults    Default values for configuration, hash reference
                           (optional).
 Returns:     $self object

=cut

sub new(;$$) {
    my ($class, $name, $defaults) = @_;

    my $self = { };
    bless $self, $class;

    if ($name) {
        $self->setToolName($name);
        my ($res, $msg) = $self->parseConfig(1, $defaults);
        die "failed to parse $name configuration: $msg\n" unless $res;
    }

    return $self;
}


=head2 getCmdLine

 Description: Returns BlastProDom command line.
 Arguments:   -
 Returns:     1, cmd on success
              0, msg on failure

=cut

sub getCmdLine() {
    my $self = shift;
    return $self->getConfigValue('cmdline');
}


=head2 parseResult

 Description: Reads a blastprodom result file and creates a hash table:

              'nhits'     => '5'
              '1'         => { 'seqid'  => 'USERPROTEIN:tmp_job3'
                  	       'start'  => '2', 
                  	       'end'    => '7',
                  	       'methid' => 'pd_PD383194',
                  	       'sp'     => 'sp_Q9X2E3_THEMA_Q9X2E3',
                  	       'dstart' => '129'
                  	       'dend'   => '280'
                  	       'score'  => '81',
                  	       'evalue' => '3e-09',
                  	       'desc'   => '(22)  PROTEOME COMPLETE PROTEASE HFLC HYDROLASE 3.4.-.- SERINE TRANSMEMBRANE FOR FTSH',
                  	       'length' => '152'
                  	     },
              '2'         =  { ... }
              ...


 Arguments:   $href          Reference to a hash.
              $result        Path to a BlastProDom result file.
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub parseResult($$$) {
    my ($self, $href, $result, $inx) = @_;
    return (0, "no blastprodom input file defined") unless $result;

    $href = { } unless $href && ref($href) eq 'HASH';
    $inx  = { } unless $inx && ref($inx) eq 'HASH';

    unless (open(FILE, "< $result")) {
        return (0, "could not read blastprodom result file $result: $!");
    }

    my $hit = 1;
    my $fpos = 0;

    while (<FILE>) {
        chomp;

        #
        # Example result:
        # 
        # WAP_RAT      1     77 //  pd_PD019552;sp_WAP_RAT_P01174      1     77 // S=347    E=9e-41  //  (2)  WHEY PRECURSOR WAP PROTEASE INHIBITOR REPEAT ACIDIC MILK SIGNAL PHOSPHORYLATION           Length = 77 
        # WAP_RAT     78     78 //  pd_PD001224;sp_WAP_RABIT_P09412     43    126 // S=183    E=1e-21  //  (11)  SIGNAL PRECURSOR PORIN INHIBITOR PROTEASE SERINE C08G9.2 WHEY MILK WAP           Length = 84 
        # WAP_RAT     79    127 //  pd_PD026912;sp_WAP_RAT_P01174     79    127 // S=295    E=1e-34  //  (40)  INHIBITOR PROTEASE PRECURSOR SIGNAL REPEAT WHEY SERINE WAP SECRETORY DOMAIN           Length = 49 
        # WAP_RAT    128    128 //  pd_PD399701;sp_WAP_MOUSE_P01173     78    134 // S=163    E=3e-19  //  (1)  WHEY WAP PROTEASE INHIBITOR REPEAT ACIDIC MILK SIGNAL PRECURSOR           Length = 57 
        # WAP_RAT    129    137 //  pd_PD006869;sp_PSTA_DICDI_P11976    449    669 // S=113    E=2e-13  //  (6)  REPEAT SIGNAL PRECURSOR PRESTALK CAMBP64 PDI CALMODULIN-BINDING NUCLEOTIDE GLYCOPROTEIN INHIBITOR           Length = 221
        # 

        if ($_ =~ /([^\s]+)\s+(\d+)\s+(\d+)\s+\/\/\s+(pd[^;]+);([^\s]+)\s+(\d+)\s+(\d+)\s+\/\/\s+S=(\d+)\s+E=([^\s]+)\s+\/\/\s+(.*?)\s+Length\s+=\s+(\d+)/) {
            $href->{$hit} = { } unless $href->{$hit};
	    $inx->{$1} = $fpos unless(exists($inx->{$1}));

            $href->{$hit}->{seqid}  = $1;
            $href->{$hit}->{start}  = $2;
            $href->{$hit}->{end}    = $3;
            $href->{$hit}->{methid} = $4;
            $href->{$hit}->{sp}     = $5;
            $href->{$hit}->{dstart} = $6;
            $href->{$hit}->{dend}   = $7;
            $href->{$hit}->{score}  = $8;
            $href->{$hit}->{evalue} = $9;
            $href->{$hit}->{desc}   = $10;
            $href->{$hit}->{length} = $11;
        }
	if($_ =~ /^\/\//){
	  $fpos = tell(FILE);
	}
	$hit++;
    }

    $href->{nhits} = $hit - 1;

    close FILE;

    return (1, '');
}


=head2 getRawResult

 Description: Returns raw formatted result string.
 Arguments:   $href      Reference to a result hash.
              $seqs      Sequence hash (see InterProScan::checkSequences).
              $iprscan   Reference to Dispatcher::Tool::InterProScan object.
                         Required for IPR fields and GO terms.
              $ipr       1: add IPR fields
                         0: no (default)
              $go        1: add GO terms
                         0: no (default)
 Returns:     1, str on success
              0, msg on failure

=cut

sub getRawResult($$;$$$) {
    my ($self, $href, $seqs, $iprscan, $ipr, $go) = @_;
    return (0, "no result hash defined")    unless $href && ref($href) eq 'HASH';
    return (0, "no sequences hash defined") unless $seqs && ref($seqs) eq 'HASH';


    if (!$iprscan && ($ipr || $go)) {
	return (0, "no InterProScan object defined");
    }
    elsif ($iprscan && ref($iprscan) ne 'Dispatcher::Tool::InterProScan') {
	return (0, "InterProScan object has wrong class: " . ref($iprscan));
    }

    my $nhits = $href->{nhits};

    return (0, "number of hits not defined in result hash") unless defined $nhits;
    return (1, '') if $nhits == 0;

    my $appl = $self->getConfigValue('applname');
    my $date = POSIX::strftime("%d-%b-%Y", localtime);
    my ($hit, $raw, $seqid, $seqcrc, $seqlen, $pd, $sp, $res, $iprac, $iprname, $goterms);

    for (my $i = 1; $i <= $nhits; $i++) {
	$hit = $href->{$i};
	next unless $hit;  # should not happen

	$seqid = $hit->{seqid};
	next unless $seqid;  # should not happen

	$seqcrc = $seqs->{$seqid}->{crc64};
	$seqlen = $seqs->{$seqid}->{len};

	$pd = $hit->{methid};
	$pd =~ s/pd_// if $pd;
	$sp = $hit->{sp};
	$sp =~ s/sp_// if $sp;

	$raw .= sprintf("%s\t%s\t%s\t", $seqid, $seqcrc, $seqlen);
        $raw .= sprintf("%s\t%s\t%s\t", $appl, $pd, $sp);
	$raw .= sprintf("%d\t%d\t", $hit->{start}, $hit->{end});
	$raw .= sprintf("%s\tT\t", $hit->{evalue});
	$raw .= sprintf("%s", $date);

	if ($ipr) {
	    ($res, $iprac, $iprname, $goterms) = $iprscan->getInterProFields($pd, $go);
	    return (0, $iprac) unless $res;

	    $raw .= sprintf("\t$iprac\t$iprname") if $ipr;
	    $raw .= sprintf("\t$goterms") if $go;
        }

	$raw .= "\n";
    }
    return (1, $raw);
}


1;
__END__

