=head1 NAME

FingerPrintScan - FingerPrintScan utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::FingerPrintScan;

=head1 DESCRIPTION

=head1 VERSIONS

$Id: FingerPrintScan.pm,v 1.6 2010/01/20 13:52:16 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

interhelp@ebi.ac.uk
Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut

package Dispatcher::Tool::FingerPrintScan;

BEGIN {
  unless($ENV{IPRSCAN_HOME}){
    print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
    exit();
  }
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool;
@ISA = qw( Dispatcher::Tool );


use Data::Dumper;
use English;
use File::Spec::Functions;
use POSIX qw(strftime);
use Sys::Hostname;
use strict;


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        FingerPrintScan tool name (optional). If given, sets
                           the Dispatcher::Config object automatically inside
                           the object. If not given, the Dispatcher::Config
                           must be set later with setConfig.
              $defaults    Default values for configuration, hash reference
                           (optional).
 Returns:     $self object

=cut

sub new(;$$) {
    my ($class, $name, $defaults) = @_;

    my $self = { };
    bless $self, $class;

    if ($name) {
        $self->setToolName($name);
        my ($res, $msg) = $self->parseConfig(1, $defaults);
        die "failed to parse $name configuration: $msg\n" unless $res;
    }

    return $self;
}


=head2 getCmdLine

 Description: Returns FingerPrintScan command line.
 Arguments:   -
 Returns:     1, cmd on success
              0, msg on failure

=cut

sub getCmdLine() {
    my $self = shift;
    return $self->getConfigValue('cmdline');
}


=head2 parseResult

 Description: Reads a fingerprintscan result file and creates a hash table:

            'WAP_RAT'   => '4DISULPHCORE'   => {
                                                'evalue'    = '1e-06',
                                                'nmotifs'   = '4',
                                                'graphscan' = 'IIII',
                                                'printsac'  = 'PR00003',
                                                '1' => {
                                                        motifnum = '1',
                                                        maxmotif    = '4',
                                                        idscore = '46.67',
                                                        pvalue  = '9.40e-04',
                                                        start   = '76',
                                                        end = '85' },
                                                '2' => { ... },
                                                ...
                                                },
                        => 'HIRUDIN'    => {...},
            'ZN433_HUMAN' ...

 Arguments:   $href          Reference to a hash.
              $result        Path to a FingerPrintScan result file.
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub parseResult($$$) {
    my ($self, $href, $result, $inx) = @_;
    return (0, "no fingerprintscan input file defined") unless $result;

    $href = { } unless $href && ref($href) eq 'HASH';
    $inx  = { } unless $inx && ref($inx) eq 'HASH';

    unless (open(SIB, "< $ENV{IPRSCAN_HOME}/data/FingerPRINTShierarchy.db")) {
        return (0, "could not read fingerprintscan hierarchy file $ENV{IPRSCAN_HOME}/data/FingerPRINTShierarchy.db: $!");
    }
                                                                                                                                                             
    my %HOH;
    my @temp_array; # temp array for storing output lines
    my %label = (
            0 => "accession",
            1 => "evalue",
            2 => "motifs",
            3 => "hierarchy",
    );
    # read the hierarchy file into hash of hashes
                                                                                                                                                             
    while (<SIB>){
        chomp;
	s/#.*//; # remove any comments from hierarchy file (indicated by a hash mark)
        @temp_array=split(/\|/);
                                                                                                                                                             
        foreach (1..4) {
            $HOH {$temp_array[0]} {$label{$_-1} } = $temp_array[$_];
        }
    }
    close (SIB);

    unless (open(FILE, "< $result")) {
        return (0, "could not read fingerprintscan result file $result: $!");
    }

    my $fprints = {}; # ref hash to store e-values for fingerprints
    my $nmotifs=1; # count the number of motifs
    my $hitnum=1;
    my $seqid; # sequence  identifier

    while (<FILE>) {
        chomp;

        # 
        # Example result:
	#
        # Sn; UniProt/Swiss-Prot|P01174|WAP_RAT Whey acidic protein precursor
        # Si; Fasta sequence
        # 1TBS
        # 1TBH 4DISULPHCORE    1.025205e-06   4-disulphide core signature                                              PR00003
        # 1TBF
        # 2TBS
        # 2TBT FingerPrint     No.Motifs SumId    AveId    ProfScore  Ppvalue     Evalue      GraphScan
        # 2TBH 4DISULPHCORE    4  of  4  2e+02    49       1112       4.5e-13     1e-06       IIII
        # 2TBN HIRUDIN         2  of  4  56.41    28.21    397        0.00088     2.6e+03     .ii.
        # 2TBN HMRNUCRECPTR    2  of  9  58.33    29.17    330        0.0035      1.2e+04     ..iI.....
        # 2TBF
        # 3TBS
        # 3TBT MotifName       No.Mots   IdScore PfScore Pvalue    Sequence                                                Len  low  pos  high
        # 3TBH 4DISULPHCORE    1  of  4  46.67   239     9.40e-04  EVQKAGRCPW                                              10   7    76   1379
        # 3TBH 4DISULPHCORE    2  of  4  51.04   254     4.03e-03  PCSIDSDC                                                8    29   101  1400
        # 3TBH 4DISULPHCORE    3  of  4  55.42   385     4.37e-05  CSGTMKCCKN                                              10   36   108  1407
        # 3TBH 4DISULPHCORE    4  of  4  44.44   234     2.72e-03  GCIMSCMDP                                               9    46   118  1417
        # 3TBB
        # 3TBN HIRUDIN         2  of  4  29.74   233     1.25e-02  CPKDNPCSIDSDC                                           13   16   96   36
        # 3TBN HIRUDIN         3  of  4  26.67   164     7.04e-02  KCCKNGCIMSCMDPEP                                        16   31   113  51
        # 3TBB
        # 3TBN HMRNUCRECPTR    3  of  9  25.00   160     6.88e-02  CSGTMKCCKNGCI                                           13   71   108  72
        # 3TBN HMRNUCRECPTR    4  of  9  33.33   170     5.02e-02  PKSPTVISFQ##                                            12   92   128  95

        if (/Sn; ([^\s]+)/) { # Start of the entry
            $seqid=$1; # store the sequence ID

        } elsif (/^2TB(H|N)/) { # The 2nd table, with FingerPrint summary
            @temp_array = split /\s+/;
            if ($temp_array[9] <= $HOH{$temp_array[1]}{evalue}) { # evalue is better than the cut off in the sibling file
                $fprints->{$seqid}->{$temp_array[1]} -> {evalue} = $temp_array[9]; # store evalue
                $fprints->{$seqid}->{$temp_array[1]} -> {nmotifs} = 1; # initialise total number of motifs
                $fprints->{$seqid}->{$temp_array[1]} -> {graphscan} = $temp_array[10]; # Store graphscan output, just in case :-)
                $fprints->{$seqid}->{$temp_array[1]} -> {printsac} = $HOH{$temp_array[1]}{"accession"}; # look up accession from sibling file
		$fprints->{$seqid}->{$temp_array[1]} -> {status} = "T";
            }
        } elsif (/^3TB(H|N)/) { # The 3rd table, with motif summary

            my ($motifname, $motifnum, $maxmotif, $idscore, $pvalue, $motiflen, $start) =
	    #/^\S+\s+(\w+)\s+(\d+)\s+of\s+(\d+)\s+(\S+)\s+\d+\s+(\S+)\s+\S+\s+(\S{1,5})\s*\S{1,5}\s*(\S{1,5})\s*\S{1,5}/; # FingerPrintScan 3.595 format
	     /^\S+\s+(\w+)\s+(\d+)\s+of\s+(\d+)\s+(\S+)\s+\d+\s+(\S+)\s+\S+\s+(\S{1,5})\s*\S{1,5}\s*(\S{1,6})\s*\S{1,6}/; # FingerPrintScan 3.596 format

            if ($fprints->{$seqid}->{$motifname}) { # if there is a significant match to this fingerprint
                # store the motifs in the hash
                $nmotifs=$fprints->{$seqid}->{$motifname}->{nmotifs};
                $fprints->{$seqid}->{$motifname}->{$nmotifs}->{motifnum} = $motifnum;
                $fprints->{$seqid}->{$motifname}->{$nmotifs}->{maxmotif} = $maxmotif;
                $fprints->{$seqid}->{$motifname}->{$nmotifs}->{idscore} = $idscore;
                $fprints->{$seqid}->{$motifname}->{$nmotifs}->{pvalue} = $pvalue;
                $fprints->{$seqid}->{$motifname}->{$nmotifs}->{start} = $start;
                $fprints->{$seqid}->{$motifname}->{$nmotifs}->{end} = $start+$motiflen-1;
                $fprints->{$seqid}->{$motifname}->{nmotifs}++;
            }

        } elsif (/^3TBF/) { #end of the 3rd table

	    my $bool=0;
	    my @hierarchy=();
	    my $prev_eval = "";

	    # first, sort the fingerprints into ascending e-value order
	    LOOP: foreach my $fprint (
                sort {
                    $fprints->{$seqid}->{$a}->{'evalue'} <=> $fprints->{$seqid}->{$b}->{'evalue'}
                }
                keys( %{$fprints->{$seqid}} ) )
                {

		# Go through them and check:
		# 1) They have the minimum number of motifs listed as required in the hierarchy file
		if ($fprints->{$seqid}->{$fprint}->{nmotifs}-1 >= $HOH{$fprint}{'motifs'}) { # more than the lowest number allowed

		    $prev_eval = $fprints->{$seqid}->{$fprint}->{'evalue'};

		    if (@hierarchy) { # you've already stored a hierarchy

			if ($HOH{$fprint}{'hierarchy'} eq "*") { # the new fprint is a domain, and so should always hit

			    $bool = 1;
			} else {

			    foreach my $member (@hierarchy) { # check the members of the current hierarchy for your hit acc
				if ($member eq $fprint) { # it is in the hierarchy
				    $bool = 1;

				    # Check if its hierarchy is more specific...
				    my @new_h = split (/,/, $HOH{$fprint}{'hierarchy'}); # must have a hierarchy if it is already in one
				    my $new_len = $#new_h;
				    my $old_len = $#hierarchy;
				    @hierarchy = @new_h if ($new_len < $old_len);
				}
			    }
			}

			if ($bool == 0) { # not in hierarchy

			    next LOOP;
			}

		    } else { # no hierarchy stored

			if ($HOH{$fprint}{'hierarchy'}) {
			    if ($HOH{$fprint}{'hierarchy'} ne "*") {
				@hierarchy = split (/,/, $HOH{$fprint}{'hierarchy'});

			    }
			} else { 
			    @hierarchy = ("$fprint"); # if no hierarchy, only allowed to hit itself!
			}
		    }

		    for (my $n=1; $n<=$fprints->{$seqid}->{$fprint}->{nmotifs}-1; $n++) {
			# go through each motif of the current fingerprint
			# and store the location of the matches, etc.
			$href->{$hitnum} = {} unless $href->{$hitnum};
			$href->{$hitnum}->{seqid}  = $seqid;
			$href->{$hitnum}->{methid} = $fprints->{$seqid}->{$fprint}->{printsac};
			$href->{$hitnum}->{prdesc} = $fprint;
			$href->{$hitnum}->{start}  = $fprints->{$seqid}->{$fprint}->{$n}->{start};
			$href->{$hitnum}->{end}    = $fprints->{$seqid}->{$fprint}->{$n}->{end};
			$href->{$hitnum}->{evalue} = $fprints->{$seqid}->{$fprint}->{evalue};
			$hitnum++;
		    } # END for...
		    $bool = 0;

		}  # END if ($fprints...
	    } # END foreach ($fprint...
	}
    }
    $href->{nhits} = $hitnum -1;
                                                                                                                                                             
    close FILE;

    return (1, '');
}


=head2 getRawResult

 Description: Returns raw formatted result string.
 Arguments:   $href      Reference to a result hash.
              $seqs      Sequence hash (see InterProScan::checkSequences).
              $iprscan   Reference to Dispatcher::Tool::InterProScan object.
                         Required for IPR fields and GO terms.
              $ipr       1: add IPR fields
                         0: no (default)
              $go        1: add GO terms
                         0: no (default)
 Returns:     1, str on success
              0, msg on failure

=cut

sub getRawResult($$;$$$) {
    my ($self, $href, $seqs, $iprscan, $ipr, $go) = @_;
    return (0, "no result hash defined")    unless $href && ref($href) eq 'HASH';
    return (0, "no sequences hash defined") unless $seqs && ref($seqs) eq 'HASH';

    if (!$iprscan && ($ipr || $go)) {
	return (0, "no InterProScan object defined");
    }
    elsif ($iprscan && ref($iprscan) ne 'Dispatcher::Tool::InterProScan') {
	return (0, "InterProScan object has wrong class: " . ref($iprscan));
    }

    my $nhits = $href->{nhits};
    return (0, "number of hits not defined in result hash") unless defined $nhits;
    return (1, '') if $nhits == 0;

    my $appl = $self->getConfigValue('applname');
    my $date = POSIX::strftime("%d-%b-%Y", localtime);
    my ($hit, $raw, $seqid, $seqcrc, $seqlen, $res, $iprac, $iprname, $goterms);

    for (my $i = 1; $i <= $nhits; $i++) {
	$hit = $href->{$i};
	next unless $hit;  # should not happen

	$seqid = $hit->{seqid};
	next unless $seqid;  # should not happen

	$seqcrc = $seqs->{$seqid}->{crc64};
	$seqlen = $seqs->{$seqid}->{len};

	$raw .= sprintf("%s\t%s\t%s\t", $seqid, $seqcrc, $seqlen);
        $raw .= sprintf("%s\t%s\t%s\t", $appl, $hit->{methid}, $hit->{prdesc});
	if ($hit->{start} < 1) {
	    $raw .= sprintf("%s\t", "1");
	} else {
	    $raw .= sprintf("%d\t", $hit->{start});
	}
	if ($hit->{end} > $seqlen) {
	    $raw .= sprintf("%s\t", $seqlen);
	} else {
	    $raw .= sprintf("%d\t", $hit->{end});
	}
	$raw .= sprintf("%s\tT\t", $hit->{evalue});
	$raw .= sprintf("%s", $date);

	if ($ipr) {
	    ($res, $iprac, $iprname, $goterms) = $iprscan->getInterProFields($hit->{methid}, $go);
	    return (0, $iprac) unless $res;

	    $raw .= sprintf("\t$iprac\t$iprname") if $ipr;
	    $raw .= sprintf("\t$goterms") if $go;
        }

	$raw .= "\n";
    }

    return (1, $raw);
}



1;
__END__

