=head1 NAME

Gene3d - Gene3d utilities

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::Gene3D;

=head1 DESCRIPTION

=head1 VERSIONS

$Id: Gene3D.pm,v 1.1 2010/10/20 19:04:44 hunter Exp $

Copyright (c) European Bioinformatics Institute 2010

=head1 AUTHORS / ACKNOWLEDGEMENTS

Sarah Hunter <hunter@ebi.ac.uk>

=cut

package Dispatcher::Tool::Gene3D;

BEGIN {
  unless($ENV{IPRSCAN_HOME}){
    print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
    exit();
  }
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool;
@ISA = qw( Dispatcher::Tool );


use Data::Dumper;
use English;
use File::Spec::Functions;
use POSIX qw(strftime);
use Sys::Hostname;
use strict;


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        Gene3d tool name (optional). If given, sets
                           the Dispatcher::Config object automatically inside
                           the object. If not given, the Dispatcher::Config
                           must be set later with setConfig.
              $defaults    Default values for configuration, hash reference
                           (optional).
 Returns:     $self object

=cut

sub new(;$$) {
    my ($class, $name, $defaults) = @_;

    my $self = { };
    bless $self, $class;

    if ($name) {
        $self->setToolName($name);
        my ($res, $msg) = $self->parseConfig(1, $defaults);
        die "failed to parse $name configuration: $msg\n" unless $res;
    }

    return $self;
}


=head2 getCmdLine

 Description: Returns Gene3d command line.
 Arguments:   -
 Returns:     1, cmd on success
              0, msg on failure

=cut

sub getCmdLine() {
    my $self = shift;
    return $self->getConfigValue('cmdline');
}


=head2 parseResult

 Description: Reads a Gene3d DomainFinder3 result file and creates a hash table:

 Arguments:   $href          Reference to a hash.
              $result        Path to a Gene3d DomainFinder3 result file.
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub parseResult($$$) {
    my ($self, $href, $result, $inx) = @_;
    return (0, "no gene3d input file defined") unless $result;

    $href = { } unless $href && ref($href) eq 'HASH';
    $inx  = { } unless $inx && ref($inx) eq 'HASH';

    unless (open(MAP, "< $ENV{IPRSCAN_HOME}/data/model2sf_map.csv")) {
	return (0, "could not read gene3d mapping file $ENV{IPRSCAN_HOME}/data/model2sf_map.csv");
    }

    my %hmap;
    my @cols;
    my $hit = 1;

    while (<MAP>) {
	chomp;
	if (/^(\S+),(\S+)/) {
	    $hmap{$1} = "$2";
	} else {
	    return (0, "mapping file is in the incorrect format, please check!");
	}
    }

    close (MAP);

    unless (open(FILE, "< $result")) {
        return (0, "could not read gene3d result file $result: $!");
    }

    while (<FILE>) {
        chomp;

        # 
        # Example result:
	#
	#sp|P11387|TOP1_HUMAN	1k4tA01	765 0	0   0	0   0	0   0	2e-69	0.00	0.00	2   202:231:320:431  1
	#sp|P11387|TOP1_HUMAN	1oisA01	765 0	0   0	0   0	0   0	3.5e-37	0.00	0.00	1   232:319  1
	#sp|P11387|TOP1_HUMAN	1a31A03	765 0	0   0	0   0	0   0	2.2e-72	0.00	0.00	1   432:579  1
	#sp|P11387|TOP1_HUMAN	1k4tA04	765 0	0   0	0   0	0   0	6e-78	0.00	0.00	1   587:765  1

	$href->{$hit} = { } unless $href->{$hit};


	@cols = split (/\s+/,$_);
	my @tmp = split (/:/,$cols[14]);
	while (@tmp) {
	    my $seq_from = shift (@tmp);
	    my $seq_to = shift (@tmp);
	    unless ($seq_from == $seq_to) { # for when HMMER3 erroneously predicts domains of 1aa length
		$href->{$hit}->{seqid} = $cols[0];
		if ($hmap{$cols[1]}) {
		    $href->{$hit}->{methid} = "G3DSA:".$hmap{$cols[1]};
		} else {
		    return (0, "$cols[1] not found in mapping file, please check it is from the same release");
		}
		$href->{$hit}->{evalue} = $cols[10];
		$href->{$hit}->{start} = $seq_from;
		$href->{$hit}->{end} = $seq_to;
		$href->{$hit}->{status} = "T";
		$href->{$hit}->{desc} = "no description";
	    }
	}
	
	$hit++;
    }
    $href->{nhits} = $hit -1;

    close FILE;

    return (1, '');
}


=head2 getRawResult

 Description: Returns raw formatted result string.
 Arguments:   $href      Reference to a result hash.
              $seqs      Sequence hash (see InterProScan::checkSequences).
              $iprscan   Reference to Dispatcher::Tool::InterProScan object.
                         Required for IPR fields and GO terms.
              $ipr       1: add IPR fields
                         0: no (default)
              $go        1: add GO terms
                         0: no (default)
 Returns:     1, str on success
              0, msg on failure

=cut

sub getRawResult($$;$$$) {
    my ($self, $href, $seqs, $iprscan, $ipr, $go) = @_;
    return (0, "no result hash defined")    unless $href && ref($href) eq 'HASH';
    return (0, "no sequences hash defined") unless $seqs && ref($seqs) eq 'HASH';

    if (!$iprscan && ($ipr || $go)) {
	return (0, "no InterProScan object defined");
    }
    elsif ($iprscan && ref($iprscan) ne 'Dispatcher::Tool::InterProScan') {
	return (0, "InterProScan object has wrong class: " . ref($iprscan));
    }

    my $nhits = $href->{nhits};
    return (0, "number of hits not defined in result hash") unless defined $nhits;
    return (1, '') if $nhits == 0;

    my $appl = $self->getConfigValue('applname');
    my $date = POSIX::strftime("%d-%b-%Y", localtime);
    my ($hit, $raw, $seqid, $seqcrc, $seqlen, $ntstart, $ntend, $res, $iprac, $iprname, $goterms);

    for (my $i = 1; $i <= $nhits; $i++) {
	$hit = $href->{$i};
	next unless $hit;  # should not happen

	$seqid = $hit->{seqid};
	next unless $seqid;  # should not happen

	$seqcrc = $seqs->{$seqid}->{crc64};
	$seqlen = $seqs->{$seqid}->{len};
	$raw .= sprintf("%s\t%s\t%s\t", $seqid, $seqcrc, $seqlen);
        $raw .= sprintf("%s\t%s\t%s\t", $appl, $hit->{methid}, $hit->{desc});
	$raw .= sprintf("%s\t%s\t", $hit->{start}, $hit->{end});
	$raw .= sprintf("%s\tT\t", $hit->{evalue});
	$raw .= sprintf("%s", $date);

	if ($ipr) {
	    ($res, $iprac, $iprname, $goterms) = $iprscan->getInterProFields($hit->{methid}, $go);
	    return (0, $iprac) unless $res;

	    $raw .= sprintf("\t$iprac\t$iprname") if $ipr;
	    $raw .= sprintf("\t$goterms") if $go;
        }

	$raw .= "\n";
    }

    return (1, $raw);
}



1;
__END__

