=head1 NAME

HAMAP.pm - HAMAP utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::HAMAP;

=head1 DESCRIPTION

=head1 VERSIONS

$Id: HAMAP.pm,v 1.1 2009/07/14 16:43:59 hunter Exp $

Copyright (c) European Bioinformatics Institute 2009

=head1 AUTHORS / ACKNOWLEDGEMENTS

Sarah Hunter <hunter (at) ebi.ac.uk>

=cut

package Dispatcher::Tool::HAMAP;

BEGIN {
  unless($ENV{IPRSCAN_HOME}){
    print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
    exit();
  }
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool;
@ISA = qw( Dispatcher::Tool );


use Data::Dumper;
use English;
use File::Spec::Functions;
use POSIX qw(strftime);
use Sys::Hostname;
use strict;


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        HAMAP tool name (optional). If given, sets
                           the Dispatcher::Config object automatically inside
                           the object. If not given, the Dispatcher::Config
                           must be set later with setConfig.
              $defaults    Default values for configuration, hash reference
                           (optional).
 Returns:     $self object

=cut

sub new(;$$) {
    my ($class, $name, $defaults) = @_;

    my $self = { };
    bless $self, $class;

    if ($name) {
        $self->setToolName($name);
        my ($res, $msg) = $self->parseConfig(1, $defaults);
        die "failed to parse $name configuration: $msg\n" unless $res;
    }

    return $self;
}


=head2 getCmdLine

 Description: Returns HAMAP command line.
 Arguments:   -
 Returns:     1, cmd on success
              0, msg on failure

=cut

sub getCmdLine() {
    my $self = shift;
    return $self->getConfigValue('cmdline');
}


=head2 parseResult

 Description: Reads a HAMAP result file and creates a hash table:

              'nhits' => '2'
              '1'     => { 'seqid'  => 'WAP_RAT',
                           'hitid'  => 'CYS_RICH',
                           'evalue' => '9.927',
                           'pos'    => '41',
                  	   'start'  => '45', 
                  	   'end'    => '123',
                           'methid' => 'PS50311',
                           'desc' => 'CYS_RICH Cysteine-rich region.'
                  	 },
              '2'     => { ... }

 Arguments:   $href          Reference to a hash.
              $result        Path to a HAMAP result file.
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub parseResult($$$) {
    my ($self, $href, $result, $inx) = @_;
    return (0, "no HAMAP input file defined") unless $result;

    $href = { } unless $href && ref($href) eq 'HASH';
    $inx = { } unless $inx && ref($inx) eq 'HASH';

    unless (open(FILE, "< $result")) {
        return (0, "could not read HAMAP result file $result: $!");
    }

    my $seqid;
    my $hit = 1;

    my $fpos = 0;

    while (<FILE>) {
        chomp;

        #
        # Example result:
        #
	#sp|P0A5Z8|PRRB_MYCTU    ps_scan|v1.57   MF_01837        59      446     9.168   .       .       Name "SASA_MF_01837" ; Level -1 ; RawScore 951 ; FeatureFrom 1 ; FeatureTo -1 ; Sequence "FAIPFVPRGLDEIPRSPNDQDALITVRRGNVIKSNSDITLPKLQDDYADTYVRGVRYRVRTVEIPGPEPTSVAVGATYDATVAETNNLHRRVLLictfaiGAaavFAwllaafavrpfKQLAE--QtRsiDAGD-EAPRVEVHGASEAIEIAeAMRGMLQRIWNEQNRTKEALASARDFAAVSSHELRTPLTAMRTNLEVL-------STLDLPDDQRKEVLNDVIRTQSRIEATLSALERLAQ-GELSTSDDHVPVDITDLLDRAAHDAARIYPDLDVSL---VPSPTCIIVGLPAGLRLAVDNAIANAVKH--GGATLVQLSAVSSRAGVEIAIDDNGSGVPEGERQVVFE---RFSRgsTASHSGSGLGLALVAQQAQLHGGTASLENSPLGGARLVLRLPGPS" ; SequenceDescription "sp|P0A5Z8|PRRB_MYCTU "
	#sp|P0A7T3|RS16_ECOLI    ps_scan|v1.57   MF_00385        1       78      24.166  .       .       Name "RS16_MF_00385" ; Level 1 ; RawScore 2091 ; FeatureFrom 1 ; FeatureTo -1 ; Sequence "MVTIRLARHGAKKRPFYQVVVADSRNARNGRFIERVGFFNPIASekEEGTRLDLDRIAHWVGQGATISDRVAALIKEV" ; SequenceDescription "sp|P0A7T3|RS16_ECOLI"
	### next sequence

        if ($_ =~ /^([^\s]+)\tps_scan\|[\w\d\.]+\t(MF_\d+)\t(\d+)\t(\d+)\t([\d\.\-\e]+)\t\S+\t\S+\tName\s+.([^\"]+)./){ 
            $href->{$hit} = { } unless $href->{$hit};
	    if(exists($inx->{$1})){
	      $inx->{$1} .= '+' . $fpos;
	    }else{
	      $inx->{$1} = $fpos;
	    }
            $href->{$hit}->{seqid}  = $1;
            $href->{$hit}->{hitid}  = $2;
            $href->{$hit}->{evalue} = $5;
            $href->{$hit}->{start}  = $3;
            $href->{$hit}->{end}    = $4;
            $href->{$hit}->{methid} = $2;
            $href->{$hit}->{desc} = $6;
	    $hit++;
        }
	$fpos = tell(FILE);
    }

    $href->{nhits} = $hit - 1;

    close FILE;

    return (1, '');
}


=head2 getRawResult

 Description: Returns raw formatted result string.
 Arguments:   $href      Reference to a result hash.
              $seqs      Sequence hash (see InterProScan::checkSequences).
              $iprscan   Reference to Dispatcher::Tool::InterProScan object.
                         Required for IPR fields and GO terms.
              $ipr       1: add IPR fields
                         0: no (default)
              $go        1: add GO terms
                         0: no (default)
 Returns:     1, str on success
              0, msg on failure

=cut

sub getRawResult($$;$$$) {
    my ($self, $href, $seqs, $iprscan, $ipr, $go) = @_;
    return (0, "no result hash defined")     unless $href && ref($href) eq 'HASH';
    return (0, "no sequences hash defined") unless $seqs && ref($seqs) eq 'HASH';

    if (!$iprscan && ($ipr || $go)) {
	return (0, "no InterProScan object defined");
    }
    elsif ($iprscan && ref($iprscan) ne 'Dispatcher::Tool::InterProScan') {
	return (0, "InterProScan object has wrong class: " . ref($iprscan));
    }

    my $nhits = $href->{nhits};
    return (0, "number of hits not defined in result hash") unless defined $nhits;
    return (1, '') if $nhits == 0;

    my $appl = $self->getConfigValue('applname');
    my $date = POSIX::strftime("%d-%b-%Y", localtime);
    my ($hit, $raw, $seqid, $seqcrc, $seqlen, $res, $iprac, $iprname, $goterms);

    for (my $i = 1; $i <= $nhits; $i++) {
	$hit = $href->{$i};
	next unless $hit;  # should not happen

	$seqid = $hit->{seqid};
	next unless $seqid;  # should not happen

	$seqcrc = $seqs->{$seqid}->{crc64};
	$seqlen = $seqs->{$seqid}->{len};

	$raw .= sprintf("%s\t%s\t%s\t", $seqid, $seqcrc, $seqlen);
        $raw .= sprintf("%s\t%s\t%s\t", $appl, $hit->{methid}, $hit->{desc});  # FIXME: check from tuco is it hitid or desc!!!
	$raw .= sprintf("%d\t%d\t", $hit->{start}, $hit->{end});
	$raw .= sprintf("%s\tT\t", $hit->{evalue});
	$raw .= sprintf("%s", $date);

	if ($ipr) {
	    ($res, $iprac, $iprname, $goterms) = $iprscan->getInterProFields($hit->{methid}, $go);
	    return (0, $iprac) unless $res;

	    $raw .= sprintf("\t$iprac\t$iprname") if $ipr;
	    $raw .= sprintf("\t$goterms") if $go;
        }

	$raw .= "\n";
    }

    return (1, $raw);
}



1;
__END__

