=head1 NAME

HMMPIR - HMMPIR utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::HMMPIR;

=head1 DESCRIPTION

=head1 VERSIONS

$Id: HMMPIR.pm,v 1.3 2010/01/20 13:52:34 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

interhelp@ebi.ac.uk
Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut

package Dispatcher::Tool::HMMPIR;

BEGIN {
  unless($ENV{IPRSCAN_HOME}){
    print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
    exit();
  }
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool;
@ISA = qw( Dispatcher::Tool );


use Data::Dumper;
use English;
use File::Spec::Functions;
use POSIX qw(strftime);
use Sys::Hostname;
use strict;


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        HMMPIR tool name (optional). If given, sets the
                           Dispatcher::Config object automatically inside
                           the object. If not given, the Dispatcher::Config
                           must be set later with setConfig.
              $defaults    Default values for configuration, hash reference
                           (optional).
 Returns:     $self object

=cut

sub new(;$$) {
    my ($class, $name, $defaults) = @_;

    my $self = { };
    bless $self, $class;

    if ($name) {
        $self->setToolName($name);
        my ($res, $msg) = $self->parseConfig(1, $defaults);
        die "failed to parse $name configuration: $msg\n" unless $res;
    }

    return $self;
}


=head2 getCmdLine

 Description: Returns HMMPIR command line.
 Arguments:   -
 Returns:     1, cmd on success
              0, msg on failure

=cut

sub getCmdLine() {
    my $self = shift;
    return $self->getConfigValue('cmdline');
}


=head2 parseResult

 Description: Reads a HMMPIR result file and creates a hash table:

              nhits  => '2',
              '1' => { 'seqid'   => 'NF00181542',
		       'methid'  => 'PIRSF001424',
                       'pirdesc' => 'Prephenate dehydratase',
		       'start'   => '1',
		       'end'     => '270',
                       'score'   => '474.8',
                       'evalue'  => '6e-142'
                     },
              '2' => { 'seqid'   => 'NF00181707',
		       'methid'  => 'PIRSF006414',
                       'pirdesc' => 'Formylmethanofuran--tetrahydromethanopterin formyltransferase',
		       'start'   => '1',
		       'end'     => '301',
                       'score'   => '772.3',
                       'evalue'  => '1.7e-231'
                     }

 Arguments:   $href          Reference to a hash.
              $result        Path to a HMMPIR result file.
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub parseResult($$) {
    my ($self, $href, $result, $inx) = @_;
    return (0, "no HMMPIR input file defined") unless $result;

    $href = { } unless $href && ref($href) eq 'HASH';
    $inx = { } unless $inx && ref($inx) eq 'HASH';

    unless (open(FILE, "< $result")) {
        return (0, "could not read HMMPIR result file $result: $!");
    }

    my $isscores = 0;
    my $hit      = 1;;

    my ($seqid, $pirid, $pirdesc);
    my $fpos = 0;

    while (<FILE>) {
        chomp;

        #
        # Example result:
        #
        # Query sequence: NF00797791 No Match
        # //
        # Query sequence: NF00798914 No Match
        # //
	#
	#Query sequence: NF00184720  matches PIRSF005538: DNA primase, small subunit
	#
	#Full-length Match:
	#Model         Domain  seq-f seq-t    hmm-f hmm-t      score  E-value
	#
	#PIRSF005538     1/1       1   333 []     1   463 []   441.6 4.4e-131
	#
	#and matches Sub-Family PIRSF500023: DNA primase, small subunit, archaeal type
	#
	#Model         Domain  seq-f seq-t    hmm-f hmm-t      score  E-value
	#
	#PIRSF500023     1/1       2   333 .]     1   405 []   567.6 2.8e-170
	#
	#//
	#
	#Query sequence: NF00184087  matches PIRSF001349: Uroporphyrinogen-III decarboxylase/methylcobamide:CoM methyltransferase
	#
	#Full-length Match:
	#Model     Domain  seq-f seq-t    hmm-f hmm-t      score  E-value
	#
	#PIRSF001349   1/1       1   332 []     1   374 []   389.6 2.1e-115
	#
	#and matches Sub-Family PIRSF500079: Uroporphyrinogen-III decarboxylase
	#
	#Model    Domain  seq-f seq-t    hmm-f hmm-t      score  E-value
	#
	#PIRSF500079   1/1       1   332 []     1   362 []   409.8 1.1e-122
	#
	#//

	if ($_ =~ /^Query\s+sequence:\s+([\w\|\.-]+)\s+matches\s+(\w+):\s+(.*)/) {
	    $seqid   = $1;
	    $pirid   = $2;
	    $pirdesc = $3;
	    $inx->{$1} = $fpos;
        } 
	elsif ($_ =~ /^and matches Sub-Family\s+(\w+):\s+(.*)/) {
	    $pirid  = $1;
	    $pirdesc = $2;
	    $inx->{$seqid} = $fpos;
	}
	elsif ($_ =~ /^Full-length\s+Match:/) {
	    $isscores  = 1;
        }
	elsif ($isscores && $_ =~ /^\w+\s+\d+\/\d+\s+(\d+)\s+(\d+)\s+[\[\.][\]\.]\s+\d+\s+\d+\s+[\[\.][\]\.]\s+([\-\d\.]+)\s+(.*)/) {
	    next unless $seqid && $pirid;
	    $href->{$hit} = { };
            $href->{$hit}->{seqid}   = $seqid;
            $href->{$hit}->{methid}  = $pirid;
            $href->{$hit}->{pirdesc} = $pirdesc if $pirdesc;
            $href->{$hit}->{start}   = $1;
            $href->{$hit}->{end}     = $2;
            $href->{$hit}->{score}   = $3;
            $href->{$hit}->{evalue}  = $4;
	    $hit++;
        }
	elsif ($_ =~ /^\/\//){
	    $isscores = 0;
	    $fpos = tell(FILE);
	}
    }

    close FILE;

    $href->{nhits} = $hit - 1;

    return (1, '');
}


=head2 getRawResult

 Description: Returns raw formatted result string.
 Arguments:   $href      Reference to a result hash.
              $seqs      Sequence hash (see InterProScan::checkSequences).
              $iprscan   Reference to Dispatcher::Tool::InterProScan object.
                         Required for IPR fields and GO terms.
              $ipr       1: add IPR fields
                         0: no (default)
              $go        1: add GO terms
                         0: no (default)
 Returns:     1, str on success
              0, msg on failure

=cut

sub getRawResult($$;$$$) {
    my ($self, $href, $seqs, $iprscan, $ipr, $go) = @_;
    return (0, "no result hash defined")    unless $href && ref($href) eq 'HASH';
    return (0, "no sequences hash defined") unless $seqs && ref($seqs) eq 'HASH';

    if (!$iprscan && ($ipr || $go)) {
	return (0, "no InterProScan object defined");
    }
    elsif ($iprscan && ref($iprscan) ne 'Dispatcher::Tool::InterProScan') {
	return (0, "InterProScan object has wrong class: " . ref($iprscan));
    }

    my $nhits = $href->{nhits};
    return (0, "number of hits not defined in result hash") unless defined $nhits;
    return (1, '') if $nhits == 0;

    my $appl = $self->getConfigValue('applname');
    my $date = POSIX::strftime("%d-%b-%Y", localtime);
    my ($hit, $raw, $seqid, $seqcrc, $seqlen, $res, $iprac, $iprname, $goterms);
    my $desc = { };

    for (my $i = 1; $i <= $nhits; $i++) {
	$hit = $href->{$i};
	next unless $hit;  # should not happen

	$seqid = $hit->{seqid};
	next unless $seqid;  # should not happen

	$seqcrc = $seqs->{$seqid}->{crc64};
	$seqlen = $seqs->{$seqid}->{len};

	$raw .= sprintf("%s\t%s\t%s\t", $seqid, $seqcrc, $seqlen);
        $raw .= sprintf("%s\t%s\t%s\t", $appl, $hit->{methid}, $hit->{pirdesc});
	$raw .= sprintf("%d\t%d\t", $hit->{start}, $hit->{end});
	$raw .= sprintf("%s\tT\t", $hit->{evalue});
	$raw .= sprintf("%s", $date);

	if ($ipr) {
	    ($res, $iprac, $iprname, $goterms) = $iprscan->getInterProFields($hit->{methid}, $go);
	    return (0, $iprac) unless $res;

	    $raw .= sprintf("\t$iprac\t$iprname") if $ipr;
	    $raw .= sprintf("\t$goterms") if $go;
        }

	$raw .= "\n";
    }

    return (1, $raw);
}


1;
__END__

