=head1 NAME

HMMPanther - Panther utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::HMMPanther;

=head1 DESCRIPTION

=head1 VERSIONS

$Id: HMMPanther.pm,v 1.1.1.1 2005/08/18 13:18:26 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut

package Dispatcher::Tool::HMMPanther;

BEGIN {
  unless($ENV{IPRSCAN_HOME}){
    print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
    exit();
  }
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool;
@ISA = qw( Dispatcher::Tool );

use Data::Dumper;
use English;
use File::Spec::Functions;
use POSIX qw(strftime);
use Sys::Hostname;
use strict;


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        HMMPanther tool name (optional). If given, sets the
                           Dispatcher::Config object automatically inside
                           the object. If not given, the Dispatcher::Config
                           must be set later with setConfig.
              $defaults    Default values for configuration, hash reference
                           (optional).
 Returns:     $self object

=cut

sub new(;$$) {
    my ($class, $name, $defaults) = @_;

    my $self = { };
    bless $self, $class;

    if ($name) {
        $self->setToolName($name);
        my ($res, $msg) = $self->parseConfig(1, $defaults);
        die "failed to parse $name configuration: $msg\n" unless $res;
    }

    return $self;
}


=head2 getCmdLine

 Description: Returns Hmmer command line.
 Arguments:   -
 Returns:     1, cmd on success
              0, msg on failure

=cut

sub getCmdLine() {
    my $self = shift;
    return $self->getConfigValue('cmdline');
}


=head2 parseResult

 Description: Reads a Hmmer result file and creates a hash table:

              nhits       => '2',
              '1'         => { seqid       => 'Y902_MYCTU',
                               ndomains    => '3',
                               '1'         => { methid => 'PF00672',
                                                desc   => 'HAMP domain',
                                                start  => '151',
                                                end    => '219',
                                                score  => '46.9',
                                                evalue => '4.3e-11'
                                              },
                               '2'         => { methid => 'PF00512',
                                                desc   => 'His Kinase A (phosphoacceptor) domain',
                                                start  => '230',
                                                end    => '296',
                                                score  => '55.8',
                                                evalue => '9.2e-14'
                                              },
                               '3'         => { methid => 'PF02518',
                                                desc   => 'Histidine kinase-, DNA gyrase B-, and HSP90',
                                                start  => '338',
                                                end    => '445',
                                                score  => '105.6',
                                                evalue => '9.6e-29'
                                              },
                             },
              '2'         => { seqid       => 'WAP_RAT',
                               ...
                             }

 Arguments:   $href          Reference to a hash.
              $result        Path to a Hmmer result file.
              $inx           Reference to a hash.
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub parseResult($$) {
    my ($self, $href, $result, $inx) = @_;
    return (0, "no Hmmer input file defined") unless $result;
    
    $href = { } unless $href && ref($href) eq 'HASH';
    $inx  = { } unless $inx && ref($inx) eq 'HASH';
  
    unless (open(FILE, "< $result")) {
	return (0, "could not read Hmmer result file $result: $!");
    }
  
    my $hit = 1;
    my $fpos = 0;
  
    while (<FILE>) {
	
	chomp;

	#Y902_MYCTU      PTHR10919       TWO COMPONENT SENSOR KINASE     2.4e-38 138.2   151-225
	#Y902_MYCTU      PTHR10919       TWO COMPONENT SENSOR KINASE     2.4e-38 138.2   226-443
	#Q9RHD9  PTHR10287:SF6   30S RIBOSOMAL PROTEIN S1        4.9e-124        422.9   4-138
	#Q9RHD9  PTHR10287:SF6   30S RIBOSOMAL PROTEIN S1        4.9e-124        422.9   139-263
	
	$href->{$hit} = { } unless $href->{$hit};
	
	my ($seqid, $pthid, $name, $eval, $score, $locs) = split(/\t/, $_);
	
	$href->{$hit} = { } unless $href->{$hit};
	$href->{$hit}->{seqid}  = $seqid;
	$href->{$hit}->{methid} = $pthid;
	$href->{$hit}->{name}   = $name || 'no name'; #if -n not given on the command line
	$href->{$hit}->{evalue} = $eval;
	$href->{$hit}->{score}  = $score;
	
	if($locs =~ /(\d+)-(\d+)/){
	    $href->{$hit}->{start} = $1;
	    $href->{$hit}->{end}   = $2;
	}else{
	  return(0, "Parsing Problem for Panther with location $locs");
      }
	
	$hit++;
	
	if(exists($inx->{$seqid})){
	    $inx->{$seqid} .= '+' . $fpos;
	}else{
	    $inx->{$seqid} = $fpos;
	}
	
	$fpos = tell(FILE);
    }

    close FILE;
    $href->{nhits} = $hit - 1;

    return (1, '');
}


=head2 getRawResult

 Description: Returns raw formatted result string.
 Arguments:   $href      Reference to a result hash.
              $seqs      Sequence hash (see InterProScan::checkSequences).
              $iprscan   Reference to Dispatcher::Tool::InterProScan object.
                         Required for IPR fields and GO terms.
              $ipr       1: add IPR fields
                         0: no (default)
              $go        1: add GO terms
                         0: no (default)
 Returns:     1, str on success
              0, msg on failure

=cut

sub getRawResult($$;$$$) {
    my ($self, $href, $seqs, $iprscan, $ipr, $go) = @_;
    return (0, "no result hash defined")     unless $href && ref($href) eq 'HASH';
    return (0, "no sequences hash defined") unless $seqs && ref($seqs) eq 'HASH';

    if (!$iprscan && ($ipr || $go)) {
	return (0, "no InterProScan object defined");
    }
    elsif ($iprscan && ref($iprscan) ne 'Dispatcher::Tool::InterProScan') {
	return (0, "InterProScan object has wrong class: " . ref($iprscan));
    }

    my $nhits = $href->{nhits};
    return (0, "number of hits not defined in result hash") unless defined $nhits;
    return (1, '') if $nhits == 0;

    my $appl = $self->getConfigValue('applname');
    my $date = POSIX::strftime("%d-%b-%Y", localtime);
    my ($hit, $raw, $seqid, $seqcrc, $seqlen, $res, $iprac, $iprname, $goterms);

    for (my $i = 1; $i <= $nhits; $i++) {
	$hit = $href->{$i};
	next unless $hit;  # should not happen

	$seqid = $hit->{seqid};
	next unless $seqid;  # should not happen

	$seqcrc = $seqs->{$seqid}->{crc64};
	$seqlen = $seqs->{$seqid}->{len};

	$raw .= sprintf("%s\t%s\t%s\t", $seqid, $seqcrc, $seqlen);
        $raw .= sprintf("%s\t%s\t%s\t", $appl, $hit->{methid}, $hit->{name});
	$raw .= sprintf("%d\t%d\t", $hit->{start}, $hit->{end});
	$raw .= sprintf("%s\tT\t", $hit->{evalue});
	$raw .= sprintf("%s", $date);

	if ($ipr) {
	    ($res, $iprac, $iprname, $goterms) = $iprscan->getInterProFields($hit->{methid}, $go);
	    return (0, $iprac) unless $res;

	    $raw .= sprintf("\t$iprac\t$iprname") if $ipr;
	    $raw .= sprintf("\t$goterms") if $go;
        }

	$raw .= "\n";
    }

    return (1, $raw);
  }


1;
__END__

