=head1 NAME

Hmmer - Hmmer utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::Hmmer;

=head1 DESCRIPTION

=head1 VERSIONS

$Id: Hmmer.pm,v 1.11 2010/01/20 13:52:30 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

Sarah Hunter <hunter@ebi.ac.uk>
Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut

package Dispatcher::Tool::Hmmer;

BEGIN {
    unless($ENV{IPRSCAN_HOME}){
	print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
	exit();
    }
    unless($ENV{IPRSCAN_LIB}){
	$ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
    }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool;
@ISA = qw( Dispatcher::Tool );

use Data::Dumper;
use English;
use File::Spec::Functions;
use POSIX qw(strftime);
use Sys::Hostname;
use strict;


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        Hmmer tool name (optional). If given, sets the
                           Dispatcher::Config object automatically inside
                           the object. If not given, the Dispatcher::Config
                           must be set later with setConfig.
              $defaults    Default values for configuration, hash reference
                           (optional).
 Returns:     $self object

=cut

sub new(;$$) {
    my ($class, $name, $defaults) = @_;

    my $self = { };
    bless $self, $class;

    if ($name) {
        $self->setToolName($name);
        my ($res, $msg) = $self->parseConfig(1, $defaults);
        die "failed to parse $name configuration: $msg\n" unless $res;
    }

    return $self;
}


=head2 getCmdLine

 Description: Returns Hmmer command line.
 Arguments:   -
 Returns:     1, cmd on success
              0, msg on failure

=cut

sub getCmdLine() {
    my $self = shift;
    return $self->getConfigValue('cmdline');
}


=head2 parseResult

 Description: Reads a Hmmer result file and creates a hash table:

              nhits       => '2',
              '1'         => { seqid       => 'Y902_MYCTU',
                               ndomains    => '3',
                               '1'         => { methid => 'PF00672.2.ls',
                                                desc   => 'HAMP domain',
                                                start  => '151',
                                                end    => '219',
                                                score  => '46.9',
                                                evalue => '4.3e-11'
                                              },
                               '2'         => { methid => 'PF00512.1.ls',
                                                desc   => 'His Kinase A (phosphoacceptor) domain',
                                                start  => '230',
                                                end    => '296',
                                                score  => '55.8',
                                                evalue => '9.2e-14'
                                              },
                               '3'         => { methid => 'PF02518.18.ls',
                                                desc   => 'Histidine kinase-, DNA gyrase B-, and HSP90',
                                                start  => '338',
                                                end    => '445',
                                                score  => '105.6',
                                                evalue => '9.6e-29'
                                              },
                             },
              '2'         => { seqid       => 'WAP_RAT',
                               ...
                             }

 Arguments:   $href          Reference to a hash.
              $result        Path to a Hmmer result file.
              $inx           Reference to a hash.
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub parseResult($$) {
  my ($self, $href, $result, $inx) = @_;
  return (0, "no Hmmer input file defined") unless $result;
  
  $href = { } unless $href && ref($href) eq 'HASH';
  $inx  = { } unless $inx && ref($inx) eq 'HASH';
  
  unless (open(FILE, "< $result")) {
    return (0, "could not read Hmmer result file $result: $!");
  }
  
  my $hit = 1;
  my $isscores  = 0;
  my $isdomains = 0;
  my $domain    = 1; 
  my $hdesc     = { };
  
  my $fpos = 0;
  
  while (<FILE>) {
    chomp;
    
    #
    # Example result (imaginary):
    #
    # hmmpfam - search one or more sequences against HMM database
    # HMMER 2.3.2 (Oct 2003)
    # Copyright (C) 1992-2003 HHMI/Washington University School of Medicine
    # Freely distributed under the GNU General Public License (GPL)
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # HMM file:                 /ebi/services/idata/hmmerdb/pfam/Pfam
    # Sequence file:            /ebi/extserv/iprscan-work/interactive/iprscan-20031126-14555233.input.3
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    #
    # Query sequence: UPI00005BE95F
    # Accession:      [none]
    # Description:    [none]
    #
    # Scores for sequence family classification (score includes all domains):
    # Model        Description                                    Score    E-value  N
    # ------------ -----------                                    -----    ------- ---
    # PF00038.9.ls Intermediate filament protein                  468.0     1e-137   1
    # PF07142.1.fs Repeat of unknown function (DUF1388)           214.7     1.8e-61 13
    #
    # Parsed for domains:
    # Model        Domain  seq-f seq-t    hmm-f hmm-t      score  E-value
    # ------------ ------- ----- -----    ----- -----      -----  -------
    # PF00038.9.ls   1/1      91   405 ..     1   359 []   468.0   1e-137
    # PF07142.1.fs   2/13    511   538 ..     1    28 []    32.7  1.2e-06
    # PF07142.1.fs   3/13    543   570 ..     1    28 []    25.1  0.00022
    # PF02818.4.fs   1/1     859   882 ..     1    28 []     0.1       17
    #
    # Alignments of top-scoring domains:
    # <snip>
    # //
    # 
    # Query sequence: Y902_MYCTU
    # Accession:      [none]
    # Description:    [none]
    # 
    # Scores for sequence family classification (score includes all domains):
    # Model             Description                                   Score    E-value  N
    # --------          -----------                                   -----    ------- ---
    # PF02518.2.ls      Histidine kinase-, DNA gyrase B-, and HSP90   105.6    9.6e-29   1
    # PF00512.1.fs      His Kinase A (phosphoacceptor) domain          55.8    9.2e-14   1
    # PF00672.18.fs     HAMP domain                                    46.9    4.3e-11   1
    # 
    # Parsed for domains:
    # Model             Domain  seq-f seq-t    hmm-f hmm-t      score  E-value
    # --------          ------- ----- -----    ----- -----      -----  -------
    # PF00672.2.ls      1/1     151   219 ..     1    73 []    46.9  4.3e-11
    # PF00512.1.fs      1/1     230   296 ..     1    87 []    55.8  9.2e-14
    # PF02518.18.fs     1/1     338   445 ..     1   127 []   105.6  9.6e-29
    # 
    # Alignments of top-scoring domains:
    # <snip>
    # //
    # 

    if ($_ =~ /^Query sequence:\s+(.*)/) {
      $href->{$hit} = { } unless $href->{$hit};
      $href->{$hit}->{seqid} = $1;
      $inx->{$1} = $fpos;
      $domain = 1;
    }
    elsif ($_ =~ /^Scores for sequence/) {
      $isscores  = 1;
      $isdomains = 0;
    }
    elsif ($isscores && $_ =~ /^([\w\.]+)\s+(.*?)\s+([\-\d\.]+)\s+([^\s]+)\s+\d+/) {
      $hdesc->{$1} = { } unless(ref($hdesc->{$1}) eq 'HASH');
      $hdesc->{$1}->{sdesc} = $self->trim($2);
      $hdesc->{$1}->{seqscore} = $3; #needed for domain vs sequence filtering
      $hdesc->{$1}->{seqeval} = $4; #needed for smart filtering
    }
    elsif ($_ =~ /^Parsed for domains/) {
      $isscores  = 0;
      $isdomains = 1;
    }
    elsif ($isdomains && /^([\w\.]+)\s+\d+\/\d+\s+(\d+)\s+(\d+)\s+[\[\.][\]\.]\s+\d+\s+\d+\s+[\[\.][\]\.]\s+([\-\d\.]+)\s+(.*)/) {
      $href->{$hit}->{$domain} = { } unless $href->{$hit}->{$domain};
      $href->{$hit}->{$domain}->{methid} = $1;
      $href->{$hit}->{$domain}->{desc}   = $hdesc->{$1}->{sdesc} || "no description";
      $href->{$hit}->{$domain}->{start}  = $2;
      $href->{$hit}->{$domain}->{end}    = $3;
      $href->{$hit}->{$domain}->{score}  = $4;
      $href->{$hit}->{$domain}->{evalue} = $5;
      $href->{$hit}->{$domain}->{seqscore} = $hdesc->{$1}->{seqscore};
      $href->{$hit}->{$domain}->{seqeval} = $hdesc->{$1}->{seqeval};
      $domain++;
    }
    elsif ($_ =~ /^\/\//) {
      $href->{$hit}->{ndomains} = $domain - 1;
      $hit++;
      $fpos = tell(FILE);
    }
  }
  
  close FILE;
  $href->{nhits} = $hit - 1;
  
  return (1, '');
}

=head2 getRawResult

 Description: Returns raw formatted result string.
 Arguments:   $href      Reference to a result hash.
              $seqs      Sequence hash (see InterProScan::checkSequences).
              $iprscan   Reference to Dispatcher::Tool::InterProScan object.
                         Required for IPR fields and GO terms.
              $ipr       1: add IPR fields
                         0: no (default)
              $go        1: add GO terms
                         0: no (default)
 Returns:     1, str on success
              0, msg on failure

=cut

sub getRawResult($$;$$$) {

    my ($self, $href, $seqs, $iprscan, $ipr, $go) = @_;
    return (0, "no result hash defined")     unless $href && ref($href) eq 'HASH';
    return (0, "no sequences hash defined") unless $seqs && ref($seqs) eq 'HASH';

    if (!$iprscan && ($ipr || $go)) {
        return (0, "no InterProScan object defined");
    } elsif ($iprscan && ref($iprscan) ne 'Dispatcher::Tool::InterProScan') {
        return (0, "InterProScan object has wrong class: " . ref($iprscan));
    }

    my $nhits = $href->{nhits};
    return (0, "number of hits not defined in result hash") unless defined $nhits;
    return (1, '') if $nhits == 0;

    my $appl = $self->getConfigValue('applname');
    my $date = POSIX::strftime("%d-%b-%Y", localtime);
    my ($hit, $domain, $model, $raw, $seqid, $seqcrc, $seqlen, $res, $msg, $iprac, $iprname, $goterms, $modelid);

    if ($appl =~ /gene3d/i) { # If gene3D, want to do post-processing with DomainFinder
        require Dispatcher::Tool::Hmmer::Gene3DPostFilter;

        my($gpf) = Dispatcher::Tool::Hmmer::Gene3DPostFilter->new();

        my($res, $msg) = $gpf->G3D_post_processing($href);
        return(0, $msg) unless($res);

        $href = $msg;
    }

    for (my $i = 1; $i <= $nhits; $i++) {
        $hit = $href->{$i};
        next unless $hit;  # should not happen

        $seqid = $hit->{seqid};
        next unless $seqid;  # should not happen

        $seqcrc = $seqs->{$seqid}->{crc64};
        $seqlen = $seqs->{$seqid}->{len};

        my $ndomains = $hit->{ndomains};

        if($appl =~ /hmmpfam/i){
            require Dispatcher::Tool::Hmmer::PfamPostFilter; # we need to do Pfam post-filtering

            my($ppf) = Dispatcher::Tool::Hmmer::PfamPostFilter->new();

            ($res, $msg) = $ppf->doLSvsFS($hit);
            return (0, $msg) unless ($res);
            $hit = $msg;

            ($res, $msg) = $ppf->removeOverlap($hit);
            return (0, $msg) unless ($res);
            $hit = $msg;
        }

        if($appl =~ /hmmsmart/i) {

            my $file = $iprscan->getConfigValue('toolinput');
            return (0, "getRawResult: unable to determine file name: $file") unless ($file);
            return (0, "getRawResult: file $file is of zero size") unless (-s $file);

	    my $smd = $self->getConfigValue('smart.desc');
	    return (0, "getRawResult: unable to determine smart.desc filename: $smd") unless ($smd);
	    my $smt = $self->getConfigValue('smart.thresholds');
	    return (0, "getRawResult: unable to determine smart.thresholds filename: $smt") unless ($smt);
	    my $smo = $self->getConfigValue('smart.overlaps');
	    return (0, "getRawResult: unable to determine smart.overlaps filename: $smo") unless ($smo);

	    require Dispatcher::Tool::Hmmer::SmartPostFilter;
	    my ($spf) = Dispatcher::Tool::Hmmer::SmartPostFilter->new();
														     
	    my $cur_dbz = $self->getConfigValue('zvalue'); # current database size; retrieved from param config file (-Z)
														     
	    if (-s $smd && -s $smt && -s $smo) { # files must exist to be able to do proprietary Smart Processing...
		($res, $msg) = $spf->doSmartFilter($hit,$cur_dbz);
		return (0, $msg) unless ($res);
		$hit = $msg;
														     
		($res, $msg) = $spf->removeOverlaps($hit);
		return (0, $msg) unless ($res);
		$hit = $msg;

	    }
														     
	    # Do this regardless of whether or not they have the files
            ($res, $msg) = $spf->checkSmartKinase($hit,$seqid,$file);
            return(0, $msg) unless($res);
            $hit = $msg;
        }


        for (my $j = 1; $j <= $ndomains; $j++) {
            $domain = $hit->{$j};
            next unless $domain;  # should not happen
            next if ($domain->{status} eq "F" && $appl=~/hmm/i); # don't want to output false results (e.g. with hmmsmart)

	    # Gene3D version 3.0.0 (July 2006).  Accession numbers should still consist solely of numbers.  However,
	    # InterPro currently requires that they start with "G3DSA:" so that they can easily be distinguished from
	    # other matches.  As soon as identifiers in the HMM file change to encorporate this, the code below will be
	    # redundant and it will be possible to remove it.
            if($appl =~ /gene3d/i){
                unless($domain->{methid} =~ /^G3DSA/){
                    $domain->{methid} = 'G3DSA:' . $domain->{methid};
                }
            }

            if($appl =~ /hmmpfam/i){
                $domain->{methid} =~ s/\.\S+$//; # remove the version and model type for output
            }

            $raw .= sprintf("%s\t%s\t%s\t", $seqid, $seqcrc, $seqlen);
            $raw .= sprintf("%s\t%s\t%s\t", $appl, $domain->{methid}, $domain->{desc});
            $raw .= sprintf("%d\t%d\t", $domain->{start}, $domain->{end});
            $raw .= sprintf("%s\t%s\t", $domain->{evalue}, $domain->{status} ? $domain->{status} : 'T');
            $raw .= sprintf("%s", $date);

            if ($ipr) {
                $modelid = $domain->{methid};

                ($res, $msg) = $self->filterID($modelid, lc($appl));
                return(0, $msg) unless $res;

                $modelid = $msg;

                ($res, $iprac, $iprname, $goterms) = $iprscan->getInterProFields($modelid, $go);
                return (0, $iprac) unless $res;

                $raw .= sprintf("\t$iprac\t$iprname") if $ipr;
                $raw .= sprintf("\t$goterms") if $go;
            }
            $raw .= "\n";
        }
    }

    return (1, $raw);
}

=head2 filterID

       Description: Filter out the id depending on the application rule. Pfam uses version number at the 
                    end of the accession number like (PFXXXXX.4). And from v4.2 we have added FS to the LS
                    models (using .ls and .fs to distinguish). So '.4.ls' breaks the link to pfam website.
       Arguments:   $id The id to filer out.
		    $appl Application name to know how to filter the id.
       Returns:     1, filter id on success
		    0, msg on error

=cut

sub filterID ($$) {

    my($self, $id, $appl) = @_;

    return(0, "filterID: no id provided.") unless ($id && $id ne '');
    return(0, "filterID: no application provided.") unless($appl && $appl ne '');

    #Put here your regular expression to treat the id depending on the application name.
    my $rules = {
		 hmmpfam => {
			     from => '\.\S+$',
                             to   => ""
                            },
		};

    unless(exists($rules->{$appl})){
        return (1, $id); # no changes on the id.
    }
    my $from = $rules->{$appl}->{from};
    my $to   = $rules->{$appl}->{to};

    $id =~ s/$from/$to/;

    return(1, $id);
}

1;
__END__

