=head1 NAME

Gene3DPostFilter - Gene3D postfiltering utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::Hmmer::Gene3DPostFilter;

=head1 DESCRIPTION

=head1 VERSIONS

$Id: Gene3DPostFilter.pm,v 1.4 2007/05/09 10:47:14 hunter Exp $

Copyright (c) European Bioinformatics Institute 2005

=head1 AUTHORS / ACKNOWLEDGEMENTS

Emmanuel Quevillon <tuco@ebi.ac.uk>
Mark Dibley <dibley@biochemistry.ucl.ac.uk>

=cut


package Dispatcher::Tool::Hmmer::Gene3DPostFilter;


BEGIN {
    unless($ENV{IPRSCAN_HOME}){
	print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
	exit();
    }
    unless($ENV{IPRSCAN_LIB}){
	$ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
    }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool::Hmmer;
@ISA = qw( Dispatcher::Tool::Hmmer );
use Data::Dumper;
use English;
use strict;


#############################
# REQUIRED GLOBAL VARIABLES #
#############################
# These hashes are used to translate the keys between IPS and DF
# Any keys that are not included will be directly translated between IPS and DF

my $hKeyIPS2DF = {"start"=>"segment_start",
		  "end"=>"segment_stop",
		  "methid"=>"match_id",
		  "evalue"=>"score",
		  "score"=>"IPSscore"};

my $hKeyDF2IPS = {"segment_start"=>"start",
		  "segment_stop"=>"end",
		  "match_id"=>"methid",
		  "score"=>"evalue",
		  "IPSscore"=>"score"};

# Global variables used by DomainFinder
my $shortest_segment_cutoff = 10;    # shortest domain allowed
my $cutoff_percent = 1;              # overlap allowed between domains
my $cutoff_residue = 0;
my $evalue_cutoff = 0.001;               # maximum allowable evalue
my $length_cutoff = 50;              # not used for HMMer

###############
# SUBROUTINES #
###############

=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   none
 Returns:     Blessed reference

=cut

sub new (){

    my($class) = @_;

    my ($self) = { };

    bless($self, $class);

    return $self;

}

# POSTPROCESSING OF INTERPROSCAN HMM RESULTS BY GENE3D DOMAINFINDER
# Requires :-
# G3D_post_processing           - main control function for post-processing InterProScan Gene3D results
# ipscan_2_df(%)                - takes a InterProScan hmmscore hash slice and returns a hash for DomainFinder
# sort_df(%);					- sorts the DomainFinder hash into ascending evalue order and returns it - helper for DomainFinder
# DomainFinder(%)               - takes a DomainFinder hash and starts working through hits - returns DomainFinder chain and segment hash
# identify_clashes($$$$%%)      - helper for DomainFinder
# df_2_ipscan(%%)               - takes DomainFinder chain and segment hash and converts them back to a IPS hash slice
# Written by: Mark Dibley, Dave Lee
# Date      : 20-5-2005,   2003

=head2 G3D_post_processing

 Description: Main control function for post-processing InterProScan Gene3D results
 Arguments:   Gene3D parsed results hash table
 Returns:     $hIPS

=cut

sub G3D_post_processing(%){

	my($self, $hIPS) = @_;
	
	# Work through each hit in the IPS hash
	foreach(my $hit = 1; $hit <= $hIPS->{"nhits"}; $hit++){
	
		my($hChain, $hSeg);
		
		#verbose("\n$hit   $hIPS->{$hit}->{'seqid'}   $hIPS->{$hit}->{'ndomains'}"); # Remove after testing
		
		# do not process any non-hits
		next if($hIPS->{$hit}->{'ndomains'} == 0);
		my($res);
		# Convert from IPS to DF
		($res, $hChain)    = $self->ipscan_2_df($hIPS->{$hit});
		return(0, $hChain) unless($res);

		# DomainFinder
		($hChain, $hSeg)   = $self->DomainFinder($hChain);
	
		# Convert from DF to IPS
		$hIPS->{$hit}      = $self->df_2_ipscan($hChain, $hSeg);
	}
	
	return (1, $hIPS);
}


=head2 iprscan_2_df

 Description: Convert InterProScan hmmscore hash to a DomainFinder-readable hash
 Arguments:   $hIPS Domains parsed results
 Returns:     1, Converted hash domain parsed results
              0, error on error
=cut

sub ipscan_2_df(%){

	my($self, $hIPS) = @_;
	
	my $hChain = {};

	for(my $ndomain = 1; $ndomain <= $hIPS->{"ndomains"}; $ndomain++){
	
		# Make DomainFinder hash - hmmscore does not do segments so all segments = 1
		# Use dynamic key exchange - key hash has defined key exchange
		# All other keys can use original IPS names
		foreach my $IPSkey (keys(%{$hIPS->{$ndomain}})){
			
			# If key is defined as a special key for exchange then give Chain hash relevant key
			if(exists $hKeyIPS2DF->{$IPSkey}){
		
				$hChain->{$ndomain}->{1}->{$hKeyIPS2DF->{$IPSkey}} = $hIPS->{$ndomain}->{$IPSkey};
			}
			# If the key is a non-special key and doesn't clash with the Chain hash keys
			elsif(! exists $hKeyDF2IPS->{$IPSkey}){
			
				$hChain->{$ndomain}->{1}->{$IPSkey} = $hIPS->{$ndomain}->{$IPSkey};
			}
			# IPS and DF use the same key names that are not dealt with by the key hashes - add a conversion to $hKeyIPS2DF an $hKeyDF2IPS
			else{
			    return(0, "Error: Clash of keys!!!\n");
			}
		}

		$hChain->{$ndomain}->{1}->{"chain_id"} = $hIPS->{"seqid"};
		
		#verbose(sprintf("%2d   %2d   %4d  %4d  %8s  %22s", 
		#$ndomain, 1, $hChain->{$ndomain}->{1}->{"segment_start"}, $hChain->{$ndomain}->{1}->{"segment_stop"}, 
		#$hChain->{$ndomain}->{1}->{"score"}, $hChain->{$ndomain}->{1}->{"match_id"}));                        # Remove after testing
	}
	
	return (1, $hChain);
}

=head2 sort_df

 Description: Sorting DomainFinder into evalue ascending order
 Arguments:   $hChain Chain to sort
 Returns:     Sorted hash table

=cut

sub sort_df(%){

	my($self, $hChain) = @_;
	
	# Make a hash keyed by the evalues from the DF hash for sorting
	my $hEvalues = {};
	foreach my $ndomain(keys(%{$hChain})){

		# CALCULATE DOMAIN LENGTH
		my $length = $hChain->{$ndomain}->{1}->{"segment_stop"} - $hChain->{$ndomain}->{1}->{"segment_start"} + 1;
		
		# ONLY KEEP DOMAINS THAT ARE LONG ENOUGH AND SCORE BETTER THAN THE GIVEN EVALUE
		if($length >= $shortest_segment_cutoff && $hChain->{$ndomain}->{1}->{"score"} <= $evalue_cutoff){
		
			$hEvalues->{$hChain->{$ndomain}->{1}->{"score"}}->{$length} = $ndomain;
		}
	}


	# Sort hash evalues
	my $order_number = 0;
	my $hSortedChain = {};
	# SORT BY ASCENDING E-VALUE - BEST EVALUE FIRST
	foreach my $evalue (sort {$a<=>$b} keys (%{$hEvalues})){

		# SORT BY ASCENDING LENGTH - SHORTEST LENGTH FIRST
		foreach my $length (sort {$a<=>$b} keys (%{$hEvalues->{$evalue}})){

		        $order_number++;
		
			my $orig_ndomain = $hEvalues->{$evalue}->{$length};
		
			# Make DomainFinder hash - hmmscore does not do segments so all segments = 1
			$hSortedChain->{$order_number} = $hChain->{$orig_ndomain};
		}
	}

	# Return DomainFinder hash
	return $hSortedChain;
}

=head2 DomainFinder

 Description: Search for best Gene3D domain
 Arguments:   $hChain Sorted hash table
 Returns:     $hChain with infos about Gene3D chain
              $hSeg info with start and end matches positions

=cut

sub DomainFinder(%){
	
	my ($self, $hChain) = @_;
	my $hSeg = {};

	# Sort Chain hash
	$hChain = $self->sort_df($hChain);

	# Loop through array in order of best scoring hits
	foreach my $order_number (sort {$a<=>$b} keys(%{$hChain})){
	
		# Get each segment from that hit
		foreach my $segment_num (sort keys(%{$hChain->{$order_number}})){

			# For where entries have been deleted but still exist in the loop memory go on to the next
			next if(!exists $hChain->{$order_number} && !exists $hChain->{$order_number}->{$segment_num} && !exists $hChain->{$order_number}->{$segment_num}->{"segment_start"});

			# Get starts and stops
			my $segment_start = $hChain->{$order_number}->{$segment_num}->{"segment_start"};
			my $segment_stop  = $hChain->{$order_number}->{$segment_num}->{"segment_stop"};

			# Identify clashes
			($hChain, $hSeg) = $self->identify_clashes($segment_start, $segment_stop, $order_number, $segment_num, $hChain, $hSeg);
		}
	}
	
	return ($hChain, $hSeg);
}


=head2 identify_clashes

 Description: Identify clashes between hits and remove wrong one.
 Arguments:   $sacred_start  Start of the match
              $sacred_stop   End of the match
              $sacred_order_number Ordered number
              $sacred_segment_number  Segment number
              $hChain Hash table for this match with infos in it
              $hSeg   Hash table for this segment with positions

 Returns:     $hChain & $hSeg

=cut


sub identify_clashes($$$$%%){

	my($self, $sacred_start, $sacred_stop, $sacred_order_number, $sacred_segment_number, $hChain, $hSeg) = @_;

	# Skip previous segment backclashed sacred
	return($hChain, $hSeg) if($hChain->{$sacred_order_number} eq "");
	
	# Calculate length of segment
	my $sacred_segment_length = ($sacred_stop - $sacred_start) + 1;
			
	# Ignore sacred segment if it isn't long enough to be included
	if($sacred_segment_length <= $shortest_segment_cutoff){
		
		#verbose("$sacred_order_number:$sacred_segment_number -> sacred too short"); # Remove after testing
		$hChain->{$sacred_order_number}->{$sacred_segment_number}->{"ignore"} = "yes";
	}
	# Work through hash to compare the segment sent with all the other segments matched against the chain
	else{
		
		#verbose("SACRED $sacred_order_number:$sacred_segment_number"); # Remove after testing
		NEXTHIT:foreach my $order_number (sort {$a <=> $b} keys(%{$hChain})){
		
			# Don't against self and hit hasn't been backclashed
			if($order_number ne $sacred_order_number){
		
				# Get each segment
				foreach my $segment_number (sort keys(%{$hChain->{$order_number}})){
					
					my $segment_start = $hChain->{$order_number}->{$segment_number}->{"segment_start"};
					my $segment_stop  = $hChain->{$order_number}->{$segment_number}->{"segment_stop"};
					my $segment_length = $segment_stop - $segment_start;
					
					
					# Ignore if previously ignored
					next if(exists $hChain->{$order_number}->{$segment_number}->{"ignore"} &&
					    $hChain->{$order_number}->{$segment_number}->{"ignore"} eq "yes");
						
						
					# Ignore segment if segment is small enough to be ignored
					if($segment_length <= $shortest_segment_cutoff){
			
						#verbose("$order_number:$segment_number -> ignore short seg"); # Remove after testing
						$hChain->{$order_number}->{$segment_number}->{"ignore"} = "yes";
						next;
					}
	
					# Calculate how far the sacred segment can overlap with segment being looked at in hash loop
					# using "!=0" allows negative values to be used (which could be used for padding between hits)

					my $cutoff = 0; # it's zero by default

					if ($cutoff_percent != 0) { # cutoff_percent is non-zero
					    $cutoff = (($cutoff_percent/100) * $sacred_segment_length);
					} elsif ($cutoff_residue != 0) { # cutoff_residue is non-zero
					    $cutoff = $cutoff_residue;
					}

					# Identify clashes
					# ##################  SEGMENT INSIDE SACRED SEGMENT  ####################### || 
					# ########  SEGMENT OVERLAPS SACRED END TOO MUCH AND IS NOT INSIDE  ######## || 
					# #####  SEGMENT END OVERLAPS SACRED START TOO MUCH AND IS NOT INSIDE  ##### || 
					# ###################  SEGMENT COMPLETELY COVERS SACRED  ###################

					if(
					($segment_start >= $sacred_start && $segment_stop <= $sacred_stop) || 
					($segment_start <= ($sacred_stop - $cutoff) && $segment_stop >= $sacred_stop) || 
					($segment_stop >= ($sacred_start + $cutoff) && $segment_start <= $sacred_start) || 
					($segment_start <= $sacred_start && $segment_stop >= $sacred_stop)
					) {
						
						# Sacred can't delete better score match just because it is shorter and its overlap >= 30% 
						if($sacred_order_number > $order_number){
						
							#verbose("$order_number:$segment_number ->  backclash <- $sacred_order_number:$sacred_segment_number"); # Remove after testing
							delete $hChain->{$sacred_order_number};
							return ($hChain, $hSeg);
						}
						# Delete clash hit
						else{
						
							#verbose("$order_number:$segment_number -> clash"); # Remove after testing
							delete $hChain->{$order_number};
							next NEXTHIT;
						}
					}
					
					#verbose("$order_number:$segment_number -> didn't overlap");
				}
			}
			else{
			
				next NEXTHIT;
			}
		}
	}	


	# Where previous segments, especially segment number 1, have been ignored	
	my $working_segment_number = 1;
	for(my $loop = 1; $loop < $sacred_segment_number; $loop++){

	 	if(exists $hChain->{$sacred_order_number}->{$loop}->{"ignore"}){
	 		$working_segment_number++;
	 	}
	} 


	# Make segment (1) start hash for writing out results
	if($sacred_segment_number == 1 || $working_segment_number == $sacred_segment_number){

	 	#verbose("keep $sacred_order_number $sacred_segment_number  $working_segment_number  $sacred_start"); # Remove after testing
	 	$hSeg->{"domain_order"}->{$sacred_start}->{"order_number"} = $sacred_order_number;
	 	$hSeg->{"domain_order"}->{$sacred_start}->{"segment_number"} = $sacred_segment_number;
	}
   

	$hSeg->{"segment_order"}->{$sacred_start}->{"order_number"}   = $sacred_order_number;
	$hSeg->{"segment_order"}->{$sacred_start}->{"segment_number"} = $sacred_segment_number;

	# Finished identifying clashes for this segment
	return ($hChain, $hSeg);
}


###################################################################
# Trim IPS hash to show only the domains approved by DomainFinder #
###################################################################
sub df_2_ipscan(%%){

	my ($self, $hChain, $hSeg) = @_;
	
	# Loop through Segment in hash in the order that the 1st segments of hits appear on the chain
	my($order_number, $segment_number);
	my $domain_number = 0;
	my $hIPS;

	foreach my $ordered_segment_start (sort {$a<=>$b} keys(%{$hSeg->{"domain_order"}})){
	
		$order_number   = $hSeg->{"domain_order"}->{$ordered_segment_start}->{"order_number"};
		$segment_number = $hSeg->{"domain_order"}->{$ordered_segment_start}->{"segment_number"};
		
			
		next if(exists $hChain->{$order_number} &&
				exists $hChain->{$order_number}->{$segment_number} &&
				exists $hChain->{$order_number}->{$segment_number}->{"ignore"});

		# backclash hits will still have hSeg entries but no hChain entries
		next if(!defined $hChain->{$order_number});

		# New Domain
		$domain_number++;
		
		# Make IPS hash
		foreach my $DFkey (keys(%{$hChain->{$order_number}->{$segment_number}})){
		
			# Special cases of settin seqid - only once
			if($DFkey eq "chain_id" && ! exists $hIPS->{"seqid"}){
			
				$hIPS->{"seqid"}  = $hChain->{$order_number}->{$segment_number}->{"chain_id"};
			}
			# Convert between DF and IPS special keys
			elsif(exists $hKeyDF2IPS->{$DFkey}){
			
				$hIPS->{$domain_number}->{$hKeyDF2IPS->{$DFkey}} = $hChain->{$order_number}->{$segment_number}->{$DFkey};
			}
			# Inherit DFkeys which were directly inherited from IPS
			else{
			
				$hIPS->{$domain_number}->{$DFkey} = $hChain->{$order_number}->{$segment_number}->{$DFkey};
			}
		}
	}
	
	# Finish IPS hash
	$hIPS->{"ndomains"} = $domain_number;
	
	# Return IPS hash
	return $hIPS;
}

1;
