=head1 NAME

SmartPostFilter - Smart postfiltering utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::Hmmer::SmartPostFilter;

  #replaces subroutines/methods in Dispatcher::Tool::Hmmer

=head1 DESCRIPTION

=head1 VERSIONS

$Id: SmartPostFilter.pm,v 1.1 2006/09/22 12:35:03 hunter Exp $

Copyright (c) European Bioinformatics Institute 2005

=head1 AUTHORS / ACKNOWLEDGEMENTS

Sarah Hunter <hunter@ebi.ac.uk>

=cut


package Dispatcher::Tool::Hmmer::SmartPostFilter;


BEGIN {
    unless($ENV{IPRSCAN_HOME}){
	print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
	exit();
    }
    unless($ENV{IPRSCAN_LIB}){
	$ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
    }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool::Hmmer;
@ISA = qw( Dispatcher::Tool::Hmmer );
use Data::Dumper;
use English;
use strict;


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   none
 Returns:     Blessed reference

=cut

sub new (){

    my($class) = @_;

    my ($self) = { };

    bless($self, $class);

    return $self;

}

=head2 doSmartFilter
                                                                                                                                                             
 Description: Perform filtering on Smart results and e-value. Need smart.thresholds & smart.desc files (not public)
 Argument:  $hash - the hash of hits for the protein
	    $cur_dbz - the current Z-value for the database
 Returns:     1 on success
                                                                                                                                                             
=cut
                                                                                                                                                             
sub doSmartFilter($$) {

    my($self, $hash, $cur_dbz) = @_;
                                                                                                                                                             
    require Index::SmartThresholds;
                                                                                                                                                             
    return(0, "doSmartFilter: Need a reference to a hash") unless($hash && ref($hash) eq 'HASH');

    # Initialise some variables
    my ($res, $msg);
    my $ndomains = $hash->{ndomains}; # get the number of hits to this sequence
    my ($smartT, $smartD, $smartO); # Smart objects 
    my ($smtr_pos, $smdsc_pos, $smol_pos, $T_entry, $O_entry); # cut-offs, etc
    my ($methid, $dom_cut, $fam_cut, $dbz, $repeat_cut, $num_rpts, $family, $domain);
    my %rpt_min; # hash to store repeat min numbers...
    my %rpt_cnt; # hash to count how many times a repeat happens


    for (my $i = 1; $i <=$ndomains; $i++) { # For each domain hitting the sequence

        # First, initiate new SmartThreshold object for the method
        ($res, $smartT) = Index::SmartThresholds->new(undef, 1);
        return(0, $smartT) unless $res;
                                                                                                                                                             
        # Need to get some variables from our results hash for use in processing and data retrieval
	$methid = $hash->{$i}->{methid}; # get the method ID e.g. SM000123
        my $seq_Eval = $hash->{$i}->{seqeval}; # get the whole seq evalue for the hit vs our query sequence
	
        # Check if the method has any values in the THRESHOLDS file.  
        ($res, $smtr_pos) = $smartT->getIndex($methid);
        next unless(defined($smtr_pos));

        #If so, retrieve the entry 
        ($res, $T_entry) = $smartT->getEntry($methid);
        return (0, "doSmartFilter: error getting T_entry from thresholds file") unless $res;
        $T_entry = $T_entry->[0] if($T_entry && ref($T_entry) eq 'ARRAY'); # returned array ref only has one entry because thresholds (should be) unique

        # Now retrieve all the values we need for the next section (comparisons and filtering)

        # DB size (for scaling)
        ($res, $dbz) = $smartT->getField('dbsize', \$T_entry);
        $dbz = $dbz->[0] if($dbz);
        return(0, "doSmartFilter: error setting dbz: $dbz") unless $res;

        # Thresholds
        ($res, $dom_cut) = $smartT->getField('cut_tp', \$T_entry);
        $dom_cut = $dom_cut->[0] if($dom_cut);
        return (0, "doSmartFilter: error setting dom_cut: $dom_cut") unless $res;
                                                                                                                                                             
        ($res, $fam_cut) = $smartT->getField('cut_fam', \$T_entry);
        $fam_cut = $fam_cut->[0] if($fam_cut);
        return (0, "doSmartFilter: error setting fam_cut: $fam_cut") unless $res;
                                                                                                                                                             
        ($res, $repeat_cut) = $smartT->getField('repeats_cut', \$T_entry);
        $repeat_cut = $repeat_cut->[0] if($repeat_cut);
        return (0, "doSmartFilter: error setting repeat_cut: $repeat_cut") unless $res;

        # Re-scale Thresholds
        $dom_cut = $cur_dbz * $dom_cut / $dbz;
        $fam_cut = $cur_dbz * $fam_cut / $dbz unless ($fam_cut eq '-');
        $repeat_cut = $cur_dbz * $repeat_cut / $dbz unless ($repeat_cut eq '-');

        # Other data from the file...
        ($res, $num_rpts) = $smartT->getField('repeats', \$T_entry);
        $num_rpts = $num_rpts->[0] if ($num_rpts);
        return (0, "doSmartFilter: error setting num repeats: $num_rpts") unless $res;
	$rpt_min{$methid}=$num_rpts; # Store this for later use

        ($res, $family) = $smartT->getField('famid', \$T_entry);
        $family = $family->[0] if ($family);
        return (0, "doSmartFilter: error setting family name: $family") unless $res;

        ($res, $domain) = $smartT->getField('domain', \$T_entry); # (domain will be used as the description too...)
        $domain = $domain->[0] if ($domain);
        return (0, "doSmartFilter: error setting domain name: $domain") unless $res;
                                                                                                                                                             
        # Hits processing (e-value)
        #=================

        # First of all, check if your current hit ($i) has a significant e-value

        if ($seq_Eval > $dom_cut) { # is the whole seq eval greater than domain cut-off? 
            $hash->{$i}->{status}="F"; # if so, the hit is false
            next;
        } else {
            my $dom_Eval = $hash->{$i}->{evalue}; # get the single domain evalue

	    if ($repeat_cut && $repeat_cut ne '-') { # Check if there is a repeat cut off for the method
		if ($dom_Eval > $repeat_cut) { # Is the domain eval greater than repeat cutoff?  
		    $hash->{$i}->{status}="F"; # if so, the hit is false
		    next;
		} else {
		    # accept the hit (ensure status = "T") for comparison later
		    $hash->{$i}->{status}="T"; 
		    $rpt_cnt{$methid}++; # Increment the counter for the number of times this repeat hits
		}
	    }

	    if ($fam_cut && $fam_cut ne "-") { # Or is the hit part of a family and has siblings 
		if ($dom_Eval > $dom_cut) { # the domain evalue is worse than the cut-off
		    $hash->{$i}->{status}="F"; # Hit is false
		    next;
		} else { 
		    $hash->{$i}->{status}="T";
		    $rpt_cnt{$methid}++; # increment the counter for the number of times the sig hits
		} # END if/else ($dom_Eval...
	    } else {
		$hash->{$i}->{status}="T";
		$rpt_cnt{$methid}++; # increment the counter for the number of times the sig hits
	    } # END elsif ($fam_cut ne...
	} # END if/else ($seq_Eval...

	$hash->{$i}->{desc}="$domain"; # set description to be the same as Smart domain name
    } # END for (my $i=...

    # Check repeats... 
    # Finally, go through all domains (that are still true), and mark them as false if 
    # they have the same method id as the repeat and the number of
    # repeats does not reach the minimum required
    for(my $r=1; $r<=$ndomains; $r++) { # Go through the hits again and check which have a minimum rpts required

	unless ($hash->{$r}->{status} eq "F") { # don't bother though, if it's already been flagged as false!
	    $methid=$hash->{$r}->{methid};
	    if ($rpt_min{$methid}) { # the method is a repeat
		if ($rpt_cnt{$methid}>=$rpt_min{$methid}) { # The number of repeats seen for this method is equal to or more than the min required
		    $hash->{$r}->{status}="T";
		} else {
		    $hash->{$r}->{status}="F";
		}
	    }
	}
    }

    return (1, $hash);

}

=head2 removeOverlaps

 Description:	checks all significant hits to the sequence and, if they overlap and are 
		siblings (belong to the same family according to the THRESHOLDS file), they
		are merged/split according to the OVERLAPS file.
 Arguments:		hashref of hits for the protein (N.B. modified so that insignificant hits flagged as "F")
 Returns:		1, $hash on success
			0, msg if error

=cut

sub removeOverlaps ($) {

    my ($self, $hash) = @_;
                                                                                                                                                             
    require Index::SmartOverlaps;
    require Index::SmartThresholds;
						     
    return(0, "removeOverlaps: Need a reference to a hash") unless($hash && ref($hash) eq 'HASH');


    # Initialise some variables
    my ($res, $msg);
    my $ndomains = $hash->{ndomains}; # get the number of hits to this sequence
    my ($smartO); # Smart object
    my ($smol_pos_i, $smol_pos_j, $O_entry_i, $O_entry_j);

    for (my $i = 1; $i <=$ndomains; $i++) { # For each domain hitting the sequence

	next if ($hash->{$i}->{status} eq "F"); # we don't want to bother with hits that don't fulfil the cut-off criteria from "doSmartFilter"

	for (my $j=$i+1; $j <=$ndomains; $j++) { # look at the next domain hitting the sequence...

	    next if ($hash->{$j}->{status} eq "F"); # (don't bother if hit flagged as false, as before)

	    # If both hits are true, calculate how much they overlap by (if at all)
	    my ($res, $overlap) = $self->overlapsBy($hash,$i,$j);
	    return (0, "removeOverlaps: $overlap") unless $res;

	    # if they overlap by 10 or more aas
	    if ($overlap >= 10) { # hits overlap by more than 10 aa 

		# First, initiate new SmartOverlaps object for the methods
		($res, $smartO) = Index::SmartOverlaps->new(undef, 1);
		return(0, $smartO) unless $res;

		# Check if the methods have any values in the OVERLAPS file
		($res, $smol_pos_i) = $smartO->getIndex($hash->{$i}->{desc});
		next unless(defined($smol_pos_i));
		($res, $smol_pos_j) = $smartO->getIndex($hash->{$j}->{desc});
		next unless(defined($smol_pos_j));

		#If so, retrieve the entry using the domain name (stored as "desc" in the results hash)
		($res, $O_entry_i) = $smartO->getEntry($hash->{$i}->{desc});
		return (0, "removeOverlaps: error getting O_entry_i from overlaps file") unless $res;
		$O_entry_i = $O_entry_i->[0] if ($O_entry_i && ref($O_entry_i) eq 'ARRAY'); 
		($res, $O_entry_j) = $smartO->getEntry($hash->{$j}->{desc});
		return (0, "removeOverlaps: error getting O_entry_j from overlaps file") unless $res;
		$O_entry_j = $O_entry_j->[0] if ($O_entry_j && ref($O_entry_j) eq 'ARRAY'); 

		# Use the retrieved overlap entry to get the priority, overlap type and family name
		my ($family_j, $family_i, $priority_j, $priority_i, $overlap_type);

		($res, $family_j) = $smartO->getField('famid', \$O_entry_j);
		$family_j = $family_j->[0] if ($family_j);
		return (0, "removeOverlaps: error getting family j for overlap checking from overlaps file") unless $res;
		($res, $family_i) = $smartO->getField('famid', \$O_entry_i);
		$family_i = $family_i->[0] if ($family_i);
		return (0, "removeOverlaps: error getting family i for overlap checking from overlaps file") unless $res;

		($res, $overlap_type) = $smartO->getField('type', \$O_entry_i);
		$overlap_type= $overlap_type->[0] if ($overlap_type);
		return (0, "removeOverlaps: error getting overlap type for overlap checking from overlaps file") unless $res;

		($res, $priority_j) = $smartO->getField('priority', \$O_entry_j);
		$priority_j = $priority_j->[0] if ($priority_j);
		return (0, "removeOverlaps: error getting priority j for overlap checking from overlaps file") unless $res;
		($res, $priority_i) = $smartO->getField('priority', \$O_entry_i);
		$priority_i = $priority_i->[0] if ($priority_i);
		return (0, "removeOverlaps: error getting priority i for overlap checking from overlaps file") unless $res;

		# LOGIC:
		# if $i is part of the same family as $j
		if ($family_j && $family_i && ($family_i eq $family_j)) {

		    if ($overlap_type eq "split") { # if resolution type is "split"
			# then mark the hit with the highest priority (ergo lowest number) as true, and the other as false
			if ($priority_i < $priority_j) { # i is a higher priority than j
			    $hash->{$i}->{status} = "T";
			    $hash->{$j}->{status} = "F";
			} else { # j is a higher priority than i
			    $hash->{$j}->{status} = "T";
			    $hash->{$i}->{status} = "F";
			}
		    } elsif ($overlap_type eq "merge") { # else, if the resolution type is "merge"
			# then accept the hit with the best domain eval and replace the method ac with it's merge method ac, rejecting the other

			# Check if the methods have any values in the THRESHOLDS file
			# (we only need to do this once, as they should have the same family, etc. info)
			my ($smartT, $smtr_pos, $T_entry, $new_methid);
			($res, $smartT) = Index::SmartThresholds->new(undef, 1);
			return (0, $smartT) unless $res;

			($res, $T_entry) = $smartT->getEntry($family_i);
			return (0, "removeOverlaps:error getting T_entry from thresholds file") unless $res;
			$T_entry = $T_entry->[0] if ($T_entry && ref($T_entry) eq 'ARRAY');

			# Look up the methid for the domain with the same name as the family
			# (assumed to be the same as the domain with priority "1"
			($res, $new_methid) = $smartT->getField('ac', \$T_entry);
			$new_methid = $new_methid->[0] if ($new_methid && ref($new_methid) eq 'ARRAY');
			return (0, "removeOverlaps: error getting ac from entry") unless $res;


			if ($hash->{$i}->{evalue} < $hash->{$j}->{evalue}) { # i evalue is better (lower) than j dom evalue
			    $hash->{$j}->{status} = "F";
			    $hash->{$i}->{status} = "T";
			    # change to methid for i:
			    $hash->{$i}->{methid}=$new_methid;
			} else { # j has a better dom evalue than i
			    $hash->{$i}->{status} = "F";
			    $hash->{$j}->{status} = "T";
			    # change to methid for j
			    $hash->{$j}->{methid}=$new_methid;
			}
		    } # END if ($overlap_type...
		} # END if ($family_j...
	    } # END if ($overlap... 
	} # END for (my $j...
    } # END for (my $i...

    return (1, $hash);
}


=head2 overlapsBy
                                                                                                                                                             
 Description: returns amount that 2 hits overlap
 Arguments:   hash - the hashref of hits for the protein
              id1 
              id2
 Returns:     1, size of overlap on success 
              0, msg if error

=cut

sub overlapsBy ($$$){

    my ($self, $hash, $id1, $id2) = @_;
    my $overlap;
    return (0, "overlapsBy: need results hash, plus 2 key ids as arguments.") unless ($hash, $id1, $id2);
							 
    my($s1, $e1, $s2, $e2) = ($hash->{$id1}->{start}, $hash->{$id1}->{end}, $hash->{$id2}->{start}, $hash->{$id2}->{end});
							 
    # Rule is, if we have an overlap for two hits, we need to check by how much

    # |=================================================| protein
    #             s1             e1
    #             |--------------|                        hit1
    #
    #                 s2      e2                          overlapping
    #                 |-------|                           hit 2
    #
    #       s2        e2    s2       e2                   overlapping
    #       |---------|     |--------|                    hit 2
    #   
    #    s2       e2             s2     e2                overlapping (1aa)
    #    |--------|              |------|                 hit 2
    #
    # s2    e2                       s2          e2       non-overlapping
    # |-----|                        |-----------|        hit 2

    # Firstly, figure out which one is the earlier hit..
    # Then figure out how big the overlap is.
    # N.B. In the case of no overlaps, number returned is <0

    if ($s1 < $s2) { # hit 1 is earlier on the sequence...
	if ($e1 < $e2) { # if end of hit 1 is earlier than end of hit2
	    $overlap = $e1 - $s2;
	} else {
	    $overlap = $e2 - $s2;
	}
    } else { # hit 2 is later on the sequence...
	if ($e1 < $e2) { # if end of hit 1 is earlier than end of hit2
	    $overlap = $e1 - $s1;
	} else {
	    $overlap = $e2 - $s1;
	}
    }

    return ('1', $overlap);

}

=head2 checkSmartKinase
							 
Description:    If 2 kinases (Ser/Thr and Tyr) are hitting a sequence, check
		whether the sequence contains a prosite-like pattern.  Depending on
		which it contains, you can determine which kinase type it is.
Arguments:      $hash hash reference containing all sequences
		$seqid sequence identifier of current sequence
		$fasta file of sequences
Returns:        1, $hash with kinase removed
		0 msg on error
							 
=cut
							 
sub checkSmartKinase($$$) {
							 
    # FYI ####
    # Tyr Kinase (SM00219)
    # Prosite: [LIVMFYC] - {A} - [HY] - x - D - [LIVMFY] - [RSTAC] - {D} - x - N - [LIVMFYC](3)
    # regexp: /[LIVMFYC][^A][HY]\wD[LIVMFY][RSTAC][^D]\wN[LIVMFYC]{3}/
    # simple pattern: /HRD[LIV][AR]\w\wN/
    #
    # Ser/Thr Kinase (SM00220)
    # Prosite: [LIVMFYC] - x - [HY] - x - D - [LIVMFY] - K - x(2) - N - [LIVMFYCT](3)
    # regexp: /[LIVMFYC]\w[HY]\wD[LIVMFY]K\w{2}N[LIVMFYCT]{3}/
    # simple pattern: /D[LIVM]K\w\wN/

    require Index::Fasta; # For retrieving sequence

    my ($self, $hash, $seqid, $seqfile) = @_;
							 
    return(0, "checkSmartKinase: no list given") unless($hash && $seqid && $seqfile);
    return(0, "checkSmartKinase: need a reference to a hash table") unless(ref($hash) eq 'HASH');
    return(1, $hash) unless(keys %$hash);
							 
    my $ndomains = $hash->{ndomains};
    my ($fastaSeq,$methid,$res,$msg,$fseq_pos,$entry);
							 
    ($res, $fastaSeq) = Index::Fasta->new($seqfile,1);
    return(0, "checkSmartKinase: Cannot create new Fasta Index object: $fastaSeq") unless ($res);
							 
    ($res, $entry) = $fastaSeq->getEntry($seqid);
    $entry = $entry->[0] if ($entry && ref($entry) eq 'ARRAY');
    return(0, "checkSmartKinase: Cannot retrieve fasta entry: $entry") unless ($res);
							 
    $entry =~ s/^>.*\n(\w)/$1/;
    $entry =~ s/\n//g;
							 
    my $seen;
    my %seen;
    my $remove;
							 
    for(my $i = 1 ; $i <= $ndomains; $i++){
	$methid = $hash->{$i}->{methid};
							 
	if (($methid =~/SM00220/i || $methid =~/SM00219/i)  && $hash->{$i}->{status} eq "T") { #It's a kinase (and a true hit)
	    $seen{$methid}=$i; # store position of methid in hit array
	    if ($seen) { # you've already seen another kinase
		# check for which pattern is there
		if ($entry =~/HRD[LIV][AR]\w\wN/i && $entry =~/D[LIVM]K\w\wN/i ) { 
		    # If there is the regexp for Tyrosine kinase and one for Ser/Thr kinase
		    # they are both still true
		} elsif ($entry =~/HRD[LIV][AR]\w\wN/i) { # only tyr kinase regexp hits
		    $remove = $seen{"SM00220"}; # so remove the Ser/Thr hit
		    $hash->{$remove}->{status}="F"; # as $seen stores the "other" kinase hit number
		} elsif ($entry =~/D[LIVM]K\w\wN/i) { # But one for Ser/Thr kinase
		    $remove=$seen{"SM00219"}; # remove the Tyr/Kinase hit
		    $hash->{$remove}->{status}="F"; # as $i is the current hit
		} else { # (or neither!) 
		    $hash->{$seen{"SM00219"}}->{status}="F";
		    $hash->{$seen{"SM00220"}}->{status}="F";
		}
	    }
	} else {
	    $seen = $i;
	} # END if (methid=~...
    } # END for (i=1...

    return(1, $hash);

}

1;
