=head1 NAME

Hmmer3 - Hmmer3 utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::Hmmer3;

=head1 DESCRIPTION

Is created for the HMMER3.0b3 version of the binary (hmmscan)

=head1 VERSIONS

$Id: Hmmer3.pm,v 1.2 2010/10/20 19:03:18 hunter Exp $

Copyright (c) European Bioinformatics Institute 2009

=head1 AUTHORS / ACKNOWLEDGEMENTS

Sarah Hunter <hunter@ebi.ac.uk>

=cut

package Dispatcher::Tool::Hmmer3;

BEGIN {
    unless($ENV{IPRSCAN_HOME}){
	print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
	exit();
    }
    unless($ENV{IPRSCAN_LIB}){
	$ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
    }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool;
@ISA = qw( Dispatcher::Tool );

use Data::Dumper;
use English;
use File::Spec::Functions;
use POSIX qw(strftime);
use Sys::Hostname;
use strict;


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        Hmmer tool name (optional). If given, sets the
                           Dispatcher::Config object automatically inside
                           the object. If not given, the Dispatcher::Config
                           must be set later with setConfig.
              $defaults    Default values for configuration, hash reference
                           (optional).
 Returns:     $self object

=cut

sub new(;$$) {
    my ($class, $name, $defaults) = @_;

    my $self = { };
    bless $self, $class;

    if ($name) {
        $self->setToolName($name);
        my ($res, $msg) = $self->parseConfig(1, $defaults);
        die "failed to parse $name configuration: $msg\n" unless $res;
    }

    return $self;
}


=head2 getCmdLine

 Description: Returns Hmmer3 command line.
 Arguments:   -
 Returns:     1, cmd on success
              0, msg on failure

=cut

sub getCmdLine() {
    my $self = shift;
    return $self->getConfigValue('cmdline');
}


=head2 parseResult

 Description: Reads a Hmmer3 result file and creates a hash table:

              nhits       => '2',
              '1'         => { seqid       => 'Y902_MYCTU',
                               ndomains    => '3',
                               '1'         => { methid => 'PF00672.2.ls',
                                                desc   => 'HAMP domain',
                                                start  => '151',
                                                end    => '219',
                                                score  => '46.9',
                                                evalue => '4.3e-11',
						alignment => 'ABCDE----DJvkkdiIIE'
                                              },
                               '2'         => { methid => 'PF00512.1.ls',
                                                desc   => 'His Kinase A (phosphoacceptor) domain',
                                                start  => '230',
                                                end    => '296',
                                                score  => '55.8',
                                                evalue => '9.2e-14'
                                              },
                               '3'         => { methid => 'PF02518.18.ls',
                                                desc   => 'Histidine kinase-, DNA gyrase B-, and HSP90',
                                                start  => '338',
                                                end    => '445',
                                                score  => '105.6',
                                                evalue => '9.6e-29'
                                              },
                             },
              '2'         => { seqid       => 'WAP_RAT',
                               ...
                             }

 Arguments:   $href          Reference to a hash.
              $result        Path to a Hmmer3 result file.
              $inx           Reference to a hash.
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub parseResult($$) {
   my ($self, $href, $result, $inx) = @_;
   return (0, "no Hmmer input file defined") unless $result;

   $href = { } unless $href && ref($href) eq 'HASH';
   $inx  = { } unless $inx && ref($inx) eq 'HASH';

   unless (open(FILE, "< $result")) {
      return (0, "could not read Hmmer result file $result: $!");
   }

   # Set boolean markers
   my $isscores = 0;
   my $isdomains = 0;
   my $isaligns = 0;
   my $hit = 1;
   my $methid;
   my $domain    = 1; 
   my $hdesc     = { };
   my %dommap; # map methid and hmmer domain num to iprscan domain counter
   my $a_dom = 0; # Store the number of the domain for the method in HMMER
   my $fpos = 0;

   # Read the HMMER3 output file
   while (<FILE>) {
      chomp;

   if (/^#\s+\S+/) { # Ignore HMMER3 header section
      next;
   }

#Sequence info section:
# Query:       tr|Q9RHD9|Q9RHD9_PSEAE  [L=267]
# Description: RpsA (Fragment) OS=Pseudomonas aeruginosa GN=rpsA PE=4 SV=1

      elsif (/^Query:\s+(\S+).*\[L=(\d+)\]/) {
	 $href->{$hit} = { } unless $href->{$hit};
	 $href->{$hit}->{seqid} = $1;
	 $href->{$hit}->{seqlen} = $2;
	 $inx->{$1} = $fpos;
	 $domain = 1;
      }

#Seqscores section:
#Scores for complete sequence (score includes all domains):
#   --- full sequence ---   --- best 1 domain ---    -#dom-
#    E-value  score  bias    E-value  score  bias    exp  N  Model      Description
#    ------- ------ -----    ------- ------ -----   ---- --  --------   -----------
#    1.6e-53  178.4  17.5    4.7e-19   67.9   2.9    3.1  3  PF00575.16 S1 RNA binding domain

      elsif ($_ =~ /^Scores for complete sequence/) {
	 $isscores = 1;
	 $isdomains = 0;
	 $isaligns = 0;
      }

	                       # 1. seqeval  2. seqscore                                          3. methid 4. desc
      elsif ($isscores && $_ =~ /\s+(\S+)\s+([\-\d\.]+)\s+[\-\d\.]+\s+\S+\s+\S+\s+\S+\s+\S+\s+\d+\s+(\S+)\s+(.*)/) {
	    $methid = $3;
	    $hdesc->{$3} = { } unless(ref($hdesc->{$3}) eq 'HASH');
	    $hdesc->{$3}->{seqscore} = $2; #needed for domain vs sequence filtering
	    $hdesc->{$3}->{seqeval} = $1; #needed for smart filtering
	    $hdesc->{$3}->{sdesc} = $4;
      }

#Domainscores and alignment section:
# Domain annotation for each model:

      elsif ($_ =~ /^Domain annotation for each/) {
	 $isscores = 0;
	 $isdomains = 1;
	 $isaligns = 0;
      }

# >> PF00575.16  S1 RNA binding domain
#    #    score  bias  c-Evalue  i-Evalue hmmfrom  hmm to    alifrom  ali to    envfrom  env to     acc
#  ---   ------ ----- --------- --------- ------- -------    ------- -------    ------- -------    ----
#    1 !   53.5   0.5   1.3e-18   1.5e-14      24      74 .]       4      55 ..       3      55 .. 0.96
#    2 !   66.1   0.2   1.5e-22   1.8e-18       2      74 .]      69     142 ..      68     142 .. 0.97
#    3 !   67.9   2.9   3.9e-23   4.7e-19       2      74 .]     156     228 ..     155     228 .. 0.98
#<snip>
#Alignments for each domain:
# == domain 2    score: 86.4 bits;  conditional E-value: 8.4e-28
#                            XXXXXXXXXXXXXXXXXXXXX.XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX RF
#                1go3E01   1 gevveGkVtnvtdfGafVelg.gveGlvhiselsddrveydekkqalvgedpskvlkvGdevevkvlevdkergrisLslkqleeep 86 
#                            g+ + G ++++tdfG+f+ l+ g++Glvh+s++s++ v          ge++ + +k+Gde+e  +l+vd+er+risL++kqle++p
# tr|Q9RHD9|Q9RHD9_PSEAE  72 GDRISGSIKSITDFGIFIGLDgGIDGLVHLSDISWNEV----------GEEAVRRFKKGDELETVILSVDPERERISLGIKQLEDDP 148
#                            78999***************66****************..........9************************************** PP
#
#                            XXXXXX RF
#                1go3E01  87 aeeele 92 
#                            +++ ++
# tr|Q9RHD9|Q9RHD9_PSEAE 149 FSNYAS 154
#                            998766 PP
#<snip>
# >> <etc>

      elsif ($isdomains && $_ =~ /^\>\>\s+(\S+)/) {   # You have reached per-model per-domain section
	    $methid = $1;
      }

	        #           1: dom_num            2:score                  3:dom_iEvalue 4:hmmfrom              5:start  6:end
      elsif ($isdomains && /^\s+(\d+)\s+[\!|\?]\s+([\-\d\.]+)\s+[\d\.]+\s+\S+\s+(\S+)\s+(\d+)\s+\d+\s+[\.|\]|\[]{2}\s+(\d+)\s+(\d+)/) {
	 $href->{$hit}->{$domain}->{methid} = $methid;
	 $href->{$hit}->{$domain}->{score}    = $2;
	 $href->{$hit}->{$domain}->{evalue}   = $3;
	 $href->{$hit}->{$domain}->{hmmf}     = $4;
	 $href->{$hit}->{$domain}->{start}    = $5;
	 $href->{$hit}->{$domain}->{end}      = $6;
	 $href->{$hit}->{$domain}->{desc}   = $hdesc->{$methid}->{sdesc} || "no description";
	 $href->{$hit}->{$domain}->{seqscore} = $hdesc->{$methid}->{seqscore};
	 $href->{$hit}->{$domain}->{seqeval} = $hdesc->{$methid}->{seqeval};
	 $dommap{$methid}{$1} = $domain; # so we know which ipr domain is equivalent to which hmmer domain
	 $domain++;
      }

      # store the alignment info for that domain for that model in a hashref
      elsif ($isdomains && $_ =~/^\s*==\s+domain\s+(\d+)/ ) {
	 $a_dom = $1;
	 $isaligns = 1;
      }

      elsif ($isaligns && $_ =~ /^\s*(\S+)\s+\d+\s+(\S+)\s+\d+/ && $href->{$hit}->{seqid} eq $1) {
	 my $dom = $dommap{$methid}{$a_dom};
	 $href->{$hit}->{$dom}->{alignment} .= $2;
      }

#You've reached the end of the result:
# Internal pipeline statistics summary:
# -------------------------------------
# Query sequence(s):                         1  (267 residues)
# Target model(s):                       10019  (1527317 nodes)
#<snip>

      elsif ($_ =~ /^Internal pipeline/) { # end of the interesting stuff (for us)
	 $isaligns = 0;
	 $isdomains = 0;
	 $isscores = 0;

	 $href->{$hit}->{ndomains} = $domain - 1;
	 $hit++;
	 $fpos = tell(FILE);
      }

   }

   close FILE;
   $href->{nhits} = $hit - 1;

   return (1, '');
}

=head2 getRawResult

 Description: Returns raw formatted result string.
 Arguments:   $href      Reference to a result hash.
              $seqs      Sequence hash (see InterProScan::checkSequences).
              $iprscan   Reference to Dispatcher::Tool::InterProScan object.
                         Required for IPR fields and GO terms.
              $ipr       1: add IPR fields
                         0: no (default)
              $go        1: add GO terms
                         0: no (default)
 Returns:     1, str on success
              0, msg on failure

=cut

sub getRawResult($$;$$$) {

    my ($self, $href, $seqs, $iprscan, $ipr, $go) = @_;
    return (0, "no result hash defined")     unless $href && ref($href) eq 'HASH';
    return (0, "no sequences hash defined") unless $seqs && ref($seqs) eq 'HASH';

    if (!$iprscan && ($ipr || $go)) {
        return (0, "no InterProScan object defined");
    } elsif ($iprscan && ref($iprscan) ne 'Dispatcher::Tool::InterProScan') {
        return (0, "InterProScan object has wrong class: " . ref($iprscan));
    }

    my $nhits = $href->{nhits};
    return (0, "number of hits not defined in result hash") unless defined $nhits;
    return (1, '') if $nhits == 0;

    my $appl = $self->getConfigValue('applname');
    my $date = POSIX::strftime("%d-%b-%Y", localtime);
    my ($hit, $domain, $model, $raw, $seqid, $seqcrc, $seqlen, $ntstart, $ntend, $res, $msg, $iprac, $iprname, $goterms, $modelid);

    for (my $i = 1; $i <= $nhits; $i++) {
        $hit = $href->{$i};
        next unless $hit;  # should not happen

        $seqid = $hit->{seqid};
        next unless $seqid;  # should not happen

        $seqcrc = $seqs->{$seqid}->{crc64};
        $seqlen = $seqs->{$seqid}->{len};
        $ntstart = $seqs->{$seqid}->{ntstart}; # NEW
        $ntend = $seqs->{$seqid}->{ntend}; # NEW


        my $ndomains = $hit->{ndomains};

        if($appl =~ /hmmpfam/i){
            require Dispatcher::Tool::Hmmer::PfamPostFilter; # we need to do Pfam post-filtering

            my($ppf) = Dispatcher::Tool::Hmmer::PfamPostFilter->new();

            ($res, $msg) = $ppf->removeOverlap($hit);
            return (0, $msg) unless ($res);
            $hit = $msg;
        }

        if($appl =~ /hmmsmart/i) {

            my $file = $iprscan->getConfigValue('toolinput');
            return (0, "getRawResult: unable to determine file name: $file") unless ($file);
            return (0, "getRawResult: file $file is of zero size") unless (-s $file);

	    my $smd = $self->getConfigValue('smart.desc');
	    return (0, "getRawResult: unable to determine smart.desc filename: $smd") unless ($smd);
	    my $smt = $self->getConfigValue('smart.thresholds');
	    return (0, "getRawResult: unable to determine smart.thresholds filename: $smt") unless ($smt);
	    my $smo = $self->getConfigValue('smart.overlaps');
	    return (0, "getRawResult: unable to determine smart.overlaps filename: $smo") unless ($smo);

	    require Dispatcher::Tool::Hmmer::SmartPostFilter;
	    my ($spf) = Dispatcher::Tool::Hmmer::SmartPostFilter->new();
														     
	    my $cur_dbz = $self->getConfigValue('zvalue'); # current database size; retrieved from param config file (-Z)
														     
	    if (-s $smd && -s $smt && -s $smo) { # files must exist to be able to do proprietary Smart Processing...
		($res, $msg) = $spf->doSmartFilter($hit,$cur_dbz);
		return (0, $msg) unless ($res);
		$hit = $msg;
														     
		($res, $msg) = $spf->removeOverlaps($hit);
		return (0, $msg) unless ($res);
		$hit = $msg;

	    }
														     
	    # Do this regardless of whether or not they have the files
            ($res, $msg) = $spf->checkSmartKinase($hit,$seqid,$file);
            return(0, $msg) unless($res);
            $hit = $msg;
        }


        for (my $j = 1; $j <= $ndomains; $j++) {
            $domain = $hit->{$j};
            next unless $domain;  # should not happen
            next if ($domain->{status} eq "F" && $appl=~/hmm/i); # don't want to output false results (e.g. with hmmsmart)

            if($appl =~ /hmmpfam/i){
                $domain->{methid} =~ s/\.\S+$//; # remove the version and model type for output
            }

	    if ($ntstart) { # NEW
		if ($ntstart < $ntend ) { # Forward strand
		    $domain->{start} = ($domain->{start} * 3) - 2 + ($ntstart -1);
		    $domain->{end} = ($domain->{end} * 3) + ($ntstart -1);
		} elsif ($ntstart > $ntend) { # Reverse strand
		    $seqid .= "_REV";
		    $domain->{start} = $ntstart - ($domain->{start} * 3) +3;
		    $domain->{end} = $ntstart - ($domain->{end} * 3) +1;
		}
	    }

            $raw .= sprintf("%s\t%s\t%s\t", $seqid, $seqcrc, $seqlen);
            $raw .= sprintf("%s\t%s\t%s\t", $appl, $domain->{methid}, $domain->{desc});
            $raw .= sprintf("%d\t%d\t", $domain->{start}, $domain->{end});
            $raw .= sprintf("%s\t%s\t", $domain->{evalue}, $domain->{status} ? $domain->{status} : 'T');
            $raw .= sprintf("%s", $date);

            if ($ipr) {
                $modelid = $domain->{methid};

                ($res, $msg) = $self->filterID($modelid, lc($appl));
                return(0, $msg) unless $res;

                $modelid = $msg;

                ($res, $iprac, $iprname, $goterms) = $iprscan->getInterProFields($modelid, $go);
                return (0, $iprac) unless $res;

                $raw .= sprintf("\t$iprac\t$iprname") if $ipr;
                $raw .= sprintf("\t$goterms") if $go;
            }
            $raw .= "\n";
        }
    }

    return (1, $raw);
}

=head2 filterID

       Description: Filter out the id depending on the application rule. Pfam uses version number at the 
                    end of the accession number like (PFXXXXX.4). And from v4.2 to v4.5 we have added FS to the LS
                    models (using .ls and .fs to distinguish). So '.4.ls' breaks the link to pfam website.
       Arguments:   $id The id to filer out.
		    $appl Application name to know how to filter the id.
       Returns:     1, filter id on success
		    0, msg on error

=cut

sub filterID ($$) {

    my($self, $id, $appl) = @_;

    return(0, "filterID: no id provided.") unless ($id && $id ne '');
    return(0, "filterID: no application provided.") unless($appl && $appl ne '');

    #Put here your regular expression to treat the id depending on the application name.
    my $rules = {
		 hmmpfam => {
			     from => '\.\S+$',
                             to   => ""
                            },
		};

    unless(exists($rules->{$appl})){
        return (1, $id); # no changes on the id.
    }
    my $from = $rules->{$appl}->{from};
    my $to   = $rules->{$appl}->{to};

    $id =~ s/$from/$to/;

    return(1, $id);
}

1;
__END__

