=head1 NAME
InterProScan - InterProScan utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::InterProScan;

=head1 DESCRIPTION

=head1 VERSIONS

$Id: InterProScan.pm,v 1.17 2011/04/12 18:39:04 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

interhelp@ebi.ac.uk
Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut

BEGIN{
  unless($ENV{IPRSCAN_HOME}){
    print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
    exit;
  }
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

package Dispatcher::Tool::InterProScan;

use lib $ENV{IPRSCAN_LIB};

use Dispatcher::Queue;
use Dispatcher::Tool;
use Dispatcher::Tool::BlastProDom;
use Dispatcher::Tool::Coils;
use Dispatcher::Tool::FingerPrintScan;
use Dispatcher::Tool::Gene3D;
use Dispatcher::Tool::HAMAP;
use Dispatcher::Tool::HMMPanther;
use Dispatcher::Tool::HMMPIR;
use Dispatcher::Tool::Hmmer;
use Dispatcher::Tool::Hmmer3;
use Dispatcher::Tool::ProfileScan;
use Dispatcher::Tool::ScanRegExp;
use Dispatcher::Tool::PatternScan;
use Dispatcher::Tool::Seg;
use Dispatcher::Tool::SignalP;
use Dispatcher::Tool::SuperFamily;
use Dispatcher::Tool::TMHMM;
use Dispatcher::Tool::InterProScanParser;
@ISA = qw( Dispatcher::Tool );

use Index::Fasta;
use Index::IprMatches;
use Index::InterPro;
use Index::Hmm;
use Index::Prints;
use Index::PfamClan;
use Index::PfamA;
use Index::SmartDesc;
use Index::SmartThresholds;
use Index::SmartOverlaps;

use Data::Dumper;
use English;
use File::Basename;
use File::Copy;
use File::Path;
use File::Spec::Functions;
use IO::String;
use Sys::Hostname;
use strict;


# map InterProScan application to dispatcher tool name
my $ApplNameToToolName = {
    blastprodom      => 'blastprodom',
    gene3d           => 'gene3d',
    coil             => 'coils',
    coils            => 'coils',
    fingerprintscan  => 'fpscan',
    fprintscan       => 'fprintscan',
    fpscan           => 'fprintscan',
    hamap            => 'hamap',
    hmmpanther       => 'hmmpanther',
    panther          => 'hmmpanther',
    hmmpfam          => 'hmmpfam',
    hmmpir           => 'hmmpir',
    hmmsmart         => 'hmmsmart',
    hmmtigrfam       => 'hmmtigr',
    hmmtigr          => 'hmmtigr',
    profilescan      => 'profilescan',
    pfscan           => 'profilescan',
    scanregexp       => 'scanregexp',
    patternscan	     => 'patternscan',
    seg              => 'seg',
    signalp          => 'signalp',
    signalphmm       => 'signalp',
    superfamily      => 'superfamily',
    tmhmm            => 'tmhmm'
};


# map submission parameter to description
my $ParamToDescription = {
    appl        => 'Applications',
    crc         => 'Perform CRC64 check',
    email       => 'Email address',
    goterms     => 'Show GO terms',
    interact    => 'Interactive submission',
    iprlookup   => 'InterPro lookup',
    jobid       => 'Job identifier',
    nseqs       => 'Number of input sequences',
    scores      => 'Report scores',
    title       => 'Title',
    trlen       => 'Transcript length threshold',
    trtable     => 'Codon table'
};

#map abbreviated Taxonomy to Taxonomy
my $Abrev2Taxo = {
		  'AraTh'  => 'Arabidopsis thaliana',
		  'Arch'   => 'Archaea',
		  'Arthro' => 'Arthropoda',
		  'Bact'   => 'Bacteria',
		  'CaeEl'  => 'Caenorhabditis elegans',
		  'Chor'   => 'Chordata',
		  'Cyan'   => 'Cyanobacteria',
		  'Euka'   => 'Eukaryota',
		  'FrFly'  => 'Fruit Fly',
		  'Fung'   => 'Fungi',
		  'GrePl'  => 'Green Plants',
		  'Huma'   => 'Human',
		  'Meta'   => 'Metazoa',
		  'Mous'   => 'Mouse',
		  'Nema'   => 'Nematoda',
		  'OthEuk' => 'Other Eukaryotes',
		  'PlasGrp'=> 'Plastid Group',
		  'Rice'   => 'Rice spp',
		  'SacCer' => 'Saccharomyces cerevisiae',
		  'Synec'  => 'Synechocystis PCC 6803',
		  'Unclass'=> 'Unclassified',
		  'Vir'    => 'Virus'
		 };

my $TranslationTableValues = {
    '0'  => 'Standard (0)',
    '2'  => 'Vertebrate Mitochondrial (2)',
    '3'  => 'Yeast Mitochondrial (3)',
    '4'  => 'Mold, Protozoan, Coelenterate Mito. and Myco/Spiroplasma (4)',
    '5'  => 'Invertebrate Mitochondrial (5)',
    '6'  => 'Ciliate Nuclear, Dasycladacean Nuclear, Hexamita Nuclear (6)',
    '9'  => 'Echinoderm Mitochondrial (9)',
    '10' => 'Euploid Nuclear (10)',
    '11' => 'Bacterial (11)',
    '12' => 'Alternative Yeast Nuclear (12)',
    '13' => 'Ascidian Mitochondrial (13)',
    '14' => 'Flatworm Mitochondrial (14)',
    '15' => 'Blepharisma Macronuclear (15)',
    '16' => 'Chlorophycean Mitochondrial (16)',
    '21' => 'Trematode Mitochondrial (21)'
};



my $FrameValues = {
    'all' => '6',
    '1'   => '1',
    '2'   => '2',
    '3'   => '3',
    '4'   => '-1',
    '5'   => '-2',
    '6'   => '-3'
};


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        InterProScan tool name (optional). If given, sets the
                           Dispatcher::Config object automatically inside
                           the object. If not given, the Dispatcher::Config
                           must be set later with setConfig.
              $defaults    Default values for configuration, hash reference
                           (optional).
 Returns:     $self object

=cut

sub new(;$$) {
    my ($class, $name, $defaults) = @_;

    my $self = { param2desc    => $ParamToDescription,
	       };
    bless $self, $class;

    if ($name) {
        $self->setToolName($name);
        my ($res, $msg) = $self->parseConfig(1, $defaults);
        die "failed to parse $name configuration: $msg\n" unless $res;
    }

    return $self;
}


=head2 isInteractive

 Description: Checks whether search is interactive or not.
 Arguments:   -
 Returns:     1 interactive search
              0 otherwise

=cut

sub isInteractive($) {
    my $self = shift;

    my $params = $self->getParams();
    return 0 unless $params;

    my $search = $params->{srchtype};
    return 1 if $search && $search eq 'interactive';
    return 0;
}


=head2 checkParams

 Description: Checks InterProScan input parameters, sets the defaults and writes
              the input sequence(s) to file(s). The input sequence may also
              be given in the input parameters with key 'sequence'.
 Arguments:   -
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub checkParams() {
    my $self = shift;
    my $params = $self->getParams();
    return (0, "checkParams: no input parameters") unless $params;

    my $checked = { };

    $checked->{tool} = $self->getToolName();
    $checked->{jobid} = $self->getConfigValue('jobid');
    $checked->{submission} = $self->checkString($params->{submission}, 'cgi', ['cgi', 'cli', 'api', 'email']);

    my $html = ($checked->{submission} eq 'cgi' ? 1 : 0);

    # check access to this tool
    my ($res, $msg) = $self->hasAccess();
    return (0, "checkParams: no tool access: $msg") unless $res;
    $checked->{remotehost} = $msg;

    my $cmd = $params->{cmd};
    $cmd = 'check' unless $cmd;
    if ($cmd =~ /submit/i) {
	$cmd = 'submit';
    }
    elsif ($cmd =~ /check/i) {
	$cmd = 'check';
    }
    else {
        return (0, "checkParams: unrecognized command '$cmd'");
    }

    if ($cmd eq 'check') {
        my $pfile = $self->findFile('toolparams');
        return (0, "checkParams: configuration error, toolparams not defined") unless $pfile;

	unless (-e $pfile) {
	    return (0, "checkParams: Could not find InterProScan job $checked->{jobid}. " .
		    "Either the job results have been removed or the job identifier " .
		    "is malformed. Sorry for the inconvenience.");
        }

        ($res, $msg) = $self->readParamFile($pfile, $checked);
        return (0, "checkParams: unable to read parameter file $pfile: $msg") unless $res;

        $checked->{cmd} = 'check';

        $checked->{view} = $self->checkString(lc($params->{view}), 'picture', ['picture', 'table']);
        $checked->{imgwidth} = $self->checkInteger($params->{imgwidth}, undef, '100', '5000');
	$checked->{cnk} = $self->checkInteger($params->{cnk}, 1);

        $self->setParams($checked);

        # reload the config in case parameters affect it (e.g., srchtype)
	($res, $msg) = $self->parseConfig(1, $self->getParams(), 1);
        $self->errorExit($msg, undef, 1) unless $res;

        return (1, '');
    }

    $checked->{cmd} = 'submit';

    # validate the search type
    my $search = $params->{srchtype};
    $search = 'interactive' unless $search;
    if (lc($search) eq 'interactive') {
        $checked->{srchtype} = 'interactive';
    }
    elsif (lc($search) eq 'email') {
        $checked->{srchtype} = 'email';
    }
    else {
        return (0, "checkParams: unrecognized search type '$search'");
    }

    $checked->{email} = $params->{email} || $params->{emaddr};

    if ($checked->{srchtype} eq 'email' && (!$checked->{email} || $checked->{email} !~ /@/)) {
        return (0, "checkParams: no valid email address specified");
    }

    my $sapplist = $self->getConfigValue('applications');
    return (0, "checkParams: configuration error, applications not configured") unless $sapplist;

    my @sapps = split(/\,/, $sapplist);
    return (0, "checkParams: configuration error, applications list is empty") unless @sapps > 0;

    # create temporary hash of supported applications for lookups
    my $sahref = { };
    my $sapp;

    foreach $sapp (@sapps) {
        $sapp = lc($sapp);
        $sahref->{$sapp} = 1;
    }

    my $applist = $params->{appl};
    my @apps = split(/\+/, $applist) if $applist;

    if (@apps && @apps > 0) {
        my $ok = 1;
        my ($err, $app);

        # user has requested a subset of supported applications
        foreach $app (@apps) {
	  $app = lc($app);
            unless (exists $sahref->{$app}) {
                $err .= "checkParams: Application $app is not supported.\n";
		$err .= "<br>" if $html;
                $ok = 0;
            }
        }

        unless ($ok) {
	    $err .= "<br>" if $html;
            $err .= "\nSupported applications are:\n";
	    $err .= "<br>" if $html;
	    foreach $app (sort keys %$sahref) {
		$err .= " - $app\n";
		$err .= "<br>" if $html;
	    }
	    $err .= "<br>";
            $err .= "Please check the submission parameters and the InterProScan configuration.<br>\n";

            return (0, "checkParams: application selection error: $err");
        }

        $checked->{appl} = join(',', @apps);
    }
    else {
        # default is all supported applications
        $checked->{appl} = join(',', @sapps);
    }

    $checked->{submission} = $self->checkString($params->{submission}, 'cgi', ['cgi', 'cli', 'email']);

    my $seqtype = $params->{seqtype} || $params->{stype};
    if ($seqtype && lc($seqtype) =~ /^n/) {
      $checked->{seqtype} = 'N';
    }
    elsif($seqtype && lc($seqtype) =~/^p/) {
      $checked->{seqtype} = 'P';
    } else {
      $checked->{seqtype} = 'auto';
    }

    my @trtables = keys %$TranslationTableValues;
    $checked->{trtable} = $self->checkString($params->{trtable}, undef, \@trtables);

    my @frames = keys %$FrameValues;
    my $frame = $self->checkString($params->{frame}, 'all', \@frames);
    $checked->{frame} = $FrameValues->{$frame};
    $checked->{trlen} = $self->checkInteger($params->{trlen}, undef, '20', '150');
    #EQ
    $checked->{format}    = $self->checkString($params->{format}, 'xml', ['ebixml', 'xml', 'txt', 'html', 'htm', 'raw']);
    $checked->{crc}       = $params->{crc};
    $checked->{iprlookup} = $params->{iprlookup};
    $checked->{view}      = $params->{view};
    $checked->{onego}     = $params->{onego};
    ############

    # write the input to two files: original sequences and formatted/translated
    my $seqfile = $self->getConfigValue('uploadfile.1');
    my $dirmode = $self->getConfigValue('usermode') || $self->getConfigValue('dirmode');
    my $orig    = $self->getConfigValue('toolseqs');
    my $input   = $self->getConfigValue('toolinput');

    if ($seqfile && -e $seqfile) {
	# sequence has been written to a temporary file
	unless(open(SEQ, "< $seqfile")) {
	    close SEQ;
	    return (0, "checkParams: could not read input sequences from $seqfile: $!");
	}
	($res, $msg) = $self->formatSequences($input, $orig, undef, \*SEQ,
					      $checked->{seqtype},
					      $checked->{frame},
					      $checked->{trtable},
					      $checked->{trlen},
					      $dirmode);
	close SEQ;
	
	unless ($checked->{submission} eq 'cli') {
	    # don't delete the input file when CLI is used
	    unlink($seqfile);
	}
    }
    else {
	($res, $msg) = $self->formatSequences($input, $orig, $params->{sequence}, undef,
					      $checked->{seqtype},
					      $checked->{frame},
					      $checked->{trtable},
					      $checked->{trlen},
					      $dirmode);
    }
    return (0, "checkParams: unable to create input files: $msg") unless $res;
    #We reset it in case user made a mistake, formatSequences automatically determines it.
    $checked->{seqtype} = $params->{seqtype};


    if ($checked->{submission} eq 'email') {
	# at the moment always use CRC checks for email submissions
	$checked->{crc} = '1';
    }
    else {
	$checked->{crc} = $self->checkInteger($params->{crc}, undef, '0', '1');
    }

    $checked->{goterms}    = $self->checkInteger($params->{goterms}, undef, '0', '1');
    $checked->{scores}     = $self->checkInteger($params->{scores}, undef, '0', '1');
    $checked->{title}      = $params->{title};

    $self->setParams($checked);

    return (1, '');
}


=head2 formatSequences

 Description:  Formats input sequences to fasta format.
 Arguments:    $out        Path to the output file (fasta formatted, translated).
               $orig       Path to original output (non-formatted, non-translated).
               $seqs       Input sequences passed in a string (optional).
               $seqf       Input sequences passed in a file or file handle (optional).
               $seqtype    P (protein) or N (nucleotide) (optional).
               $frame      Frame: 1, 2, 3 forward, -1, -2, -3 reverse (optional).
               $trtable    Translation table code for nucleotide seqs (optional).
               $trlen      Translation threshold length (optional).
               $mode       Numeric mode for the directory if it needs to be created.

 Returns:      1, hash reference on success.
               0, msg on failure

=cut

sub formatSequences($$;$$$$$$$) {
    my ($self, $out, $orig, $seqs, $seqf, $seqtype, $frame, $trtable, $trlen, $mode) = @_;
    return (0, 'formatSequences: no output file given') unless $out;
    return (0, 'formatSequences: no original output file given') unless $orig;
    $seqtype = 'P' unless $seqtype;

    my ($res, $msg, $ifh, $iclose, @tmpfiles); 

    # check the destination directory
    my ($name, $path, $ext) = fileparse($out);
    ($res, $msg) = $self->checkDirectory($path, $mode);
    return (0, "formatSequences: format sequences failed: $msg") unless $res;

    if ($seqs) {
        # use scalar string as a file handle
        $ifh = new IO::Scalar(\$seqs);
        $iclose = 1;
    }
    elsif ($seqf) {
        ($res, $iclose, $ifh) = $self->getReadHandle($seqf);
        return (0, "formatSequences: format sequences failed: $ifh") unless $res;
    }
    elsif (!$seqf) {
        return (0, 'formatSequences: no sequence given');
    }

    my $tname = $self->getToolName();

    unless (open(ORIGINAL, "> $orig")) {
        close $ifh if $iclose;
        return (0, "formatSequences: could not write file $orig: $!");
    }

    ($res, $msg) = $self->safeWrite($ifh, \*ORIGINAL, Dispatcher::Tool::MAX_INPUT_SEQUENCE_SIZE);
    close $ifh if $iclose;
    close(ORIGINAL);
    return (0, "formatSequences: could not safely write file $orig: $msg") unless $res;

    # Here you can change the format command with one of your prefered script. To do so, change the command
    # line of this tag in your iprscan/conf/iprscan.conf file.
    my $formatcmd = $self->getConfigValue('formatcmd');
    unless ($formatcmd) {
	unlink($orig);
	return (0, "formatSequences: $tname configuration error: formatcmd not defined");
    }

    my $formatted = "$out.formatted";
    push @tmpfiles, $formatted;

    $formatcmd =~ s/\$in/$orig/gi;
    $formatcmd =~ s/\$out/$formatted/gi;

    ($res, $msg) = $self->systemCall($formatcmd);
    if ($res) {
        # make sure there are fasta header lines (raw sequences have nothing,
        # which causes problems with applications)
	unless (open(FORMATTED, "< $formatted")) {
	    unlink(@tmpfiles, $orig);
            return (0, "formatSequences: could not read formatted file $formatted: $!");
	}

	my $tmpfile = "$formatted.tmp";

	unless (open(TMPFILE, "> $tmpfile")) {
	    unlink(@tmpfiles, $orig);
            return (0, "formatSequences: could not read temporary file $tmpfile: $!");
	}

	my $nseq = 1;
	my ($u, $atgc, $total);

	while (<FORMATTED>) {
	  if ($_ =~ /^>\s*([^\s]+)/) { 
	    my $seqname = $self->trim($1) if $1;
	    if (!$seqname || $seqname =~ /^EMBOSS_\d+$/) {
	      $seqname = "Sequence_$nseq";
	    }
	    else {
	      $seqname = $self->protect($1);
	    }
	    
	    print TMPFILE ">$seqname\n";
	    $nseq++;
	  }
	  else {
	    $u     += ($_ =~ tr/Uu//);
	    $atgc  += ($_ =~ tr/ATGCNatgcn//);
	    $_ =~ s/\*//g;
	    $total += length($_);
	    print TMPFILE $_;
	  } 
	} #end while

	#Redetermine the sequence type!
	if(($atgc / $total) > 0.85 || (($atgc + $u) / $total) > 0.85) {
	  if($self->getParam('seqtype') eq 'auto'){
	    $self->setParam('seqtype', 'N');
	    $seqtype = 'N';
	  }
	}
	else {
	  if($self->getParam('seqtype') eq 'auto'){
	    $self->setParam('seqtype','P');
	    $seqtype = 'P';
	  }
	}

	close FORMATTED;
	close TMPFILE;

	unless (rename($tmpfile, $formatted)) {
	    unlink(@tmpfiles, $orig);
            return (0, "formatSequences: could not rename $tmpfile: $!");
        }

    } else {
	return(0, "formatSequences: $res\n$msg");
    }

    # check the nucleotide sequence length
    unless (open(FORMATTED, "< $formatted")) {
	unlink(@tmpfiles, $orig);
	return (0, "formatSequences: could not read formatted sequence $formatted: $!\n");
    }

    my $seqn = 0;
    my $seqlen = 0;
    my $maxseqlen = 0;

    while (<FORMATTED>) {
	if ($_ =~ /^>/) {
	    $seqn++;
	    $maxseqlen = $seqlen if ($seqlen > $maxseqlen);
	    $seqlen=0;
	} elsif ($_ =~ /([^\n]+)/) {
	    $seqlen += length($1);
	} elsif (eof) {
	    if ($_ =~ /([^\n]+)/) {
		$seqlen += length($1);
	    }
	    $maxseqlen = $seqlen if ($seqlen > $maxseqlen);
	}
    }

    close FORMATTED;

    if ($seqtype eq 'N') {
	my $maxn   = $self->getConfigValue('maxinputseqs.nt'); # || '1';
	my $maxlen = $self->getConfigValue('maxseqlen.nt'); # || '5000';

	if ($maxn && $seqn > $maxn) {
	    unlink(@tmpfiles, $orig);
	    return (0, "formatSequences: too many DNA/RNA sequences ($seqn), maximum is $maxn");
	}

	if ($maxlen && $maxseqlen > $maxlen) {
	    unlink(@tmpfiles, $orig);
	    return (0, "formatSequences: One of your DNA/RNA sequences is too long ($maxseqlen) for an InterProScan job, " .
                       "maximum length is $maxlen nucleotides. Please reformat your sequence.");
        }

	# Here you can change the command line to translate in 6 frames your nucleotide sequences.
	# To do so, change the command line of the translatecmd tag in your iprscan/conf/iprscan.conf file.
        my $translatecmd = $self->getConfigValue('translatecmd');
        unless ($translatecmd) {
	    unlink(@tmpfiles, $orig);
            return (0, "formatSequences: $tname configuration error: translatecmd not defined");
        }

	my $translated = "$out.translated";
	push @tmpfiles, $translated;

	#put default value for $trlen. We try default value from iprscan.conf first, then fix it to 50.
	$trlen = $self->getConfigValue('minorfsize') unless $trlen;
	$trlen = '50' unless $trlen;

	#put default value for $trtable. We try default value from iprscan.conf first, then fix it to 0.
	$trtable = $self->getConfigValue('codon.table') unless defined($trtable);
	$trtable = '0' unless $trtable;

	$translatecmd =~ s/\$table/$trtable/gi;
	$translatecmd =~ s/\$trlen/$trlen/gi;
	$translatecmd =~ s/\$in/$formatted/gi;
	$translatecmd =~ s/\$out/$translated/gi;

        ($res, $msg) = $self->systemCall($translatecmd);
        unless ($res) {
	    unlink(@tmpfiles, $orig);
            return (0, "formatSequences: sequence translation failed: $msg\n $translatecmd");
        }

	if (-s $translated == 0) {
	    unlink(@tmpfiles, $orig, $translated);
	    return (0, "formatSequences: translated sequence did not contain open reading frames above " .
                       "the chosen threshold ($trlen bases). Please try again with " .
                       "lower open reading frame size");
	}

	# Make sure the sequences are longer than the minimum open reading frame size.
        # This fixes the problem with sixpack, because we need the first and last ORFs,
        # but we don't want all the short ORFs. ProfileScan (pfscan) dumps core with
        # sequence that is just one AA.
        unless (open(TRANSLATED, "< $translated")) {
	    unlink(@tmpfiles, $orig);
            return (0, "formatSequences: could not read translation file $translated: $!");
        }

	my $fixed = "$translated.fixed";
	push @tmpfiles, $fixed;

        unless (open(FIXED, "> $fixed")) {
            close(TRANSLATED);
	    unlink(@tmpfiles, $orig);
            return (0, "formatSequences: could not write fixed file $fixed: $!");
        }

	my ($header, $pheader);
        my $seq = '';

	# N.B. newer versions of sixpack (available from EMBOSS) have -clean and -trim options
	# which remove unwanted X and *.  Users can download and compile the new version if they
	# want, and add the -trim and -clean options to the cmdline in iprscan.conf

        while (<TRANSLATED>) {
                chomp;
                if (eof) {
                        if ($seq) {
                                # remove Xs from the start and end of the sequence
                                $seq =~s/^[Xx]+//;
                                $seq =~s/[Xx]+$//;
				$seq =~s/\*//g;
                                if (length ($seq) >= $trlen) {
                                        print FIXED "$pheader\n$seq\n";
                                }
                        }
                } elsif ($_ =~ /^>(.*)/) {
                        $header = ">$1";
                        if ($seq) {
                                # remove Xs from the start and end of the sequence
                                $seq =~s/^[Xx]+//;
                                $seq =~s/[Xx]+$//;
                                # write the previous sequence if length > ORF min size
                                if (length ($seq) >= $trlen) {
                                        print FIXED "$pheader\n$seq\n";
                                }
                        }
                                                                                                                                                             
                        $seq = '';
                        $pheader = $header;
                } elsif ($_ =~ /([^\n]+)/) {
                        $seq .= "$1\n";
                }
        }
                                                                                                                                                             
        close(TRANSLATED);
        close(FIXED);

	if (-s $fixed == 0) {
	    unlink(@tmpfiles, $orig, $translated);
	    return (0, "formatSequences: translated sequence did not contain open reading frames above " .
                       "the chosen threshold ($trlen bases). Please try again with " .
                       "lower open reading frame size");
	}

	unless (rename($fixed, $out)) {
	    unlink(@tmpfiles, $orig);
            return (0, "formatSequences: could not rename fixed file $fixed: $!");
        }
      } #end if($seqtype eq 'N')
    else {
	my $maxn = $self->getConfigValue('maxinputseqs.aa'); # || '10';
	my $minlen = $self->getConfigValue('minseqlen.aa'); # || '5';
	
	if ($maxn && $seqn > $maxn) {
	    unlink(@tmpfiles, $orig);
	    return (0, "formatSequences: too many protein sequences ($seqn), maximum is $maxn");
	}

	if ($seqlen == 0) {
	    $msg  = "formatSequences: Could not find input sequence, length is zero.<br><br>";
	    $msg .= "If you are trying to use Fasta formatted sequence, please make ";
            $msg .= "sure you have a newline after the > header line (your browser ";
            $msg .= "may wrap long lines from space characters, but you should check ";
            $msg .= "that there is a newline). For example:<br>\n<pre>";
            $msg .= ">Header\n";
            $msg .= "MRCSISLVLGLLALEVALARNLQEHVFNSVQSMCSDDSFSEDTECINCQTNEECAQNDMC\n";
            $msg .= "CPSSCGRSCKTPVNIEVQKAGRCPWNPIQMIAAGPCPKDNPCSIDSDCSGTMKCCKNGCI\n";
            $msg .= "MSCMDPEPKSPTVISFQ\n</pre>";
	    return (0, $msg);
	}

	if ($minlen && $seqlen < $minlen) {
	    unlink(@tmpfiles, $orig);
	    return (0, "formatSequences: protein sequence is too short ($seqlen aa) for an " .
                    "InterProScan job, minimum length is $minlen aa. Please " .
                    "check your input sequence. If you are using database " .
                    "references, the format is DB:ID (e.g. uniprot:WAP_RAT) " .
                    "and DB:AC (e.g. uniprot:P01174)");
        }

	unless (rename($formatted, $out)) {
	    unlink(@tmpfiles, $orig);
            return (0, "formatSequences: could not rename formatted file $formatted: $!");
        }
    }

    close $ifh if $iclose;

    unlink(@tmpfiles);

    return (1, '');
}


=head2 checkSequences

 Description: Checks the fasta formatted input sequences. This method calculates
              the CRC64 and length for each sequence and also writes the non-matching
              sequences to a file (optional). Returned hash contains following fields:

               'WAP_RAT' => { 'seqn'     => '1',
                              'len'      => '137',
                              'crc64'    => '1C2E8ADA9FD97949',
                              'rawentry' => '...'
                            },
               'WAP_PIG' => { ... }
               ...

              TODO: the hash may potentially take lot of memory depending on how
              many sequences are checked. If one entry takes 1-2KB, 50000 would
              take 50-100MB. Optionally we could write the raw entries to files,
              that would then generate 50000 files (worse?).

 Arguments:   $in           Input sequence file or file handle.
              $checkcrc     1: check CRC64 for each sequence, 0: do not
              $out          Output file or file handle for sequences that have unknown
                            CRC64 checksum ($checkcrc 1) or all sequences ($checkcrc 0).
              $apps         Reference to an array containing requested applications
                            (affects the raw entries).
              $iprfields    1: includes InterPro fields with raw entries 0: no
              $goterms      1: includes GO terms with raw entries 0: no

 Returns:     1, reference to a hash
              0, msg

=cut

sub checkSequences($;$$$$) {
  my ($self, $in, $checkcrc, $out, $apps, $iprfields, $goterms) = @_;
  return (0, "checkSequences: no input sequence given") unless $in;
  
  my ($res, $iclose, $ifh, $oclose, $ofh, $msg);
  
  ($res, $iclose, $ifh) = $self->getReadHandle($in);
  return (0, "checkSequences: could not read input file handle $ifh") unless $res;
  
  my $seqh = { };
  my $seqn = 0;
  my ($seq, $id, $preid, $crc, $len, $line, $raw);
  
  #Get the chunk size from the config value
  my $chunk = $self->getConfigValue('chunk');
  $chunk = undef unless($chunk > 0);
  return(0, "checkSequences: 'chunk' tag must be defined in your iprscan.conf.") unless(defined($chunk));

  my $mode = $self->getConfigValue('dirmode');

  my $nseq = 0;
  my $fpos = 0;
  my $inx = { };
  my $trace = { };
  my $cnknb = 1;
  $cnknb = $self->getParam('cnk') if($self->getParam('cnk'));

  my($name, $path) = fileparse($out);
  my $iprmatches;

  ($res, $msg) = $self->addChunk("$cnknb", \$out, \$path);
  return(0, "checkSequences: cannot add chunk: $msg") unless $res;
  
  if($checkcrc){
    ($res, $iprmatches) = new Index::IprMatches(undef, 1);
    return(0, "checkSequences: cannot create IprMatches object $iprmatches") unless($res);
  }

  #Creates only chunk for the first run, no need to do it if resubmit failed jobs
  #In Resubmit we call it to get info about sequences like crc64, length and id.
  if(defined($chunk) and !$self->getParam('resubmit')){
   ($res, $msg) = $self->checkDirectory($path, $mode);
   return(0, "checkSequences: Unable to check directory: $msg") unless $res;
  }

  ($res, $oclose, $ofh) = $self->getWriteHandle($out);
  unless($res){
    close $ifh if $iclose;
    return(0, "checkSequences: Unable to get write file handle $ofh");
  }


  # Read the sequence input file
  while (<$ifh>) {
    $line = $self->trim($_);  # remove white space
    next unless $line;        # skip empty lines

    if ($line =~ /^>\s*([^\s]+)/) {
      $id = $1;
      $inx->{$1} = $fpos;
      # remove commas from the end of the fasta header, otherwise the header
      # won't match the superfamily results (superfamily removes them)
      $id =~ s/\,+$//;

      if ($seq) {
	# store information about the previous sequence (i.e. all but the final sequence)
	$crc = $self->crc64(uc($seq));
	$len = length($seq);

	# Next, check for duplicate sequences in the fasta file
	if(exists($seqh->{$preid}) && $seqh->{$preid}->{crc64} eq $crc && $seqh->{$preid}->{len} == $len){
	    my $mess = "\ncheckSequences: CRC64($crc) and length($len) already exist for [$preid: CRC($seqh->{$preid}->{crc64}) LEN($seqh->{$preid}->{len})].\n";
	    $mess .= "Check you don't have duplicate sequences.\n";
	    close $ifh if $iclose;
	    close $ofh if $ofh && $oclose;
	    return(0, $mess);
	}

	$seqh->{$preid}->{len}   = $len;
	$seqh->{$preid}->{crc64} = $crc;

        if ($checkcrc) { # The user requested a crc check
          # So first check if there is a result in the match XML file
          my $entries;
          ($res, $entries) = $iprmatches->getEntry($crc);
          return (0, "checkedSequences: not possible to retrieve entry: $entries") unless ($res);

          if ($entries) { # an entry is returned
            return (0, "checkedSequences: entries is not an array: $entries") unless (ref($entries) eq 'ARRAY');

            # If there is a result, output it so that it's not run through InterProScan
            # N.B. if the protein is in the match XML file but has no matches, it will
            # return a result but nothing will be output

            if ($entries->[0] =~/match/) {    # Sequence with that CRC has a match in the match file

              # fetch and parse the full entry for the selected applications
	      ($res, $raw) = $self->getRawEntryFromIprMatches($preid, $crc, $len, $apps,$iprfields, $goterms);
	      return (0, "checkSequences: Cannot get raw entry from iprmatches: $raw") unless $res;

	      if ($raw) {
		  # Store the raw result in the hash being returned from the method
		  $seqh->{$preid}->{rawentry} = $raw;
	      } elsif ($ofh) {
                $seq =~ s/(.{1,60})/$1\n/g;
                $seq = uc($seq);               #TMHMM wants it in upper case
                print $ofh ">$preid\n$seq";
	      }

	    } elsif ($entries->[0] =~/./) {   # Sequence with that CRC is in the match file but doesn't have a match
              # Do nothing...
            } else {                        # Sequence does not exist in the match file
              my $posid = undef;
              #What we do here is to avoid to write sequence to the .nocrc file even if $raw is empty
              #because this crc is known (defined) but in the IprMatches, we filter the data to print
              #only the results from iprmatches that contain entries for the applications the user
              #selected.
              $seq =~ s/(.{1,60})/$1\n/g;
              $seq = uc($seq);             #TMHMM wants it in upper case
              print $ofh ">$preid\n$seq";

            } # end if ($entries->[0] =~/match/)
          } else { # No entry returned after first check of XML file
            # (this else statement will be made obsolete by match-less proteins in XML file)

            my $posid = undef;

            $seq =~ s/(.{1,60})/$1\n/g;
            $seq = uc($seq);             #TMHMM wants it in upper case
            print $ofh ">$preid\n$seq";

          }

	} #end checkcrc
	elsif ($ofh) { # by default, print out if -nocrc requested
	  $seq =~ s/(.{1,60})/$1\n/g;
	  $seq = uc($seq);               #TMHMM wants it in upper case
	  print $ofh ">$preid\n$seq";
	}
	
	$seq = '';
      } #end if $seq

      if(defined($chunk) && $nseq == $chunk and !$self->getParam('resubmit')){
	$cnknb++;
	$nseq = 0;
	$seqn = 0;
	close $ofh if $ofh;
	($res, $msg) = $self->addChunk("$cnknb", \$out, \$path);
	return(0, "checkSequences: $msg") unless $res;
	($res, $msg) = $self->checkDirectory($path, $mode);
	return(0, "checkSequences: $msg") unless $res;
	($res, $oclose, $ofh) = $self->getWriteHandle($out);
	unless ($res) {
	  close $ifh if $iclose;
	  return (0, "checkSequences: $ofh");
	}
      }

      $nseq++;
      $preid = $id;
      $seqn++;

      $seqh->{$id}->{seqn} = $seqn;
      $seqh->{$id}->{cnk}  = $cnknb;

    } else {
      # read the sequence to memory so we can calculate CRC64 and length
      # FIXME: implement a limit to control the memory usage
      $seq .= "$line";
    }
    $fpos = tell($ifh);
  } #end while

  # Repeat for the Last sequence
  if ($seq) {
    my $crc = $self->crc64(uc($seq));
    my $len = length($seq);

    if(exists($seqh->{$preid}) && $seqh->{$preid}->{crc64} eq $crc && $seqh->{$preid}->{len} == $len){
	my $mess = "\ncheckSequences: CRC64($crc) and length($len) already exist for [$preid].\n";
	$mess .= "Check you don't have duplicate sequences.\n";
	close $ifh if $iclose;
	close $ofh if $ofh && $oclose;
	return(0, $mess);
    }

    $seqh->{$preid}->{len}   = $len;
    $seqh->{$preid}->{crc64} = $crc;
    
    if ($checkcrc) {
      # First check if there is a result in the match XML file
      my $entries;
      ($res, $entries) = $iprmatches->getEntry($crc);
      return (0, $entries) unless ($res);

      if ($entries) {
        return (0, $entries) unless (ref($entries) eq 'ARRAY');

        # If there is a result, output it so that it's not run through InterProScan
        # N.B. if the protein is in the match XML file but has no matches, it will
        # return a result but nothing will be output

        if ($entries->[0] =~/match/) {    # Sequence with that CRC has a match in the match file

	  ($res, $raw) = $self->getRawEntryFromIprMatches($preid, $crc, $len, $apps,$iprfields, $goterms);
	  return (0, "checkSequences: Cannot get raw entry from iprmatches: $raw") unless $res;

	  if ($raw) {
	    $seqh->{$preid}->{rawentry} = $raw;
	  } # end if ($raw)
          elsif ($ofh) { # by default, print out if -nocrc requested
            $seq =~ s/(.{1,60})/$1\n/g;
            $seq = uc($seq);               #TMHMM wants it in upper case
            print $ofh ">$preid\n$seq";
          }

        } elsif ($entries->[0] =~/./) {   # Sequence with that CRC is in the match file but doesn't have a match
          # Do Nothing
        } else {                            # Sequence does not exist in the match file
          my $posid = undef;
          #What we do here is to avoid to write sequence to the .nocrc file even if $raw is empty
          #because this crc is known (defined) but in the IprMatches, we filter the data to print
          #only the results from iprmatches that contain entries for the applications the user
          #selected.
          $seq =~ s/(.{1,60})/$1\n/g;
          $seq = uc($seq);             #TMHMM wants it in upper case
          print $ofh ">$preid\n$seq";
        }
      } else {
        # in case no entry returned after first check of XML file
        # (this else statement will be made obsolete by match-less proteins in XML file)

        my $posid = undef;

        $seq =~ s/(.{1,60})/$1\n/g;
        $seq = uc($seq);             #TMHMM wants it in upper case
        print $ofh ">$preid\n$seq";

      }
    } #end checkcrc
    elsif ($ofh) { # by default, print out if -nocrc requested
      $seq =~ s/(.{1,60})/$1\n/g;
      $seq = uc($seq);               #TMHMM wants it in upper case
      print $ofh ">$preid\n$seq";
    }
  }

  close $ifh if $iclose;
  close $ofh if $ofh && $oclose;
  $seqh->{nchunk} = $cnknb if(defined($chunk) and !$self->getParam('resubmit'));
  $seqh->{inx} = $inx unless($self->getParam('resubmit'));
  return (1, $seqh);
}



=head2 indexInput

 Description:  index the input sequence by descrption field

               Ex : >wap_rat blah balh
                    rthgensvcdawwq....
                    ......
                    >another sequence
                    ...
               Will index the position of the 'wap_rat' word.

 Arguments:    $file   Path of the file to index.

 Returns:      1, on success
               0, msg on failure

=cut


sub indexInput {

  my($self, $file) = @_;

  my($res, $msg, $index);

  if($file){
    return(0, "indexInput: $file does not exist: $!") unless(-f $file);
  } else {
    $file = $self->getConfigValue('toolinput');
  }

  ($res, $index) = Index::Fasta->new($file, 1);
  return(0, "indexInput: Failed to initiate Index::Fasta object : $index") unless($res);


  ($res, $msg) = $index->buildIndex();
  return(0, "indexInput: Cannot build index: $msg") unless($res);

  ($res, $msg) = $index->indexOut($msg);
  return(0, "indexInput: Cannot output index: $msg") unless ($res);

  return(1, '');
}



=head2 submitJobs

 Description:  Submits InterProScan application jobs. Returns a hash with
               job identifiers as keys and paths to output and error files:

               jobid1 => { output => 'path/to/output',
                             errors => 'path/to/errors' },
               jobid2 => { ... },


 Arguments:    $in    Path to the input sequences file.
 Returns:      1, jobs in array reference,     ''  on success
               0, successfully submitted jobs, msg on failure

=cut

sub submitJobs($) {
    my ($self, $in) = @_;
    my $jobs = { };

    unless ($in && -e $in && -s $in > 0) {
	 return (0, $jobs, "submitJobs: submission failed, no input sequences defined");
    }

    my $params = $self->getParams();
    return (0, $jobs, "submitJobs: submission failed, no parameters defined") unless $params && ref($params) eq 'HASH';

    my $jobid = $self->getParam('jobid');
    return (0, $jobs, "submitJobs: submission failed, no job identifier defined") unless $jobid;

    my $applist = $params->{appl};
    my @apps = split(',', $applist) if $applist;
    return (0, $jobs, "submission failed, no applications defined") unless $applist && @apps > 0;

    my ($seq, $appl, $ajobid, $res, $msg, $tool, $tname, $defaults, $nseq, $outfile, $errfile);
    my $queue = new Dispatcher::Queue();
    my $seqcount = 1;

    foreach $appl (@apps) {
	$appl = lc($appl);

	$tname = $self->mapApplNameToToolName($appl);
	return (0, $jobs, "submitJobs: submission failed, $appl not supported") unless $tname;

	$defaults = { };

	#User use Chunk?
	my $chunk = '';
	if($in =~ /chunk_(\d+)/) {
	  $chunk = "$1";
	} else {
	  return(0, $jobs, "submitJobs: No chunk found in [$in]!");
	}

	#We want to be sure to keep the date...

	if ($jobid =~ /^[A-Za-z]+-([\d]{4})(\d\d)(\d\d)-(\d\d)(\d\d)(\d\d)(\d\d)/) {
	    # set date and time because they may be used in configuration (paths and URLs)
	    $defaults->{YYYY} = $1;
	    $defaults->{MM} = sprintf("%.2d", $2);
	    $defaults->{DD} = sprintf("%.2d", $3);
	    $defaults->{hh} = sprintf("%.2d", $4);
	    $defaults->{mm} = sprintf("%.2d", $5);
	    $defaults->{ss} = sprintf("%.2d", $6);
	} else {
	    return(0, $jobs, "submitJobs: could not determine jobid from [$jobid].");
	}

	$defaults->{jobid}     = "$jobid-$appl-cnk$chunk"; #Add this to be able to get make difference between jobs using LSF
	$defaults->{session}   = "$jobid";                 #to create the session directory
	$defaults->{chunk}     = "chunk_$chunk";
        $defaults->{srchtype}  = $self->getParam('srchtype');
	$defaults->{toolinput} = $in;
	
	#Create Dispatcher::Tool::Seg object, and parseConfig at the same time so we can retrieve information from .conf file.
	($res, $tool) = $self->createTool($tname, $defaults);
	return (0, $jobs, "submitJobs: creating tool failed: $tool") unless $res;

	if ($self->getConfigValue('debug')) {
	    my $dfile = $self->getConfigValue('tooldebug');
	    if ($dfile && open(DEBUG, ">> $dfile")) {
		print DEBUG $tool->getConfigValue('cmdline');
		print DEBUG "\n\n";
		close(DEBUG);
	    }
	}

	my @args;
	push(@args , split(/ /, $tool->getConfigValue('cmdline')));
	
	my $sync     = $tool->getParam('srchtype') eq "interactive" ? 1 : undef;
	my $stderr   = $tool->getConfigValue('toolerrors');
	my $stdout   = $tool->getConfigValue('tooloutput');
	my $exitcode = $tool->getConfigValue('toolexitcode');

	my $std = { stdin    => $in,
		    stderr   => $stderr,
		    stdout   => $stdout,
		    exitcode => $exitcode
		  };

	($res, $ajobid) = $queue->submitWithArgs($tool, \@args, $sync, $std, undef,$tool->getConfigValue('jobid'));
	return (0, $jobs, "submitJobs: submission failed: $ajobid") unless $res;

	$jobs->{$ajobid} = { output   => $stdout,
			     errors   => $stderr,
			     exitcode => $exitcode };
      }

    return (1, $jobs, '');
  }


=head2 terminateJobs

 Description:  Terminates jobs with given names.  Job names must begin with the
               tool name, so Dispatcher::Queue::kill can read the tool
               configuration (defines the queue).
 Arguments:    $jobids  Reference to a hash table
 Returns:      -

=cut

sub terminateJobs($) {
    my ($self, $jobids) = @_;
    return unless ($jobids && ref($jobids eq 'HASH'));

    my $queue = new Dispatcher::Queue();
    my ($res, $msg, $jobid);

    foreach $jobid (keys %$jobids) {
        if ($jobid =~ /^([a-zA-Z]+)/) {
            ($res, $msg) = $queue->kill($1, $jobid);
        }
    }
}


=head2 checkJobs

 Description:  Check jobs with given names.  Job names must begin with the tool
               name, so Dispatcher::Queue::check can read the tool configuration
               (defines the queue).
 Arguments:    $h_chunk Reference to a hash table containing info on each chunk
               $chunk   Chunk number to check (optional)
 Returns:      1, $href on success
               0, $msg  on failure

=cut

sub checkJobs($) {
    my ($self, $h_chunk, $chunk) = @_;

    return(0, "checkJobs : No hash reference given for h_chunk") unless $h_chunk && ref($h_chunk) eq 'HASH';

    my $sys = $self->getConfigValue('queue');
    my $queue = new Dispatcher::Queue();
    my ($jobid, $jobids, $tname, $status, $msg, $start, $stop);
    my $href = { };

    $start = $chunk ? $chunk : 1;
    $stop  = $chunk ? $chunk : $h_chunk->{nbchunk};

    for($start; $start <= $stop ; $start++){

	$jobids = $h_chunk->{$start}->{status};

	foreach $jobid (keys %$jobids) {
	    $tname = $self->mapJobIDToToolName($jobid);

	    if ($tname) {
		my $tries = 5;  # try few times just in case batch job system has problems
		my $defaults = { };

		if ($jobid =~ /^([A-Za-z]+-([\d]{4})(\d\d)(\d\d)-(\d\d)(\d\d)(\d\d)(\d\d))-\w+-cnk(\d+)/) {
		    # set date and time because they may be used in configuration (paths and URLs)
		    $defaults->{YYYY} = $2;
		    $defaults->{MM} = sprintf("%.2d", $3);
		    $defaults->{DD} = sprintf("%.2d", $4);
		    $defaults->{hh} = sprintf("%.2d", $5);
		    $defaults->{mm} = sprintf("%.2d", $6);
		    $defaults->{ss} = sprintf("%.2d", $7);
		    $defaults->{session} = $1;
		    $defaults->{jobid} = $jobid;
		    $defaults->{chunk} = "chunk_$9";
		} else {
		    return(0, "checkJobs : could not determine jobid from [$jobid]");
		}

		my ($res, $tool) = $self->createTool($tname, $defaults);
		return(0, "checkJobs: could not createTool $tool") unless($res);

		while ($tries > 0) {

		    ($status, $msg) = $queue->check($tool, $jobid);

		    # It looks like LSF multicluster jobs sometime return either
		    # EXIT, UNKWN or ZOMBI, even though later bjobs shows the job
		    # has finished succesfully (DONE).
		    if ($status && $status != Dispatcher::Queue::JOB_FAILED) {
			last;
		    }

		    $tries--;
		    sleep(1);
		}

		if ($status) {
		    $href->{$jobid} = $status;
		}
		else {
		    # NOTE: iprscan wrapper looks for word "failed" in the string
		    $href->{$jobid} = "failed to check status: $msg";
		}
	    }
	    else {
		$href->{$jobid} = "invalid job ID: $jobid";
	    }
	}  #end foreach loop
	$h_chunk->{$start}->{status} = $href;
	$href = { };
    } #end for loop

    return (1, $h_chunk);
}

=head2 addChunk

 Description:  Insert the chunk number in the path.
 Arguments:    $num   number of the chunk directory
	       $sref  an array of scalar references
 Returns:      1
               0, msg  on failure

=cut


sub addChunk($$$) {

   my ($self, $num, @sref) = @_;
   return(0, "addChunk: need a chunk number to be added") unless($num);
   return(0, "addChunk: Chunk number is not a digit [$num]") unless($num =~ /^\d+$/);
   return(0, "addChunk: Cannot add chunk without path, need a reference to a scalar") unless(scalar(@sref));

   my $cnk = 'chunk_';

   my $wkdir = $self->getConfigValue('workdir');
   my $jobid = $self->getConfigValue('jobid');

   return (0, "addChunk: No workdir defined in the config file") unless($wkdir);
   return (0, "addChunk: [$wkdir] does not exist: $!") unless(-d $wkdir);
   return (0, "addChunk: jobid not configured") unless($jobid);

   for(@sref){
      return(0, "addChunk: need a reference to a SCALAR") unless(ref($_) eq 'SCALAR');
      if(${$_} =~ /$wkdir\/$cnk(\d+)/){
         ${$_} =~ s!/$cnk$1!/$cnk$num!;
      }
      elsif(${$_} =~ /$wkdir/){
         ${$_} =~ s!$wkdir!$wkdir/$cnk$num!;
      }
      elsif(${$_} =~ /($jobid\.\w+)$/){
         ${$_} =~ s!$jobid\.!$cnk$num/$jobid\.!;
      }
      elsif(${$_} =~ /($jobid)(\/.+)$/){
         my $f = $2;
         ${$_} =~ s!$2!/$cnk$num$f!;
      }
      else {
         return(0, "addChunk: could not retrieve workdir from [${$_}]");
      }
   }
   return(1, );

}


=head2 cleanup

 Description:  Removes temporary files created during job submissions.
               Calls Queue::cleanup for each job.
 Arguments:    $h_chunk Reference to a hash table containing info on each chunk
               $chunk   Chunk number to check (optional)
 Returns:      -

=cut

sub cleanup($$) {
    my ($self, $h_chunk, $chunk) = @_;
    return(0, "cleanup: Need a HASH reference to cleanup") unless $h_chunk && ref($h_chunk) eq 'HASH';

    my $start = $chunk ? $chunk : 1;
    my $stop  = $chunk ? $chunk : $h_chunk->{nbchunk};
    my($res, $msg);

    my $queue = new Dispatcher::Queue();

    my ($tool, $efile, $repfile);

    for($start; $start <= $stop; $start++){
        my $jobids = $h_chunk->{$start}->{jobs};

        foreach my $jobid (keys %$jobids){
            my $tname = $self->mapJobIDToApplName($jobid);

            my $defaults = { };
	    my @dirs = ( );

            if($jobid =~ /^([A-Za-z]+-([\d]{4})(\d\d)(\d\d)-(\d\d)(\d\d)(\d\d)(\d\d))-\w+-cnk(\d+)/) {
	      # set date and time because they may be used in configuration (paths and URLs)
	      $defaults->{YYYY} = $2;
	      $defaults->{MM} = sprintf("%.2d", $3);
	      $defaults->{DD} = sprintf("%.2d", $4);
	      $defaults->{hh} = sprintf("%.2d", $5);
	      $defaults->{mm} = sprintf("%.2d", $6);
	      $defaults->{ss} = sprintf("%.2d", $7);
	      $defaults->{session} = $1;
	      $defaults->{jobid} = $jobid;
	      $defaults->{chunk} = "chunk_$9";
	    } else {
	      return(0, "cleanup: could not determine jobid from [$jobid].");
	    }

	    ($res, $tool) = $self->createTool($tname, $defaults);
	    return(0, "cleanup: could not create tool $tool") unless $res;
	    my ($file, $dir);

	    $dir = $tool->getConfigValue('default.workdir');
	    push @dirs, $dir if $dir && -d $dir;

	    $dir = $tool->getConfigValue('workdir');
	    push @dirs, $dir if $dir && -d $dir;

	    $dir = $tool->getConfigValue('sworkdir');
	    push @dirs, $dir if $dir && -d $dir;

	    $dir = $tool->getConfigValue('aworkdir');
	    push @dirs, $dir if $dir && -d $dir;

	    my $count = 0;

	    foreach $dir (@dirs) {
		$file = catfile($dir, $jobid . Dispatcher::Queue::EXT_SUBMIT);
		$count += unlink($file) if -f $file;

		$file = catfile($dir, $jobid . Dispatcher::Queue::EXT_TOOLCMD);
		$count += unlink($file) if -f $file;

		$file = catfile($dir, $jobid . Dispatcher::Queue::EXT_JOBID);
		$count += unlink($file) if -f $file;
	    }

            ($res, $msg) = $self->cleanup($tname, $jobid);
            return(0, "cleanup: Nothing cleaned") unless($count);

            if ($res) {
	        $efile = $tool->getConfigValue('toolerrors');
	        if (-e $efile && -s $efile == 0) {
		    unlink($efile);
	        }
	    }
	}
    }
    return(1, '');
}

=head2 updateStatus

 Description:  Update the status of job in a file
 Arguments:    $h_chunk  hash reference containing status for chunks jobs
               $chunk    number of the chunk (optional)
 Returns:      1
               0, msg  on failure

=cut


sub updateStatus ($;$$) {

    my($self, $h_chunk, $chunk) = @_;

    return(0, "updateStatus: Need a hash reference to a hash table") unless($h_chunk && ref($h_chunk) eq 'HASH');

    my ($jobid, $start, $stop, $status);

    $start = $chunk ? $chunk : 1;
    $stop  = $chunk ? $chunk : $h_chunk->{nbchunk};

    my $sfile = $self->getConfigValue('toolstatus');

    for($start; $start <= $stop; $start++){

      my $report = $h_chunk->{$start}->{status};
      unless (open(STATUS, ">> $sfile.$PID")) {
	# something seriously wrong with the file system
	  
	#$self->terminateJobs($jobs);
	return(0, "updateStatus: could not write status file $sfile.$PID: $!");
      }

      foreach $jobid (keys %$report) {
	$status = $report->{$jobid};
	print STATUS "$jobid=$status\n";
      }

      close STATUS;

      # rename the current status file now that it is ready
      unless (rename("$sfile.$PID", $sfile)) {
	# something seriously wrong with the file system
	#$self->terminateJobs($jobs);
	return(0, "updateStatus: could not rename status file $sfile.$PID: $!");
      }
    } #end for

    return(1, $h_chunk);
}


=head2 createTool

 Description:  Creates Dispatcher tool objects for InterProScan.
 Arguments:    $tname       Tool name.
               $defaults    Defaults for tool configuration (optional).
 Returns:      1, tool on success
               0, msg  on failure

=cut

sub createTool($;$) {
    my ($self, $tname, $defaults) = @_;
    return (0, "createTool: cannot create tool without tool name") unless $tname;

    $tname = lc($tname);
    my $tool;

    if ($tname =~ /blastprodom/) {
        $tool = new Dispatcher::Tool::BlastProDom($tname);
    }
    elsif ($tname =~ /fpscan|fprintscan/) {
        $tool = new Dispatcher::Tool::FingerPrintScan($tname);
    }
    elsif ($tname =~ /gene3d/) {
	$tool = new Dispatcher::Tool::Gene3D($tname);
    }
    elsif ($tname =~ /hmmpfam/) {
        $tool = new Dispatcher::Tool::Hmmer3($tname);
    }
    elsif ($tname =~ /hmmsmart/) {
        $tool = new Dispatcher::Tool::Hmmer($tname);
    }
    elsif ($tname =~ /hmmtigr/) {
        $tool = new Dispatcher::Tool::Hmmer3($tname);
    }
    elsif ($tname =~ /hmmpanther/) {
        $tool = new Dispatcher::Tool::HMMPanther($tname);
    }
    elsif ($tname =~ /hmmpir/) {
        $tool = new Dispatcher::Tool::HMMPIR($tname);
    }
    elsif ($tname =~ /pfscan|profilescan/) {
        $tool = new Dispatcher::Tool::ProfileScan($tname);
    }
    elsif ($tname =~ /hamap/) {
	$tool = new Dispatcher::Tool::HAMAP($tname);
    }
    elsif ($tname =~ /scanregexp/) {
        $tool = new Dispatcher::Tool::ScanRegExp($tname);
    }
    elsif ($tname =~ /patternscan/) {
        $tool = new Dispatcher::Tool::PatternScan($tname);
    }
    elsif ($tname =~ /superfamily/) {
        $tool = new Dispatcher::Tool::SuperFamily($tname);
    }
    elsif ($tname =~ /coils/) {
        $tool = new Dispatcher::Tool::Coils($tname);
    }
    elsif ($tname =~ /signalp/) {
        $tool = new Dispatcher::Tool::SignalP($tname);
    }
    elsif ($tname =~ /tmhmm/) {
        $tool = new Dispatcher::Tool::TMHMM($tname);
    }
    elsif ($tname =~ /seg/) {
        $tool = new Dispatcher::Tool::Seg($tname);
    }
    else {
        return (0, "createTool: sorry, tool [$tname] is not supported");
    }

    if ($defaults) {
        my ($res, $msg) = $tool->parseConfig(1, $defaults);
        return (0, "createTool: could not parse config $msg") unless $res;
    }

    return (1, $tool);
}


=head2 mapJobIDToApplName

 Description:  Maps InterProScan job identifier to application name. This is a
               convenience method for mapping the application job IDs to the
               real tool names.
 Arguments:    $jobid    Job identifier.
 Returns:      tool name    on success
               empty string on failure

=cut

sub mapJobIDToApplName($) {
    my ($self, $jobid) = @_;

    if ($jobid && $jobid =~ /[A-Za-z]+-[\d]{8}-[\d]{8}-([A-Za-z0-9]+)-/) {
        return $1;
    }
    
    return '';
}


=head2 mapJobIDToToolName

 Description:  Maps InterProScan job identifier to tool name. This is a
               convenience method for mapping the application job IDs
               to the real tool names.
 Arguments:    $jobid    Job identifier.
 Returns:      tool name    on success
               empty string on failure

=cut

sub mapJobIDToToolName($) {
    my ($self, $jobid) = @_;

    if ($jobid && $jobid =~ /[A-Za-z]+-[\d]{8}-[\d]{8}-([A-Za-z0-9]+)-/) {
        return $self->mapApplNameToToolName($1);
    }

    return '';
}


=head2 mapJobIDToSeqNo

 Description:  Maps InterProScan job identifier to input sequence number.
               This is not intended for the chunk submissions, which don't
               have the sequence number in the job identifier.
 Arguments:    $jobid    Job identifier.
 Returns:      sequence number on success
               empty string    on failure

=cut

sub mapJobIDToSeqNo($) {
    my ($self, $jobid) = @_;

#    if ($jobid && $jobid =~ /^[A-Za-z]+-[\d]{8}-[\d]{8}-[A-Za-z0-9]+-cnk(\d+)/) {
#        return $1;
#    }

    return '';
}


=head2 mapApplNameToToolName

 Description:  Maps InterProScan application to tool name. This is a
               convenience method.
 Arguments:    $appl    Application name, i.e., 'hmmsmart'.
 Returns:      tool name    on success
               empty string on failure

=cut


sub mapApplNameToToolName() {
    my ($self, $appl) = @_;
    return '' unless $appl;

    my $tname = $ApplNameToToolName->{"$appl"};
    return '' unless $tname;
    
    if ($self->getToolName() =~ /dev$/) {
        # use development version
        $tname .= 'dev';
    }

    return $tname;
}


=head2 mapStatus

 Description:  Maps status code to InterProScan job status text.
 Arguments:    $status    Status code.
 Returns:      status text

=cut

sub mapStatus($) {
    my ($self, $status) = @_;

    if ($status == Dispatcher::Queue::JOB_PENDING) {
        $status = $self->getConfigValue('info.jobpending');
    }
    elsif ($status == Dispatcher::Queue::JOB_RUNNING) {
        $status = $self->getConfigValue('info.jobrunning');
    }
    elsif ($status == Dispatcher::Queue::JOB_DONE) {
        $status = $self->getConfigValue('info.jobdone');
    }
    elsif ($status == Dispatcher::Queue::JOB_FAILED) {
        $status = $self->getConfigValue('info.jobfailed');
    }
    elsif ($status == Dispatcher::Queue::JOB_NOT_FOUND) {
        $status = $self->getConfigValue('info.jobnotfound');
    }

    return $status;
}


=head2 getHomePage

 Description: Returns the home page for interproscan
 Arguments:
 Returns:     ref to a variable

=cut

sub getHomePage {

my($self) = shift;

my $urlserver = $self->getConfigValue('workserver');
my $imagepath = $self->getConfigValue('imgbase');
my $wgeturl   = $self->getConfigValue('wgeturl.index');
my $logo      = $self->getConfigValue('imgurl.icon');
my $interpro  = $self->getConfigValue('imgurl.interprosmall');
my $interpro2 = $self->getConfigValue('imgurl.interprosmall2');
my $toolbox   = $self->getConfigValue('imgurl.toolbox');
my $toolbox2  = $self->getConfigValue('imgurl.toolbox2');
my $tree      = $self->getConfigValue('imgurl.tree');
my $tree2     = $self->getConfigValue('imgurl.tree2');
my $checkcrc64 = $self->getConfigValue('checkbox.crc64') eq 1 ? 'checked' : '';
my $checkiprlookup = $self->getConfigValue('checkbox.iprlookup') eq 1 ? 'checked' : '';
my $checkgoterms = $self->getConfigValue('checkbox.goterms') eq 1 ? 'checked' : '';

if($checkgoterms){
    $checkiprlookup = $checkgoterms;
}

my @appl = split(/,/, $self->getConfigValue('applications'));

my $index;
$index = "\n

<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">
<html>
 <head><link rel=\"SHORTCUT ICON\" href=\"$logo\">
  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">
  <meta name=\"KEYWORDS\" content=\"INTERPRO, DATABASE, SEARCH, SCAN\">
  <title>InterProScan (interhelp\@ebi.ac.uk)</title>
   <script langage=\"JavaScript 1.2\">

        function unselectall(number,name){
           switch(name){
            case \'appl\' :
            if(number == 1){
              for(i = 0; i < document.forms.main.appl.length; i++){
                 document.forms.main.appl[i].checked = false;
              }
            }
            if(number == 0){
              for(i = 0; i < document.forms.main.appl.length; i++){
                 document.forms.main.appl[i].checked = true;
              }
            };break;
          case \'taxonomy\' :
            if(number == 1){
              for(i = 0; i < document.forms.main.taxonomy.length; i++){
                 document.forms.main.taxonomy[i].checked = false;
              }
            }
            if(number == 0){
              for(i = 0; i < document.forms.main.taxonomy.length; i++){
                 document.forms.main.taxonomy[i].checked = true;
              }
            };break;
          }
        }

        function iprgo(num){
          if(num == 1){
           if(document.forms.main.iprlookup.checked == false && document.forms.main.goterms.checked == true){
             document.forms.main.iprlookup.checked = true;
           }
          }
          if(num == 0){
           if(document.forms.main.iprlookup.checked == false && document.forms.main.goterms.checked == true){
             document.forms.main.goterms.checked = false;
           }
          }
        }

       	function setem(){
	        if (document.cookie != ''){
			allcookies = document.cookie.split('%');
			field = allcookies[0].split('=');
			document.forms.main.emaddr.value = field[1];
                }
	}

        function cookem(){
		emaddr = document.forms.main.emaddr.value;
		expdt = new Date;
		expdt.setYear(expdt.getYear()+1);
		document.cookie = 'emaddr='+emaddr+'%expires='+expdt.toGMTString();
	}

        var trtableArray=new Array(2);
        for (i=0; i<2; i++) {
		trtableArray[i]=new Array();
        }
        trtableArray[0][0]=new Option(\"Standard Code\",\"0\");
        trtableArray[0][1]=new Option(\"Vertebrate Mitochondrial\",\"2\");
        trtableArray[0][2]=new Option(\"Yeast Mitochondrial\",\"3\");
        trtableArray[0][3]=new Option(\"Mitochondrial an Mycoplasma/Spiroplasma\",\"4\");
        trtableArray[0][4]=new Option(\"Invertebrate Mitochondrial\",\"5\");
        trtableArray[0][5]=new Option(\"Ciliate, Dasycladacean and Hexamita Nuclear\",\"6\");
        trtableArray[0][6]=new Option(\"Echinoderm Mitochondrial\",\"9\");
        trtableArray[0][7]=new Option(\"Euplotid Nuclear\",\"10\");
        trtableArray[0][8]=new Option(\"Bacterial and Plant Plastid\",\"11\");
        trtableArray[0][9]=new Option(\"Alternative Yeast Nuclear\",\"12\");
        trtableArray[0][10]=new Option(\"Ascidian Mitochondrial\",\"13\");
        trtableArray[0][11]=new Option(\"Flatworm Mitochondrial\",\"14\");
        trtableArray[0][12]=new Option(\"Blepharisma Macronuclear\",\"15\");
        trtableArray[0][13]=new Option(\"Chlorophycean Mitochondrial\",\"16\");
        trtableArray[0][14]=new Option(\"Trematode Mitochondrial\",\"21\");
        trtableArray[0][15]=new Option(\"Scenedesmus obliquus\",\"22\");
        trtableArray[0][16]=new Option(\"Thraustochytrium Mitochondrial\",\"23\");
        trtableArray[1][0]=new Option(\"None\",\"None\");

        //set trtable when you change the input sequence format
        function set_trtableArray(queryForm, sel){
        	if(sel == 1){
               	       	swapSelect(trtableArray[1],  queryForm.trtable, 0);
        		window.status=\"Protein sequence (no translation)\";
                }
                else if (sel == 2){
        	        swapSelect(trtableArray[0],  queryForm.trtable, 0);
                        window.status=\"Nucleotide sequence (all six reading frames)\";
                }
                else {
                        swapSelect(trtableArray[1],  queryForm.trtable, 0);
        	        window.status=\"Automatic sequence type detection\";
                }
        }

        function swapSelect(selectArray, dropDown, selectedOption){
            var i=0;
            var itemsToRemove= dropDown.options.length;
            for (i=itemsToRemove-1; i>=0; i--){
                dropDown.options[i]=null; // delete options
            }
            for (i=0; i<selectArray.length; i++){
               dropDown.options[i] = new Option(selectArray[i].text, selectArray[i].value);  // add new options
            }
            dropDown.options[selectedOption].selected=true;   //select appropriate option
       }

        InterPro_on  = new Image();
        InterPro_off = new Image();
        Tools_on     = new Image();
        Tools_off    = new Image();
        Ebi_on       = new Image();
        Ebi_off      = new Image();

        InterPro_on.src  = \"$interpro\";
        InterPro_off.src = \"$interpro2\";
        Tools_on.src     = \"$toolbox\";
        Tools_off.src    = \"$toolbox2\";
        Ebi_on.src       = \"$tree\";
        Ebi_off.src      = \"$tree2\";

        function switchon(name){
          switch(name){
            case\'interpro\': document.images[1].src = InterPro_on.src; break;
            case\'ebi\'     : document.images[2].src = Ebi_on.src; break;
            case\'tools\'   : document.images[3].src = Tools_on.src; break;
          }
        }

        function switchoff(name){
          switch(name){
            case\'interpro\' : document.images[1].src = InterPro_off.src;break;
            case\'ebi\'      : document.images[2].src = Ebi_off.src; break;
            case\'tools\'    : document.images[3].src = Tools_off.src; break;
          }
        }

        function checkipr4taxo(){
          iprbool = document.forms.main.iprlookup.checked;
          for(i=0; i<document.forms.main.taxonomy.length; i++){
               if(document.forms.main.taxonomy[i].checked == true){
                   iprbool = true;
                 }
            }
          document.forms.main.iprlookup.checked = iprbool;
        }

        function checktaxo4ipr(){
          if(document.forms.main.iprlookup.checked == false){
            unselectall(1,'taxonomy');
          }
        }

   </script>
  </head>
 <body  onload=\"setem()\">
 <table border=\"0\" width=\"*\">
     <tr>
       <td align=left valign=middle><img src=\"$imagepath/iprs.png\" border=\"0\">
       </td>
       <td align=\"right\" valign=\"middle\"><a href=\"http://www.ebi.ac.uk/interpro\" onMouseOver=\"switchoff(\'interpro\')\" onMouseOut=\"switchon(\'interpro\')\"><img src=\"$interpro\" alt=\"InterPro\" width=\"75\" height=\"50\" border=\"0\"></a>&nbsp;&nbsp;
                                             <a href=\"http://www.ebi.ac.uk\" onMouseOver=\"switchoff(\'ebi\')\" onMouseOut=\"switchon(\'ebi\')\"><img src=\"$tree\" alt=\"EBI\" width=\"75\" height=\"50\" border=\"0\"></a>&nbsp;&nbsp;
                                             <a href=\"http://www.ebi.ac.uk/Tools\" onMouseOver=\"switchoff(\'tools\')\" onMouseOut=\"switchon(\'tools\')\"><img src=\"$toolbox\" alt=\"Tools\@ebi\" width=\"75\" height=\"50\" border=\"0\"></a>
       </td>
     </tr>
     <tr><td colspan=\"2\"><table border=\"0\" cellpadding=\"1\" cellspacing=\"0\">
             <tr bgcolor=\"#dddddd\"><form name=\"status\" action=\"$wgeturl\" method=\"GET\">
               <td align=\"left\" valign=\"middle\">You&nbsp;can&nbsp;check&nbsp;the&nbsp;status&nbsp;of&nbsp;your&nbsp;indexed&nbsp;files
               </td>
               <td align=\"left\" valign=\"middle\"><input type=\"hidden\" name=\"index\" value=\"1\"><input type=\"submit\" value=\"Check Indexes Status\">
               </td>
               </form>
             </tr>
             <tr bgcolor=\"#dddddd\"><form name=\"status\" action=\"$wgeturl\" method=\"GET\">
               <td align=\"left\" valign=\"middle\">Query datafiles via accession number or names</td>
               <td align=\"left\" valign=\"middle\"><input type=\"hidden\" name=\"queryForm\" value=\"1\"><input type=\"submit\" value=\"Query\">
               </form>
             </tr>
             <tr bgcolor=\"#dddddd\">
               <td align=\"left\" valign=\"middle\"><a href=\"../doc/README.html\"><b>README</b></a>
                                                    &nbsp;&nbsp;&nbsp;<a href=\"../doc/FAQs.html\"><b>FAQs</b></a>
                                                    &nbsp;&nbsp;&nbsp;<a href=\"../doc/index.html\"><b>Documentation</b></a>
               </td>
               <td align=\"left\" valign=\"middle\">&nbsp;</td>
             </tr>
<!--
             <tr bgcolor=\"#dddddd\">
               <td align=\"left\" valign=\"top\"><b>Links : </b><a href=\"http://www.ebi.ac.uk/interpro\"><img src=\"$interpro\" alt=\"InterPro\"></a>&nbsp;&nbsp;<a href=\"http://www.ebi.ac.uk\"><img src=\"$tree\" alt=\"EBI\" width=\"75\" height=\"50\"></a>&nbsp;&nbsp;<a href=\"http://www.ebi.ac.uk/Tools\"><img src=\"$toolbox\" alt=\"Tools\@ebi\" width=\"75\" height=\"50\"></a></td>
               <td align=\"left\" valign=\"middle\">&nbsp;</td>
             </tr>
-->
<!--  Toolbox
             <tr bgcolor=\"#dddddd\"><form name=\"toolbox\" method=\"GET\" action=\"$wgeturl\">
               <td align=\"left\" valign=\"middle\" colspan=\"2\"><input type=\"hidden\" name=\"toolbox\" value=\"1\"><b>Access ToolBox&nbsp;:&nbsp;</b><input type=\"submit\" value=\"Go\"></td>
               </form>
             </tr>
-->
             </table>
     </td></tr>
 </table><br><br>

 <center>
 <table border=\"3\" width=\"80%\">
  <tr align=middle valign=middle>
    <td align=middle>
                <form name=\"main\" action=\"$urlserver/iprscan/iprscan\" method=\"POST\" enctype=\"multipart/form-data\">
		  <input type=\"hidden\" name=\"tool\" value=\"iprscan\">
                  <input type=\"hidden\" name=\"cmd\" value=\"submit\">
                  <table cellspacing = \"0\" cellpadding = \"4\" border=\"0\" bordercolor = \"#999999\"   class = \"tabletop\" width=\"100%\">
                    <tr >
                      <td align = \"center\" bgcolor=\"#eeeeee\"><a href=\"$imagepath/help.html\"  target=\"new_window\" >YOUR  EMAIL</a></td>
                      <td align = \"center\" bgcolor=\"#eeeeee\"><a href = \"$imagepath/help.html\"  target=\"new_window\">RESULTS</a></td>
                    </tr>
                    <tr valign=\"top\">
                      <td align = \"center\"><input type=\"text\" name = \"emaddr\" size = \"14\"></td>
                      <td align = \"center\">
                        <select   name=\"srchtype\">
                          <option value = \"interactive\" selected>interactive</option>
                          <option value = \"email\">email</option>
                        </select></td>
                    </tr>
                    <tr>
                      <td colspan=\"2\" align = \"center\" bgcolor=\"#eeeeee\"><a href=\"$imagepath/help.html\"  target=\"new_window\">APPLICATIONS TO RUN</a>
                         <input type=\"button\" onClick=\"unselectall(1, 'appl');\" value=\"Unselect all\">
                         <input type=\"button\" onClick=\"unselectall(0, 'appl');\" value=\"Select all\">
                      </td>
                    </tr>
                    <tr>  
                      <td colspan=\"2\" align = \"center\">
                        <table width=\"100%\" height=\"100%\" cellpadding=\"2\" cellspacing=\"0\">

";

my $i = 0;
my $end = 0;

$index .= "
                          <tr>
";

  foreach my $application (sort @appl){
    if($i % 5 == 0 && $i != 0){
      $end = 1;
      $index .= "
                          </tr>
                          <tr>
";
    }
    my $applurl = $self->getConfigValue("infourl.$application");
    my $name = $self->getConfigValue("appl.name.$application");
    $name = $application unless($name);
    $end = 0;
    $index .= "
                            <td>
                              <input name=\"appl\" type=\"checkbox\" class=\"text\" value=\"$application\" checked>
                              <a href=\"$applurl\" target=\"_blank\">$name</a>
                            </td>
";
  $i++;
  }
unless($end){
  $index .= "
                          </tr>
";
}

$index .= "

                        </table>
                      </td>
                    </tr>
                    <tr>
                      <td align = \"center\" class=\"tablebody\" bgcolor=\"#eeeeee\">
                        <a href=\"$imagepath/help.html\"  target=\"new_window\" class=\"tablink\">TRANSLATION</a>
                      </td>
                      <td align = \"center\" class=\"tablebody\" bgcolor=\"#eeeeee\">
                        <a href=\"$imagepath/help.html\"  target=\"new_window\" class=\"tablink\">TRANSCRIPT LENGTH THRESHOLD</a>
                      </td>
                    </tr>
                    <tr>
                      <td align=\"center\">
                        <select name=\"trtable\">
                          <option value=\"None\" selected>None</option>
                        </select>
                      </td>
                      <td align=\"center\">
                        <select name=\"trlen\">
                          <option value=\"20\">20</option>
                          <option value=\"30\">30</option>
                          <option value=\"40\">40</option>
                          <option value=\"50\" selected>50</option>
                          <option value=\"60\">60</option>
                          <option value=\"70\">70</option>
                          <option value=\"80\">80</option>
                          <option value=\"90\">90</option>
                          <option value=\"100\">100</option>
                          <option value=\"110\">110</option>
                          <option value=\"120\">120</option>
                          <option value=\"130\">130</option>
                          <option value=\"140\">140</option>
                          <option value=\"150\">150</option>
                        </select>
                       </td>
                    </tr>
                    <tr>
                      <td align=\"right\" bgcolor=\"#eeeeee\"><b>
                        Check&nbsp;<a href=\"http://swissknife.sourceforge.net/\" target=\"_blank\" >CRC64</a>&nbsp;&nbsp;
                        <input type=\"checkbox\" name=\"crc\" value=\"1\" $checkcrc64><br>
                        InterPro&nbsp;lookup&nbsp;
                        <input type=\"checkbox\" name=\"iprlookup\" value=\"1\" onClick=\"iprgo(0);checktaxo4ipr()\" $checkiprlookup><br>
                        Show&nbsp;GO&nbsp;terms&nbsp;
                        <input type=\"checkbox\" name=\"goterms\" value=\"1\" onClick=\"iprgo(1);\" $checkgoterms><br>
                      </td>
                      <td align=\"center\" bgcolor=\"#eeeeee\">
			&nbsp;
                      </td>
                    </tr>
                    <tr>
                      <td colspan=\"2\" align=\"center\" class=\"tablebody\"><b>
                      Launch&nbsp;Jobs&nbsp;:&nbsp;Parallel&nbsp;<input type=\"radio\" name=\"onego\" value=\"1\" checked>&nbsp;&nbsp;&nbsp;
                                             &nbsp;Alternatively&nbsp;<input type=\"radio\" name=\"onego\" value=\"0\"></b>
                      </td>
                    </tr>
                    <tr class = \"tablebody\">
                      <td nowrap bgcolor=\"#eeeeee\">
                              Enter or Paste a <select name=\"seqtype\" onChange=\"set_trtableArray(document.main, this.options.selectedIndex)\">
                                               <option value=\"auto\" selected>Auto</option>
                                               <option value=\"p\">Protein</option>
                                               <option value=\"n\">DNA/RNA</option>
                                               </select>
                              sequence in <a href=\"$imagepath/formats_frame.html\"  target=\"new_window\" target=\"_blank\">FASTA</a> format:
                       </td>
                       <td align=\"right\" bgcolor=\"#eeeeee\">
                              <a href = \"$imagepath/help.html\" target=\"_blank\">
                                <img src=\"$imagepath/help.gif\"   width=\"73\" height=\"27\" border = \"0\" alt = \"Help\">
                              </a>
                       </td>
                     </tr>
                     <tr>
                        <td align=\"center\" colspan=\"2\">
                          <textarea name=\"sequence\" cols=\"47\" rows=\"10\" class=\"text\" value=\"\"></textarea>
                        </td>
                     </tr>
                     <tr class = \"tablebody\">    
                        <td nowrap bgcolor=\"#eeeeee\">
                                <a href = \"$imagepath/help.html\"  target=\"new_window\"  class = \"tablink\" >Upload</A>  a file: <span  class = \"text\">
                                <input type=\"file\" name=\"upfile\" class = \"text\"></span>
                        </td>
                        <td align=right nowrap bgcolor=\"#eeeeee\">
                          <input type = \"submit\" value = \"Submit Job\" onClick = \"cookem();\"  class = \"text\">
                          <input type=\"reset\" value = \"Reset \" class = \"text\">
                        </td>
                      </tr>  
                    </table>
                </form>
    </td>
  </tr>
 </table>
 </center>

 </body>
</html>

";

return(1,\$index);

}


=head2 getNoJobsStarted

 Description: Returns an HTML page saying that no jobs have been started yet in the current chunk.
 Arguments:   $out path where to write this page otherwise write to STDOUT
 Returns:     a string

=cut

sub getNoJobsStarted ($) {

  my ($self, $out) = @_;

  $out = *STDOUT unless $out;
  my $logo = $self->getConfigValue('imgurl.icon');

  my ($res, $ofh, $oclose, $tmpfile);

  if ($self->isFileHandle($out)) {
    ($res, $oclose, $ofh) = $self->getWriteHandle($out);
  }
  else {
    $tmpfile = "$out.tmp";
    ($res, $oclose, $ofh) = $self->getWriteHandle($tmpfile);
  }

  return (0, "getNoJobsStarted: could not get write file handle $ofh") unless $res;

  my $url = $self->getConfigValue('workurl');
  return (0, "getNoJobsStarted: could not retrieve chunk number from [$out]") unless($out =~ /chunk_(\d+)/);
  my $chunk = $out =~ /chunk_(\d+)/ ? $1 : '1'; #by default we redirect to chunk_1.
  $url .= $chunk;
  my $imagepath = $self->getConfigValue('imgbase');
  
  print $ofh <<ENDHTML;
<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">
<html>
<head><link rel=\"SHORTCUT ICON\" href=\"$logo\">
<META HTTP-EQUIV="REFRESH" CONTENT="20;URL=$url">
<META HTTP-EQUIV="PRAGMA" CONTENT="NO-CACHE">
<title>No jobs started yet for chunk $chunk</title>
</head>
<body bgcolor="#FFFFFF" text="#000000">
<table width="100%" border="0" cellspacing="0" cellpadding="0" height="100%">
  <tr>
    <td align="center" valign="middle">
      <table width="450" border="0" cellspacing="0" cellpadding="0" bordercolor="#999999">
       <tr>
	 <td align="center">
	   <img src="$imagepath/iprs.png">
	 </td>
       </tr>
       <tr><td>&nbsp;</td></tr>
	<tr>
	  <td align="center">
          <b><font face="Arial, Helvetica, sans-serif" color="#000000" size="2">
	  <nobr>No Jobs have been launched yet for the chunk $chunk</nobr><br></font></b>
	  <b><font face="Arial, Helvetica, sans-serif" color="#000000" size="2">
	  When jobs start, you will see a polling page.</font></b>
	  <br>
       </tr>
       <tr>
	  <td align="center">
          <b><font face="Arial, Helvetica, sans-serif" color="#000000" size="2">
          To check if the job has been launched click <a href="$url">here</a> or wait for the page to refresh.
          </font></b>
          </td>
       </tr>
     </table>
   </td>
  </tr>
</table>
</body>
</html>
ENDHTML

  close $ofh if $oclose;

  unless(rename("$tmpfile", "$out")){
    return(0, "getNoJobsStarted: Could not rename $tmpfile: $!");
  }

  return(1, '');
}


=head2 getHtmlTop

 Description: Returns top part of an HTML page.
 Arguments:   -
 Returns:     string

=cut

sub getHtmlTop() {
    my $self = shift;

#    return topWithServicesHelpMenu($self->getConfigValue('htmlhelper.menuitem'),
#				    $self->getConfigValue('htmlhelper.title'),
#				    $self->getConfigValue('htmlhelper.pagetitle'),
#				    $self->getConfigValue('htmlhelper.pageurl'),
#				    $self->getConfigValue('htmlhelper.firstword'),
#				    $self->getConfigValue('htmlhelper.lastword'),
#				    $self->getConfigValue('htmlhelper.loadmethod'),
#				    $self->getConfigValue('htmlhelper.unloadmethod'),
#				    $self->getConfigValue('htmlhelper.script'),
#				    ["pop",
#				     $self->getConfigValue('htmlhelper.helplinkname'),
#				     $self->getConfigValue('htmlhelper.helplinkurl')]);
    return '<html><head><title>InterProScan Results - interhelp@ebi.ac.uk</title></head>';
}


=head2 getHtmlBottom

 Description: Returns bottom part of an HTML page.
 Arguments:   -
 Returns:     string

=cut

sub getHtmlBottom() {
    my $self = shift;
#    return bottom();
    return '';
}


=head2 getResultPageTop

 Description: Returns the top part of the HTML result page.
 Arguments:   $cnk Chunk number to point to the right files
 Returns:     top part string

=cut

sub getResultPageTop($) {
    my ($self, $cnk) = @_;

    #The user asked for html output using command line. No need to display buttons...
    return '' if($self->getParam('format') =~ /^htm/);

    my $top     = $self->getHtmlTop();
    my $tname   = $self->getToolName();
    my $jobid   = $self->getParam('jobid');
    my $cmd     = $self->getParam('cmd');
    my $view    = $self->getParam('view');
    my $dview   = $self->getParam('view');  #the default view the user is looking at.
    my $workurl = $self->getConfigValue('workurl');
    my $rawurl  = $self->getConfigValue('rawurl');
    my $txturl  = $self->getConfigValue('txturl');
    my $xmlurl  = $self->getConfigValue('xmlurl');
    my $sequrl  = $self->getConfigValue('origsequrl');
    my $suburl  = $self->getConfigValue('submiturl');
    my $repfile = $self->getConfigValue('reporturl');
    my $reportfile = $self->getConfigValue('toolreport');
    my $logo    = $self->getConfigValue('imgurl.icon');

    my $params = { };

    my $link2cnk = "";

    if ($view eq 'table') {
        $view = 'Picture';
    }
    elsif ($view eq 'picture') {
        $view = 'Table';
    }

    unless(opendir(CNK, $self->getConfigValue('workdir'))){
      return(0, "getResultPageTop: Could not open directory " . $self->getConfigValue('workdir') . ": $!");
    }
    my @cnk =  sort { $a <=> $b } grep { s/chunk_(\d+)/$1/o } readdir CNK;
    closedir CNK;
    if($#cnk > 0){
      $link2cnk = "<table border=\"1\" bgcolor=\"#eeeeee\" cellSpacing=\"0\" cellPadding=\"3\" bordercolor=\"#999999\">\n<tr>\n<td>\n<table border=\"0\">\n<tr>\n<td align=\"left\">\n<b>See&nbsp;results&nbsp;for&nbsp;chunk&nbsp;:&nbsp;</b>\n</td>\n<td align=\"left\"><b>";
      my $cnt = 0;
      my $limit = $self->getConfigValue('chunk.display') || '50';
      for(@cnk){
	if($cnt == $limit){
	  $cnt=0;
	  $link2cnk .= "</b>\n</td>\n</tr>\n<tr>\n<td>&nbsp;</td>\n<td align=\"left\"><b>";
	}
	
	$link2cnk .= ($cnk == $_) ? "$_&nbsp;&nbsp;" : "<a href=\"$workurl$_&view=$dview\">$_</a>&nbsp;&nbsp;";
	$cnt++;
      }
      $link2cnk .= "</td>\n</tr>\n</table></td>\n</tr>\n</table>\n\n";
    }

    if($cnk){
      my ($res, $msg) = $self->addChunk("$cnk", \$rawurl, \$txturl, \$xmlurl, \$reportfile, \$repfile);
      return(0, "getResultsPageTop: could not add chunk $msg") unless($res);
      $workurl .= "$cnk";
    }

    #Check if we have some errors with some jobs.
    my $report = "";

    if($repfile && -f $reportfile){
      my $critical  = $self->getConfigValue('imgurl.critical');
      my $submiturl = $self->getConfigValue('resubmiturl');
      my $email     = $self->getParam('email');
      my $crc       = $self->getParam('crc');
      my $iprlkup   = $self->getParam('iprlookup');
      my $goterms   = $self->getParam('goterms');
      my $stype     = $self->getParam('stype');
      my $upfile    = $self->getConfigValue('toolinput');
      
      $report = "
<table border=\"2\">
<tr><td>
<table border=\"0\">
<tr>
<td align=left colspan=\"2\" nowrap><img src=\"$critical\" border=\"0\" alt=\"Errors\">&nbsp;<b>Errors occured in some jobs.<br>Please read report below and fix errors.</b></td>
</tr>
<tr>
<td> <form method=\"GET\" action=\"$repfile\">
    <input type=\"submit\" value=\"See Errors\">
    </form>
</td>
<td align=\"left\">
    <form action=\"$submiturl\" method=\"POST\" enctype=\"multipart\/form-data\">\n"; #to debug

      $params = $self->getParams();
      $report .= "    <input type=\"hidden\" name=\"jobid\" value=\"" . $params->{'jobid'} . "\">\n";
      $report .= "    <input type=\"hidden\" name=\"tool\" value=\"" . $self->getToolName() . "\">\n";
      $report .= "    <input type=\"hidden\" name=\"YYYY\" value=\"" . $params->{'YYYY'} . "\">\n";
      $report .= "    <input type=\"hidden\" name=\"MM\" value=\"" . $params->{'MM'} . "\">\n";
      $report .= "    <input type=\"hidden\" name=\"DD\" value=\"" . $params->{'DD'} . "\">\n";
      $report .= "    <input type=\"hidden\" name=\"view\" value=\"" . $params->{'view'} . "\">\n";

      #We give the workdir so that Resubmit script can directly read params from the param file in the workdir.
      #It avoids to passes all the parameters through the url (can crash if too long).
      $report .= "    <input type=\"hidden\" name=\"toolparams\" value=\"" . $self->getConfigValue('toolparams') . "\">\n";
      $report .= "    <input type=\"hidden\" name=\"srchtype\" value=\"" . $params->{'srchtype'} . "\">\n";
      $report .= "    <input type=\"hidden\" name=\"cmd\" value=\"submit\">\n";
      $report .= "    <input type=\"submit\" value=\"Resubmit failed jobs\">
    </form>
</td>
</tr>
</table>
</td></tr></table>
<br>
\n";
    }


    my $buttons = <<END_OF_HTML;
<link rel=\"SHORTCUT ICON\" href=\"$logo\">
$report
$link2cnk
<br>
<span class="text">
<table border="0">
<tr>
  <td>
    <form method="GET" action="$workurl">
    <input type="hidden" name="tool" value="$tname">
    <input type="hidden" name="jobid" value="$jobid">
    <input type="hidden" name="cmd" value="$cmd">
    <input type="hidden" name="view" value="$view">
    <input type="hidden" name="cnk" value="$cnk">
    <input type="submit" value="$view View">
    </form>
  </td>
  </td>
  <td>
    <form method="GET" action="$rawurl">
    <input type="submit" value="Raw Output">
    </form>
  </td>
  <td>
    <form method="GET" action="$txturl">
    <input type="submit" value="Text Output">
    </form>
  </td>
  <td>
    <form method="GET" action="$xmlurl">
    <input type="submit" value="XML Output">
    </form>
  </td>
  <td>
    <form method="GET" action="$sequrl">
    <input type="submit" value="Original Sequences">
    </form>
  </td>
  <td>
    <form method="GET" action="$suburl">
    <input type="submit" value="SUBMIT ANOTHER JOB">
    </form>
  </td>
</tr>
</table>
<br>
END_OF_HTML
    return $top . $buttons;
}


=head2 getResultPageBottom

 Description: Returns the bottom part of the HTML result page.
 Arguments:   -
 Returns:     bottom part string

=cut

sub getResultPageBottom($) {
    my ($self, $cnk) = @_;

    #The user asked for html output using command line. No need to display buttons...
    return '' if($self->getParam('format') =~ /^htm/);

    my $bottom = $self->getHtmlBottom();

    my $tname   = $self->getToolName();
    my $jobid   = $self->getParam('jobid');
    my $cmd     = $self->getParam('cmd');
    my $view    = $self->getParam('view');
    my $dview   = $self->getParam('view');  #the default view the user user is looking at.
    my $workurl = $self->getConfigValue('workurl');
    my $rawurl  = $self->getConfigValue('rawurl');
    my $txturl  = $self->getConfigValue('txturl');
    my $xmlurl  = $self->getConfigValue('xmlurl');
    my $sequrl  = $self->getConfigValue('origsequrl');
    my $suburl  = $self->getConfigValue('submiturl');

    my $link2cnk = "";
    if ($view eq 'table') {
        $view = 'Picture';
    }
    elsif ($view eq 'picture') {
        $view = 'Table';
    }

    if ($view eq 'table') {
        $view = 'Picture';
    }
    elsif ($view eq 'picture') {
        $view = 'Table';
    }

    unless(opendir(CNK, $self->getConfigValue('workdir'))){
      return(0, "getResultPageBottom: Could not open directory " . $self->getConfigValue('workdir') . ": $!");
    }
    my @cnk = sort { $a <=> $b } grep { s/chunk_(\d+)/$1/o } readdir CNK;
    closedir CNK;
    if($#cnk > 0){
      $link2cnk = "<table border=\"1\" bgcolor=\"#eeeeee\" cellSpacing=\"0\" cellPadding=\"3\" bordercolor=\"#999999\">\n<tr>\n<td>\n<table border=\"0\">\n<tr>\n<td align=\"left\">\n<b>See&nbsp;results&nbsp;for&nbsp;chunk&nbsp;:&nbsp;</b></font>\n</td>\n<td align=\"left\"><b>";
      my $cnt = 0;
      my $limit = $self->getConfigValue('chunk.display') || '50';
      for(@cnk){
	if($cnt == $limit){
	  $cnt=0;
	  $link2cnk .= "</b>\n</td>\n</tr>\n<tr>\n<td>&nbsp;</td>\n<td align=\"left\"><b>";
	}

	$link2cnk .= ($cnk == $_) ? "$_&nbsp;&nbsp;" : "<a href=\"$workurl$_&view=$dview\">$_</a>&nbsp;&nbsp;";
	$cnt++;
      }
    }

    $link2cnk .= "</td>\n</tr>\n</table></td>\n</tr>\n</table>\n\n";

    if($cnk){
      my ($res, $msg) = $self->addChunk("$cnk", \$rawurl, \$txturl, \$xmlurl);
      return(0, $msg) unless($res);
      $workurl .= "$cnk";
    }

    my $buttons = <<END_OF_HTML;
<br>
<table border="0">
<tr>
  <td>
    <form method="GET" action="$workurl">
    <input type="hidden" name="tool" value="$tname">
    <input type="hidden" name="jobid" value="$jobid">
    <input type="hidden" name="cmd" value="$cmd">
    <input type="hidden" name="view" value="$view">
    <input type="hidden" name="cnk" value="$cnk">
    <input type="submit" value="$view View">
    </form>
  </td>
  <td>
    <form method="GET" action="$rawurl">
    <input type="submit" value="Raw Output">
    </form>
  </td>
  <td>
    <form method="GET" action="$txturl">
    <input type="submit" value="Text Output">
    </form>
  </td>
  <td>
    <form method="GET" action="$xmlurl">
    <input type="submit" value="XML Output">
    </form>
  </td>
  <td>
    <form method="GET" action="$sequrl">
    <input type="submit" value="Original Sequences">
    </form>
  </td>
  <td>
    <form method="GET" action="$suburl">
    <input type="submit" value="SUBMIT ANOTHER JOB">
    </form>
  </td>
</tr>
</table>
<br>
$link2cnk
</span>
END_OF_HTML
    return $buttons . $bottom;
}


=head2 createStatusPage

 Description: Creates an HTML status page.

              NOTE 1: HTTP headers must be added by the caller, because it is
              impossible to know when they should be added (caller may have
              started the page and this call finishes it).

              NOTE 2: If you use stdout and have created a poll page for the URL,
              beware that the poll page will be overwritten and the user will get
              an empty page as soon as this sub starts writing the result. In such
              cases it is better to pass $in as a path, because this method creates
              a temporary file and renames it when the file has been written.

 Arguments:   $in        Job result.
              $out       Output (default: stdout).
              $top       Page top (default: getHtmlTop).
              $bottom    Page bottom (default: getHtmlBottom).

 Returns:     1, ''  on success
              0, msg on failure

=cut

sub createStatusPage($;$$$) {
    my ($self, $in, $out, $top, $bottom) = @_;
    return (0, "createStatusPage: no job result file given")  unless $in;
    $out = \*STDOUT unless $out;

    my $params = $self->getParams() || { };
    my ($res, $ifh, $ofh, $iclose, $oclose, $tmpfile);

    ($res, $iclose, $ifh) = $self->getReadHandle($in);
    return (0, "createStatusPage: cannot get read handle $ifh") unless $res;

    if ($self->isFileHandle($out)) {
        ($res, $oclose, $ofh) = $self->getWriteHandle($out);
    }
    else {
        $tmpfile = "$out.tmp";
        ($res, $oclose, $ofh) = $self->getWriteHandle($tmpfile);
    }

    unless ($res) {
        close $ifh if $iclose;
        return (0, "createStatusPage: $ofh");
    }

    if ($top) {
        print $ofh $top;
    }
    else {
        print $ofh $self->getHtmlTop();
    }

    my ($jobname, $status, $msg);
    my $report = { };

    while(<$ifh>) {
        ($jobname, $status) = split /=/, $_;
        next unless $jobname && $status;
        $report->{$jobname} = $status;
    }

    my $queue = new Dispatcher::Queue();
    ($status, $msg) = $queue->check($self->getToolName(), $self->getConfigValue('jobid'));
    return (0, "createStatusPage: $msg") unless $status;

    my ($tool, $result, $seqid);
    my $jobid = $self->getParam('jobid');

    print $ofh "<br>\n<span class=\"text\">\n";
    print $ofh "You can <a href=\"";
    print $ofh $self->getConfigValue('workurl');
    print $ofh "\">refresh this page</a> to monitor the status of your job.\n<br><br>";

    print $ofh "<table width=\"460\" cellSpacing=\"0\" cellPadding=\"3\" border=\"1\" bordercolor=\"#999999\" bgcolor=\"#eeeeee\" class=\"tabletop\">\n";
    print $ofh "<tr class=\"tablebody\">\n";
    print $ofh "<td>JOB $jobid</td>\n";
    print $ofh "<td>";
    print $ofh "<b>" . $self->mapStatus($status) . "</b>";

    if ($status == Dispatcher::Queue::JOB_FAILED) {
        my $efile = $self->getConfigValue('toolerrors');
        if ($efile && -f $efile && open(ERRORS, "< $efile")) {
            print $ofh "<pre>\n";
            while (<ERRORS>) { print $ofh $_; }
            print $ofh "</pre>\n";
        }
    }

    print $ofh "</td>\n</tr>\n";

    foreach $jobname (sort keys %$report) {
        $status = $report->{$jobname};
        print $ofh "<tr>\n\n";

        if ($jobname =~ /^[A-Za-z]+-[\d]{8}-[\d]{8}-([A-Za-z]+)-(\d+)/) {
            $seqid = $self->getParam("seq.$2.id");
            print $ofh "<td class=\"text\" nowrap>$seqid $1</td>\n";
            $status = $self->mapStatus($status);
            print $ofh "<td class=\"text\" nowrap>$status</td>\n";
        }
        else {
            print $ofh "<td class=\"text\" nowrap colspan=\"3\">Unrecognized job: $jobname ($status)</td>\n";
        }

        print $ofh "</tr>\n";
    }

    print $ofh "</table>\n</span>\n";

    if ($bottom) {
        print $ofh $bottom;
    }
    else {
        print $ofh $self->getResultPageBottom();
    }

    close $ifh if $iclose;
    close $ofh if $oclose;

    if ($tmpfile) {
        unless (rename($tmpfile, $out)) {
            return (0, "createStatusPage: could not rename temporary file $tmpfile: $!");
        }
    }

    return (1, '');
}


=head2 createResultPicture

 Description: Creates an HTML result page.

              NOTE 1: HTTP headers must be added by the caller, because it is
              impossible to know when they should be added (caller may have
              started the page and this call finishes it).

              NOTE 2: If you use stdout and have created a poll page for the URL,
              beware that the poll page will be overwritten and the user will get
              an empty page as soon as this sub starts writing the result. In such
              cases it is better to pass $in as a path, because this method creates
              a temporary file and renames it when the file has been written.

 Arguments:   $in        InterProScan XML result file.
              $out       Output (default: stdout).
              $top       Page top (default: getResultPageTop).
              $bottom    Page bottom (default: getResultPageBottom).

 Returns:     1, ''  on success
              0, msg on failure

=cut

sub createResultPicture($;$$$) {
    my ($self, $in, $out, $top, $bottom) = @_;
    return (0, "createResultPicture: no job result file given")  unless $in;
    $out = \*STDOUT unless $out;

    my $params = $self->getParams() || { };
    my ($res, $msg, $ofh, $oclose, $tmpfile);

    if ($self->isFileHandle($out)) {
        ($res, $oclose, $ofh) = $self->getWriteHandle($out);
    }
    else {
        $tmpfile = "$out.tmp";
        ($res, $oclose, $ofh) = $self->getWriteHandle($tmpfile);
    }

    return (0, "createResultPicture: Cannot get file handle for writing: $ofh") unless $res;

    my $cnk;
    unless($self->getParam('format') =~ /^htm/){
	$cnk = ($in =~ /chunk_(\d+)/ ? $1 : 1);
    }

    my $parser = new Dispatcher::Tool::InterProScanParser();

    # InterProScanParser::parse() returns 1 only when ok, 1 + msg when warnings, 0 when not ok
    ($res, $msg) = $parser->parse($in);
    unless ($res) {
      close $ofh if $oclose;
      return (0, "createResultPicture: Cannot parse XML file $in: $msg");
    }

    if ($res && $msg) {
        close $ofh if $oclose;
        return (0, "createResultPicture: Cannot parse XML file: $msg");
    }

    if ($top) {
        print $ofh $top;
    }
    else {
        print $ofh $self->getResultPageTop($cnk);
    }

    my $maxwidth   = $self->getParam('imgwidth') || $self->getConfigValue('defaultimgwidth') || '500';
    my $imgline    = $self->getConfigValue("imgurl.line");
    my $imgend     = $self->getConfigValue("imgurl.end");
    my $imglogo    = $self->getConfigValue("imgurl.interpro");
    my $imgsrslogo = $self->getConfigValue("imgurl.srs");
    my $imgzoomin  = $self->getConfigValue("imgurl.zoomin");
    my $imgzoomout = $self->getConfigValue("imgurl.zoomout");
    my $iprurl     = $self->getConfigValue("entryurl.interpro");
    my $iprsrsurl  = $self->getConfigValue("entryurl.interpro.srs"); #We maybe can remove this from config file? Ask User during installation
    my $noiprid    = $self->getConfigValue('unintegrated.id');
    my $noiprname  = $self->getConfigValue('unintegrated.name');
    my $noiprtype  = $self->getConfigValue('unintegrated.type');
    my $wgeturl    = $self->getConfigValue('wgeturl');
    my $nbchunk    = $self->getConfigValue('chunk');
    my $nseqs = $self->getParam('nseqs');

    my $sequences  = { };
    my $toplen     = 0;  # longest protein (no zoom for short proteins)
    my ($seqid, $sequrl, $entryurl, $iprlookup, $crc);

    $iprlookup = $self->getParam('iprlookup');           #EQ added.
    $crc       = $self->getParam('crc');                 #EQ added.

    #We need to do some calculation
    my $rest;
    if($self->getParam('format') =~ /^htm/){
	return(0, "createResultPicure: nseqs not found in params file") unless $nseqs;
	$rest = int($nseqs/$nbchunk);
	$rest++ if($nseqs%$nbchunk);
    }


    my $start = ($cnk ? $cnk : 1);
    my $stop  = ($cnk ? $cnk : $rest);

    for($start; $start <= $stop; $start++){

	for (my $i = 1; $i <= $nbchunk; $i++) {

	    $cnk = $start;

	    $seqid     = $self->getParam("seq.$cnk.$i.id");
	    $sequrl    = $self->getConfigValue("iprscansequrl");

	    next unless($seqid);

	    my ($res, $protein) = $parser->getProtein($seqid);
	    my $error = $protein unless $res;

	    unless ($res) {
		$protein = { id     => $seqid,
			     length => $self->getParam("seq.$cnk.$i.len"),
			     crc64  => $self->getParam("seq.$cnk.$i.crc64") };
	    }

        print $ofh "<table width=\"100%\" cellSpacing=\"0\" cellPadding=\"3\" border=\"1\"";
        print $ofh " bordercolor=\"#999999\" bgcolor=\"#eeeeee\" class=\"tabletop\">\n";
        print $ofh "<tr class=\"tablebody\">\n";

        if ($sequrl) {
	  #We want entry from input sequence
	  $wgeturl =~ s/%toolname/iprscan/;
	  print $ofh "<td colspan=\"2\"><b>SEQUENCE:</b>&nbsp;<b><a href=\"$wgeturl$protein->{id}\" target=\"newwindow\">$protein->{id}</a></b>\n";
        }
        else {
            print $ofh "<td colspan=\"2\"><b>SEQUENCE:</b>&nbsp;<b>$protein->{id}</b>\n";
        }

        print $ofh "<b>CRC64:</b>&nbsp;$protein->{crc64}\n";
        print $ofh "<b>LENGTH:</b>&nbsp;$protein->{length}&nbsp;aa\n";

	#If user setted a webserver (workserver defined in iprscan.conf) then we can display Zoom(in/out) pictures
        if ($protein->{length} > $self->getConfigValue('defaultimgwidth') && $self->getConfigValue('workserver')) {
            my $zin  = $maxwidth + 100;
            my $zout = $maxwidth - 100;
            my $url  = $self->getConfigValue('pictureurl');
	    $url =~ s/(cnk=)/$1$cnk/;

            print $ofh "&nbsp;&nbsp;<a href=\"$url&imgwidth=$zin\">";

            if ($imgzoomin) {
                print $ofh "<img border=\"0\" src=\"$imgzoomin\"></a>";
            }
            else {
                print $ofh "+</a>";
            }

            print $ofh "&nbsp;&nbsp;<a href=\"$url&imgwidth=$zout\">";

            if ($imgzoomout) {
                print $ofh "<img border=\"0\" src=\"$imgzoomout\"></a>";
            }
            else {
                print $ofh "-</a>";
            }
	  }

        print $ofh "</td>\n";
        print $ofh "</tr>\n";


        my $length = $protein->{length};
        $toplen = $length if $length > $toplen;

        my $interpros = $protein->{interpros};
        my $uimatches = $protein->{matches};  # unintegrated matches (not in INTERPRO)

        if ($uimatches && @$uimatches > 0) {     #there are some unintegrated entries...
            my $unintegrated = { id      => $noiprid,
                                 name    => $noiprname,
                                 type    => $noiprtype,
                                 matches => $uimatches };
            push @$interpros, $unintegrated;
        }

        unless ($interpros) {
            print $ofh "<tr>\n";
            print $ofh "<td colspan=\"2\">No hits reported.</td>\n";
            print $ofh "</tr>\n";
        }

        my $ninterpro = 1;

        foreach my $interpro (@$interpros) {
            if ($ninterpro % 2 == 0) {
                print $ofh "<tr bgcolor=\"#dddddd\">\n";
            }
            else {
                print $ofh "<tr>\n";
            }

	    if($iprlookup || $crc){     ## If user doesn't want to see InterPro entries
	      unless ($interpro->{id} eq $noiprid) {
                print $ofh "<td valign=\"top\">";
                print $ofh "<b>InterPro<br>\n</b>";
                print $ofh "<a href=\"$iprurl$interpro->{id}\" target=\"newwindow\">$interpro->{id}</a><br>$interpro->{type}\n";
		
		if ($iprsrsurl) {
		  $iprsrsurl =~ s/\$id/$interpro->{id}/g;
		  print $ofh "<br>\n";
		  print $ofh "<a href=\"$iprsrsurl\" target=\"newwindow\">";
		  
		  if ($imgsrslogo) {
		    print $ofh "<img src=\"$imgsrslogo\" border=\"0\" width=\"66\" height=\"44\">";
		  }
		  else {
		    print $ofh "Link&nbsp;to&nbsp;SRS";
		  }
		  
		  print $ofh "</a>\n";
		}
		
                print $ofh "</td>\n";
	      } #end unless
	      else {
                # no link for the unintegrated
                print $ofh "<td valign=\"top\">$interpro->{id}<br>$interpro->{type}</td>\n";
	      }
	    }

            my $matches = $interpro->{matches};
            my $nmatch  = 1;
	    my $urlid;
	    my $colspan = 4;

	    $colspan-- if($crc);

            print $ofh "<td valign=\"top\">\n";
            print $ofh "<table width=\"100%\" cellSpacing=\"0\" cellPadding=\"2\" border=\"0\">\n";

	    #If user asked for InterPro lookup we display InterPro name
	    if($iprlookup || $crc){
	      print $ofh "<tr>\n";
	      print $ofh "<td valign=\"top\" colspan=\"$colspan\">\n";
	      print $ofh "<b><nobr>$interpro->{name}</nobr></b>\n";
	      print $ofh "</td>\n";
	      print $ofh "</tr>\n";
	    }

	    my $seen;
	    my %db;
	    for(@$matches){
	      $db{$_->{dbname}}++;
	    }

            foreach my $match (@$matches) {
                print $ofh "<tr bgcolor=\"#ffffff\">\n";

                $entryurl = $self->getConfigValue("entryurl." . lc($match->{dbname}));

		my $wgeturlid = $self->getConfigValue('wgeturl');
		my $tool = lc($match->{locations}[0]->{evidence});
		$tool = $self->mapApplNameToToolName($tool) unless($tool =~ /^hmm/);
		my $paurl = $self->getConfigValue('imgurl.pa');
		#my $pdbview = $self->getConfigValue('url.pdbview') if($tool && $tool =~ /profilescan/i);
		#my $molecule = $self->getConfigValue('imgurl.mol') if($tool && $tool =~ /profilescan/i);
		my $rowspan = $db{$match->{dbname}};
		
		$wgeturlid =~ s/%toolname/$tool/;
		$wgeturlid =~ s/\.input/\-$tool\.output/;

		if($cnk){
		    ($res, $msg) = $self->addChunk("$cnk", \$wgeturlid);
		    return(0, "createResultPicture: Cannot add chunk $cnk: $msg") unless $res;
		    $wgeturlid =~ s/\-$tool\.output/\-$tool\-cnk$cnk\.output/;
		}

		my $html;

		if($wgeturlid && !$crc && $seen ne $tool && $self->getParam('format') !~ /^htm/){
		    $html = "<td rowspan=\"$rowspan\" width=\"50\"><a href=\"$wgeturlid$protein->{id}\" target=\"newwindow\"><img src=\"$paurl\" border=\"0\"></a></td>";
		    $seen = $tool;
		}

		$html .= "<td width=\"10%\">";


		if($entryurl){
		    $urlid = $match->{id};
		    $urlid =~ s/\.\d+$// if (lc($match->{dbname}) eq 'pfam'); #remove Pfam version number

		    $html .= "<a href=\"$entryurl$urlid\" target=\"newwindow\">$match->{id}</a>";
		    ## For further implementations......allow to link ProfileScan entry with 3D/RasMol/PDB entry (MSD at EBI)
		    #  if($pdbview){
		    #    $html .= "&nbsp;<a href=\"$pdbview$match->{id}\" target=\"newwindow\"><img src=\"$molecule\" border=\"0\"></a>";
		    #  }		
		} else {
		    $html .= "$match->{id}";
		}

		$html .= "</td>\n";
		
		print $html;
                print "<td width=\"$maxwidth\"><nobr>";

                my $locations = $match->{locations};
                my ($start, $end, $score, $status, $evidence, $imgurl, $alt, $width, $scale);
                my $pos = 1;

                # scale the image width for long proteins
                if ($length > $maxwidth) {
                    $scale = $maxwidth / $length;
                }
                else {
                    $scale = 1;
                }

                foreach my $location (sort byStartPosition @$locations) {
                    $start    = $location->{start};
                    $end      = $location->{end};
                    $score    = $location->{score};
                    $status   = $location->{status};
                    $evidence = $location->{evidence};
                    $imgurl   = $self->getConfigValue("imgurl." . lc($evidence) . ".$status");
                    $imgurl   = $self->getConfigValue("imgurl.default") unless $imgurl;
                    $alt      = "[$start-$end]$status";
                    $alt     .= " $score" if defined $score;
                    $alt     .= " $match->{id} $match->{name}";

                    $start = int($start * $scale);
                    $end   = int($end * $scale);

                    if ($start > $pos + 1) {
                        $width = $start - $pos - 1;
                        print $ofh "<img src=\"$imgline\" width=\"$width\" height=\"7\">";
                        $pos = $start;
                    }

                    $width = $end - $pos - 1;
                    print $ofh "<img src=\"$imgend\" width=\"1\" height=\"7\">";
                    print $ofh "<img src=\"$imgurl\" width=\"$width\" height=\"7\" onMouseOver=\"window.status='$alt'; return true\" alt=\"$alt\">";
                    print $ofh "<img src=\"$imgend\" width=\"1\" height=\"7\">";
                    $pos = $end;
                }

                if ($pos < int($length * $scale)) {
                    $width = int($length * $scale) - $pos - 1;
                    print $ofh "<img src=\"$imgline\" width=\"$width\" height=\"7\">";
                }

                print $ofh "</nobr></td>\n";
                print $ofh "<td>$match->{name}</td>\n";
                print $ofh "</tr>\n";
            }

            print $ofh "</table>\n</td>";
            print $ofh "</tr>\n";
            $ninterpro++;
        }

        print $ofh "</table>\n";
        print $ofh "<br>\n";

      } #end for loop
				       } # end for($start; $start <= $stop; $start++)

    if ($bottom) {
        print $ofh $bottom;
    }
    else {
        print $ofh $self->getResultPageBottom($cnk);
    }

    close $ofh if $oclose;

    if ($tmpfile) {
        unless (rename($tmpfile, $out)) {
            return (0, "createResultPicture: could not rename temporary file $tmpfile: $!");
        }
    }

    return (1, '');
}


=head2 createResultTable

 Description: Creates an HTML result page.

              NOTE 1: HTTP headers must be added by the caller, because it is
              impossible to know when they should be added (caller may have
              started the page and this call finishes it).

              NOTE 2: If you use stdout and have created a poll page for the URL,
              beware that the poll page will be overwritten and the user will get
              an empty page as soon as this sub starts writing the result. In such
              cases it is better to pass $in as a path, because this method creates
              a temporary file and renames it when the file has been written.

 Arguments:   $in        InterProScan XML result file.
              $out       Output (default: stdout).
              $top       Page top (default: getResultPageTop).
              $bottom    Page bottom (default: getResultPageBottom).

 Returns:     1, ''  on success
              0, msg on failure

=cut

sub createResultTable($;$$$) {
    my ($self, $in, $out, $top, $bottom) = @_;
    return (0, "createResultTable: no job result file given")  unless $in;
    $out = \*STDOUT unless $out;

    my $params = $self->getParams() || { };
    my ($res, $msg, $ofh, $oclose, $tmpfile);

    if ($self->isFileHandle($out)) {
        ($res, $oclose, $ofh) = $self->getWriteHandle($out);
    }
    else {
        $tmpfile = "$out.tmp";
        ($res, $oclose, $ofh) = $self->getWriteHandle($tmpfile);
    }

    return (0, "createResultTable: Cannot open filehandle $ofh") unless $res;

    my $cnk;
    unless($self->getParam('format') =~ /^htm/){
	$cnk = $in =~ /chunk_(\d+)/ ? $1 : 1;
    }

    my $parser = new Dispatcher::Tool::InterProScanParser();
    ($res, $msg) = $parser->parse($in);
    unless ($res) {
        close $ofh if $oclose;
        return (0, "createResultTable: Cannot parse input $in: $msg");
    }

    if ($top) {
        print $ofh $top;
    }
    else {
        print $ofh $self->getResultPageTop($cnk);
    }

    my $proteins   = $parser->getProteins();
    my $imglogo    = $self->getConfigValue("imgurl.interpro");
    my $imgsrslogo = $self->getConfigValue("imgurl.srs");
    my $iprurl     = $self->getConfigValue("entryurl.interpro");
    my $iprsrsurl  = $self->getConfigValue("entryurl.interpro.srs");
    my $noiprid    = $self->getConfigValue('unintegrated.id'),
    my $noiprname  = $self->getConfigValue('unintegrated.name'),
    my $noiprtype  = $self->getConfigValue('unintegrated.type'),
    my $wgeturl    = $self->getConfigValue('wgeturl');   #EQ added
    my $nbchunk    = $self->getConfigValue('chunk');     #EQ added
    my $nseqs      = $self->getParam('nseqs');

    my ($seqid, $sequrl, $entryurl, $iprlookup, $crc, $goterms);

    $iprlookup = $self->getParam('iprlookup');           #EQ added.
    $crc       = $self->getParam('crc');                 #EQ added.
    $goterms   = $self->getParam('goterms');             #EQ added.

    #We need to do some calculation
    my $rest;
    if($self->getParam('format') =~ /^htm/){
	return(0, "createResultTable: nseqs not found in params file") unless $nseqs;
	$rest = int($nseqs/$nbchunk);
	$rest++ if($nseqs%$nbchunk);
    }

    my $start = ($cnk ? $cnk : 1);
    my $stop  = ($cnk ? $cnk : $rest);

    for($start; $start <= $stop; $start++){

	$cnk = $start;

	for (my $i = 1; $i <= $nbchunk; $i++) {
	    $seqid  = $self->getParam("seq.$cnk.$i.id");
	    $sequrl = $self->getConfigValue("iprscansequrl");

	    next unless $seqid;
	    
	    my ($res, $protein) = $parser->getProtein($seqid);
	    my $error = $protein unless $res;

	    unless ($res) {
		$protein = { id     => $seqid,
			     length => $self->getParam("seq.$cnk.$i.len"),
			     crc64  => $self->getParam("seq.$cnk.$i.crc64") };
	    }

        print $ofh "<table width=\"100%\" cellSpacing=\"0\" cellPadding=\"3\" border=\"1\"";
        print $ofh " bordercolor=\"#999999\" bgcolor=\"#eeeeee\" class=\"tabletop\">\n";
        print $ofh "<tr class=\"tablebody\">\n";

        if ($sequrl) {
	    #We want entry from input sequence
	    $wgeturl =~ s/%toolname/iprscan/;
	    print $ofh "<td colspan=\"4\"><b>SEQUENCE:</b>&nbsp;<b><a href=\"$wgeturl$protein->{id}\" target=\"newwindow\">$protein->{id}</a></b>\n";
	    #print $ofh "<td colspan=\"3\"><b>SEQUENCE:</b>&nbsp;<a href=\"$sequrl\" target=\"newwindow\">$protein->{id}</a>\n";
        }
        else {
            prinzt $ofh "<td colspan=\"4\"><b>SEQUENCE:</b>&nbsp;<b>$protein->{id}</b>\n";
        }

        print $ofh "<b>CRC64:</b>&nbsp;$protein->{crc64}\n";
        print $ofh "<b>LENGTH:</b>&nbsp;$protein->{length}&nbsp;aa\n";
        print $ofh "</td>\n";
        print $ofh "</tr>\n";

        my $interpros = $protein->{interpros};
        my $uimatches = $protein->{matches};  # unintegrated matches (not in INTERPRO)

        if ($uimatches && @$uimatches > 0) {
            my $unintegrated = { id      => $noiprid,
                                 name    => $noiprname,
                                 type    => $noiprtype,
                                 matches => $uimatches };
            push @$interpros, $unintegrated;
        }

        unless ($interpros) {
            print $ofh "<tr>\n";
            print $ofh "<td colspan=\"4\">No hits reported.</td>\n";
            print $ofh "</tr>\n";
        }

        my $ninterpro = 1;

        foreach my $interpro (@$interpros) {
            if ($ninterpro % 2 == 0) {
                print $ofh "<tr bgcolor=\"#dddddd\">\n";
            }
            else {
                print $ofh "<tr>\n";
            }

	    if($iprlookup || $crc){     ## If user doesn't want to see InterPro entries
	      unless ($interpro->{id} eq $noiprid) {
                print $ofh "<td valign=\"top\" width=\"1%\">";
                print $ofh "<b>InterPro<br>\n</b>";
                print $ofh "<a href=\"$iprurl$interpro->{id}\" target=\"newwindow\">$interpro->{id}</a><br>$interpro->{type}\n";
		
		if ($iprsrsurl) {
		  $iprsrsurl =~ s/\$id/$interpro->{id}/g;
		  print $ofh "<br>\n";
		  print $ofh "<a href=\"$iprsrsurl\" target=\"newwindow\">";
		  
		  if ($imgsrslogo) {
		    print $ofh "<img src=\"$imgsrslogo\" border=\"0\" width=\"66\" height=\"44\">";
		  }
		  else {
		    print $ofh "Link&nbsp;to&nbsp;SRS";
		  }
		  
		  print $ofh "</a>\n";
		}
		
                print $ofh "</td>\n";
	      } # end unless
	      else {
                # no link for the unintegrated
                print $ofh "<td valign=\"top\" width=\"1%\">$interpro->{id}<br>$interpro->{type}</td>\n";
	      }
	    } #iprlookup

            my $matches = $interpro->{matches};
            my $nmatch  = 1;
	    my $urlid;
	    my $colspan = 4;

	    $colspan-- if($crc);

	    #EQ If user asked for InterPro lookup we display InterPro name
	    if($iprlookup || $crc){
	      print $ofh "<td valign=\"top\">\n";
	      print $ofh "<table width=\"100%\" cellSpacing=\"0\" cellPadding=\"2\" border=\"0\">\n";
	      print $ofh "<tr>\n";
	      print $ofh "<td valign=\"top\" colspan=\"$colspan\">\n";
	      print $ofh "<b><nobr>$interpro->{name}</nobr></b>\n";
	      print $ofh "</td>\n";
	      print $ofh "</tr>\n";
	    }

	    my $seen;
	    my %db;

	    for(@$matches){
	      $db{$_->{dbname}}++;
	    }

            foreach my $match (@$matches) {
                print $ofh "<tr bgcolor=\"#ffffff\">\n";

                $entryurl = $self->getConfigValue("entryurl." . lc($match->{dbname}));

		my $wgeturlid = $self->getConfigValue('wgeturl');
		my $tool = lc($match->{locations}[0]->{evidence});
		$tool = $self->mapApplNameToToolName($tool) unless($tool =~ /^hmm/);
		my $paurl = $self->getConfigValue('imgurl.pa');
		#my $pdbview = $self->getConfigValue('url.pdbview') if($tool && $tool =~ /profilescan/i);
		#my $molecule = $self->getConfigValue('imgurl.mol') if($tool && $tool =~ /profilescan/i);
		my $rowspan = $db{$match->{dbname}};

		$wgeturlid =~ s/%toolname/$tool/;
		$wgeturlid =~ s/.input/\-$tool\.output/;

		if($cnk){
		  ($res, $msg) = $self->addChunk("$cnk", \$wgeturlid);
		  return(0, "createResultTable: Cannot add chunk $cnk: $msg") unless $res;
		  $wgeturlid =~ s/\-$tool\.output/\-$tool\-cnk$cnk\.output/;
		}

		my $html;

		if($wgeturlid && !$crc && $seen ne $tool && $self->getParam('format') !~ /^htm/){
		  $html = "<td rowspan=\"$rowspan\" width=\"50\"><a href=\"$wgeturlid$protein->{id}\" target=\"newwindow\"><img src=\"$paurl\" border=\"0\"></a></td>";
		  $seen = $tool;
		}
		$html .= "<td>$match->{dbname}";

		#For further implementations...allow to link ProfileScan entry with 3D/RasMol/PDB entry (MSD at EBI)
		#if($pdbview){
		#  $html .= "&nbsp;<a href=\"$pdbview$match->{id}\" target=\"newwindow\"><img src=\"$molecule\"></a>";
		#}
		
		$html .= "</td><td width=\"10%\">";

		if($entryurl){
		    $urlid = $match->{id};
		    $urlid =~ s/\.\d+$// if (lc($match->{dbname}) eq 'pfam'); #remove Pfam version number

		    $html .= "<a href=\"$entryurl$urlid\" target=\"newwindow\">$match->{id}</a>";
		}
		else {
		  $html .= "$match->{id}</td>\n";
                }
		
		print $html;

                print "<td><i>$match->{name}</i></td>\n";

                my $locations = $match->{locations};
                my ($start, $end, $score, $status, $evidence);

                print "<td>\n";
		print "<table border=\"0\" cellPadding=\"0\"><tr>\n";
		
		my $bgcolor;
		if($ninterpro % 2 == 0){
		  $bgcolor = "#dddddd";
		} else {
		  $bgcolor = "#eeeeee";
		}
		my $tdcmp = 0;
		my $color;
                foreach my $location (@$locations) {
                    $start    = $location->{start};
                    $end      = $location->{end};
                    $score    = $location->{score};
                    $status   = $location->{status};
                    $evidence = $location->{evidence};

		    #EQ
		    if($tdcmp % 4 == 0 && $tdcmp != 0){
		      print "</tr><tr>\n";
		    }
		    if($tdcmp % 2 == 0){
		      $color = "#ffffff";
		    } else {
		      $color = $bgcolor;
		    }
		    #########
                    print "<td bgcolor=\"$color\">&nbsp;$score [$start-$end]$status&nbsp;</td>\n";
		    $tdcmp++;
                }

		#EQ
		while($tdcmp % 4 != 0){
		  print "<td bgcolor=\"#ffffff\">&nbsp;</td>\n";
		  $tdcmp++;
		}
		print "</tr></table>\n";
		###############
                print "</td>\n";
                print "</tr>\n";
            }

            print $ofh "</table>\n";
            print $ofh "</td>\n";
            print $ofh "</tr>\n";

	    if($iprlookup || $crc){
	      if ($ninterpro % 2 == 0) {
                print $ofh "<tr bgcolor=\"#dddddd\">\n";
	      }
	      else {
                print $ofh "<tr>\n";
	      }
	      
	      print $ofh "<td valign=\"top\"><b>Parent</b></td>\n";
	      
	      my $parent = $interpro->{parent_id};
	      if ($parent) {
                print $ofh "<td valign=\"top\"><a href=\"$iprurl$parent\" target=\"newwindow\">$parent</a></td>\n";
	      }
	      else {
                print $ofh "<td valign=\"top\">no&nbsp;parent</td>\n";
	      }
	      
	      print $ofh "</tr>\n";
	      
	      if ($ninterpro % 2 == 0) {
                print $ofh "<tr bgcolor=\"#dddddd\">\n";
	      }
	      else {
                print $ofh "<tr>\n";
	      }
	      
	      print $ofh "<td valign=\"top\"><b>Children</b></td>\n";
	      
	      my $children = $interpro->{child_list};
	      if ($children && @$children > 0) {
                my $child;
		
                print $ofh "<td>";
                foreach $child (@$children) {
		  print $ofh "<a href=\"$iprurl$child\" target=\"newwindow\">$child</a>\n";
                }
                print $ofh "</td>\n";
	      }
	      else {
                print $ofh "<td valign=\"top\">no&nbsp;children</td>\n";
	      }
	      
	      print $ofh "</tr>\n";
	      
	      if ($ninterpro % 2 == 0) {
                print $ofh "<tr bgcolor=\"#dddddd\">\n";
	      }
	      else {
                print $ofh "<tr>\n";
	      }
	      
	      print $ofh "<td valign=\"top\"><b>Found&nbsp;in</b></td>\n";
	      
	      my $found = $interpro->{found_in};
	      if ($found && @$found > 0) {
                my $entry;
		
                print $ofh "<td valign=\"top\">";
                foreach $entry (@$found) {
		  print $ofh "<a href=\"$iprurl$entry\" target=\"newwindow\">$entry</a>\n";
                }
                print $ofh "</td>\n";
	      }
	      else {
                print $ofh "<td valign=\"top\">no entries</td>\n";
	      }
	      
	      print $ofh "</tr>\n";
	      
	      if ($ninterpro % 2 == 0) {
                print $ofh "<tr bgcolor=\"#dddddd\">\n";
	      }
	      else {
                print $ofh "<tr>\n";
	      }
	      
	      print $ofh "<td valign=\"top\"><b>Contains</b></td>\n";
	      
	      my $contains = $interpro->{contains};
	      if ($contains && @$contains > 0) {
                my $entry;
		
                print $ofh "<td valign=\"top\">";
                foreach $entry (@$contains) {
		  print $ofh "<a href=\"$iprurl$entry\" target=\"newwindow\">$entry</a>\n";
                }
                print $ofh "</td>\n";
	      }
	      else {
                print $ofh "<td valign=\"top\">no entries</td>\n";
	      }
	      
	      print $ofh "</tr>\n";
	      if($goterms || $crc){
		if ($ninterpro % 2 == 0) {
		  print $ofh "<tr bgcolor=\"#dddddd\">\n";
		}
		else {
		  print $ofh "<tr>\n";
		}
		
		print $ofh "<td valign=\"top\"><b>GO&nbsp;terms</b></td>\n";
		print $ofh "<td valign=\"top\">\n";
		
		my $classifications = $interpro->{classifications};
		if ($classifications && @$classifications > 0) {
		  my ($classification, $cid);
		  my $gourl = $self->getConfigValue('entryurl.go');
		  
		  foreach $classification (@$classifications) {
		    print $ofh "$classification->{category}: $classification->{description} ";
		    
		    if ($classification->{class_type} eq 'GO') {
		      $cid = $classification->{id};
		      print $ofh "(<a href=\"$gourl$cid\" target=\"newwindow\">$cid</a>)\n";
		    }
		    
		    print $ofh "<br>\n";
		  }
		}
		else {
		  print $ofh "none";
		}
		
		print $ofh "</td>\n";
		print $ofh "</tr>\n";
	      } #goterms
	    } #iprlookup
            $ninterpro++;
	  }
	
        print $ofh "</table>\n";
        print $ofh "<br>\n";
	} #end for($i; $i <= $nbchunk; $i++)
    } #end for($start; $start <= $stop; $start++)
    
    if ($bottom) {
      print $ofh $bottom;
    }
    else {
      print $ofh $self->getResultPageBottom($cnk);
    }
    
    close $ofh if $oclose;
    
    if ($tmpfile) {
      unless (rename($tmpfile, $out)) {
	return (0, "createResultTable: could not rename temporary file $tmpfile: $!");
      }
    }
    
    return (1, '');
}


=head2 filterTaxoResults

    Description: Filter the results for each ids found in a run.
                 It is possible to use 'and, or, and/or' search.
		 N.B. This is no longer used
    Arguments:   $id the id to look for.
    Returns:     1, 1/0 to accept the id on success
                 0, $msg on error
=cut

sub filterTaxoResults {
    my($self, $id) = @_;

    return(0, "filterTaxoResults: No id given") unless $id;

    my($res, $msg, $entries, $inx);
    ($res, $inx) = Index::InterPro->new(undef, 1);
    return(0, "filterTaxoResults: $inx") unless $res;

    $id =~ s/\.\d+$//; #remove Pfam version number.
    $id =~ s/^pd_//;   #BlastProDom filtering.

    ($res, $entries) = $inx->getEntry($id);
    return(0, "filterTaxoResults: $entries") unless $res;

    #We do not display this id, it does not exist in InterPro (unintegrated for example).
    unless($entries){
	return(1, 0); #We return 0 to tell there is not match according to the taxonomy.
    }
    #ID member database should not be present in more than one InterPro entry
    return(0, "filterTaxoResults: More than on InterPro entry contain $id!") if(scalar(@$entries) > 1);

    #Id must be unique in an InterPro entry (in theory) see above.
    my $entry = $entries->[0];
    ($res, $msg) = $inx->parseFields(\$entry);
    return(0, "filterTaxoResults: cannot parse entry: $msg") unless $res;

    my $taxos;

    ($res, $taxos) = $inx->get_taxonomy();
    return(0, "filterTaxoResults: cannot get taxonomy: $taxos") unless $res;

    #Get the full taxonomy name from user.
    my $usertaxos = [ ];
    @$usertaxos = map { $Abrev2Taxo->{$_} } split /,/,$self->getParam('taxonomy');

    my $retvalue;
    ($res, $retvalue) = $self->compareTaxos($taxos, $usertaxos);
    return(0, "filterTaxoResults: cannot compare taxonomy: $retvalue") unless $res;

    return(1, $retvalue);
}

=head2 compareTaxos

    Description: Make a comparison between two lists of taxonomy.
    Arguments:   $iprtaxos reference to an array containing InterPro taxonomy list.
                 $ustaxos  reference to an array containing user taxonomy list.
    Returns:     1, $retvalue on success 
                 0, $msg on error

=cut

sub compareTaxos($$){

    my($self, $iprtaxos, $ustaxos) = @_;
    return(0, "compareTaxos: Need a reference to an array for InterPro taxonomy.") unless($iprtaxos && ref($iprtaxos) eq 'ARRAY');
    return(0, "compareTaxos: Need a reference to an array for user taxonomy. [$ustaxos]") unless($ustaxos && ref($ustaxos) eq 'ARRAY');

    my $htaxos = { };
    for(@$iprtaxos){
	$htaxos->{$_} = 1;
    }

    my $rule = $self->getParam('taxorule');

    my $defret = ($rule == 0 ? 1 : 0);

    for my $name (@$ustaxos){
	#AND
	if($rule == 0){
	    $defret = 0 unless(exists($htaxos->{$name}));
	}#OR
	elsif($rule == 1){
	    $defret = 1 if(exists($htaxos->{$name}));
	}
    }

    return(1, $defret);
}

=head2 createRawResult

 Description: Creates raw result file. All jobs must be finished.
 Arguments:   $seqs        Sequence hash as returned by checkSequences.
              $out         Output handle or file path (default: stdout).
              $jobnames    Job names in an array (optional).
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub createRawResult($;$$) {

    my ($self, $seqs, $h_chunk, $chunk) = @_;

    return (0, "createRawResult: no input sequences defined") unless $seqs;
    return (0, "createRawResult: input sequences must be passed in a hash reference") if $seqs && ref($seqs) ne 'HASH';
    return (0, "createRawResult: need a hash reference to read informations about files") unless($h_chunk && ref($h_chunk) eq 'HASH');

    my($start, $stop, $res, $msg);

    $start = $chunk ? $chunk : 1;
    $stop  = $chunk ? $chunk : $h_chunk->{nbchunk};

    for($start; $start <= $stop; $start++){

	my $out = $self->getConfigValue('toolraw');
	($res, $msg) = $self->addChunk("$start", \$out);
	return(0, "createRawResult: cannot add chunk $msg")  unless($res);
	$out = \*STDOUT unless $out;

	my $params = $self->getParams() || { };
	my ($ofh, $oclose, $tmpfile, $seq) = (undef, undef, undef, undef);

	if ($self->isFileHandle($out)) {
	    ($res, $oclose, $ofh) = $self->getWriteHandle($out);
	}
	else {
	    $tmpfile = "$out.tmp";
	    ($res, $oclose, $ofh) = $self->getWriteHandle($tmpfile);
	}

	return (0, "createRawResult: cannot get write handle $ofh") unless $res;

	my $cnksize = $self->getConfigValue('chunk');
	return(0, "'createRawResult: chunk' tag must be defined in your iprscan.conf") unless($cnksize);

	# print the raw entries from sequences with CRC matches first
	# We need to iterate over the number of sequences into a chunk to write the right rawentry in the right chunk.
	# Otherwise, the rawentry is written into each chunk, so we have duplicate entries in the final raw file.
	foreach $seq (sort keys %$seqs) {
	    for(my $i = 1; $i <= $cnksize; $i++){
		if ($seqs->{$seq}->{rawentry} && $self->getParam("seq.$start.$i.id") eq $seq) {
		    print $ofh $seqs->{$seq}->{rawentry};
		}
	    }
	}
	
	if ($h_chunk->{$start}->{jobs}) {   #if checkcrc is on, this table might be empty if all the crc'sequences are known.
	    my ($jobid, $appl, $tname, $tool, $msg, $raw, $defaults);

	    foreach $jobid (keys %{$h_chunk->{$start}->{jobs}}) {
		
		$appl = $self->mapJobIDToApplName($jobid);
		return (0, "createRawResult: could not map application name from $jobid") unless $appl;

		$tname = $self->mapApplNameToToolName($appl);
		return (0, "createRawResult: $appl not supported") unless $tname;

		if ($jobid && $jobid =~ /^([A-Za-z]+-([\d]{4})(\d\d)(\d\d)-(\d\d)(\d\d)(\d\d)(\d\d))-\w+-cnk(\d+)/) {
		    # set date and time because they may be used in configuration (paths and URLs)
		    $defaults->{YYYY} = $2;
		    $defaults->{MM} = sprintf("%.2d", $3);
		    $defaults->{DD} = sprintf("%.2d", $4);
		    $defaults->{hh} = sprintf("%.2d", $5);
		    $defaults->{mm} = sprintf("%.2d", $6);
		    $defaults->{ss} = sprintf("%.2d", $7);
		    $defaults->{session} = $1;
		    $defaults->{chunk}   = "chunk_$9";
		}
		else {
		    return(0, "createRawResult: could not determine jobid from $jobid") unless $jobid;
		}
		
		$defaults->{jobid}    = $jobid;
		$defaults->{srchtype} = $self->getParam('srchtype');
		
		($res, $tool) = $self->createTool($tname, $defaults);
		return (0, "createRawResult: cannot create tool $tool") unless $res;
		
		my $tooloutput = $tool->getConfigValue('tooloutput');
		next unless -e $tooloutput;  # CRC matched for this sequence
		
		my $result = { };
		
		#EQ Maybe the best place to index the results at the same time than parsing them...
		my $inx =  { };
		($res, $msg) = $tool->parseResult($result, $tooloutput, $inx);
		return (0, "createRawResult: cannot parse result $msg") unless $res;
		
		#wget.pl is working as command line as well.

		#unless($self->getParam('submission') eq 'cli'){
		my $index;
		($res, $index) = Index->new($tooloutput);
		return(0, "createRawResult: $appl : $index") unless($res);
		
		($res, $msg) = $index->indexOut($inx);
		return(0, "createRawResult: $appl : $msg") unless($res);
		
		#}
		
		#Taxonomy filtering on each ID.
		if($self->getConfigValue('taxonomy.use')){
		    if($self->getParam('taxonomy') && ($appl ne 'signalp' && $appl ne 'coils' && $appl ne 'tmhmm' && $appl ne 'seg')){
			
			my $nhits = $result->{nhits};
			
			for (my $i = 1; $i <= $nhits; $i++){
			    my $hit = $result->{$i};
			    my $retvalue;
			    next unless $hit;
			    
			    if($hit->{ndomains}){ #Hmm applications only
				for (my $j = 1; $j <= $hit->{ndomains}; $j++){
				    
				    my $domain = $hit->{$j};
				    next unless $domain;
				    
				    ($res, $retvalue) = $self->filterTaxoResults($domain->{methid});
				    return(0, "createRawResult: $retvalue") unless $res;
				    
				    delete($hit->{$j}) unless $retvalue;
				}
			    } else {
				
				($res, $retvalue) = $self->filterTaxoResults($hit->{methid});
				return(0, "createRawResult: $retvalue") unless $res;
				
				delete($result->{$i}) unless $retvalue;
			    }
			}
		    }
		}
		
		($res, $raw) = $tool->getRawResult($result, $seqs, $self, $self->getParam('iprlookup'), $self->getParam('goterms'));
		return (0, "createRawResult: $appl : $raw") unless $res;
		print $ofh $raw;
	    } #enf foreach $jobid
	}
	close $ofh if $oclose;
	
	if (defined($tmpfile)) {
	    unless (rename($tmpfile, $out)) {
		return (0, "createRawResult: could not rename temporary file $tmpfile: $!");
	    }
	}
    }# end for loop
    return (1, '');
}


=head2 createFailedReport

 Description: create a report for failed jobs.
              It contains command launched, error status and how to relaunch
              the jobs failed and concatenate the whole results.
 Arguments:   $jobid     Job identifier
              $h_chunk   Hash table for chunks
              $file      Report file for the run
              $erfile    Error File from application
              $mode      To overwrite the file if already exists instead of filling it.
 Returns:     1, '' on success
              0, msg   on failure

=cut


sub createFailedReport($$) {

   my ($self, $jobid, $h_chunk, $file, $erfile) = @_;

   return(0, 'createFailedReport: No jobid given') unless($jobid);
   if($h_chunk){
     return(0, "createFailedReport: Need a reference to a hash table to retrieve info") unless($h_chunk && ref($h_chunk) eq 'HASH');
   }
   my $tname = $self->mapJobIDToToolName($jobid);
   my $name  = $self->mapJobIDToApplName($jobid);
   my $defaults = { };
   my ($chunk, $res, $msg, $tool);

   $defaults->{jobid} = $jobid;

   if ($jobid && $jobid =~ /^([A-Za-z]+-([\d]{4})(\d\d)(\d\d)-(\d\d)(\d\d)(\d\d)(\d\d))-\w+-cnk(\d+)/) {
     # set date and time because they may be used in configuration (paths and URLs)
     $defaults->{YYYY} = $2;
     $defaults->{MM} = sprintf("%.2d", $3);
     $defaults->{DD} = sprintf("%.2d", $4);
     $defaults->{hh} = sprintf("%.2d", $5);
     $defaults->{mm} = sprintf("%.2d", $6);
     $defaults->{ss} = sprintf("%.2d", $7);
     $defaults->{session} = $1;
     $defaults->{chunk}   = "chunk_$9";
     $chunk = $9;
   } else {
     return(0, "createFailedReport: could not determine jobid from [$jobid]");
   }

   #workdir is build with jobid and session
   ($res, $tool) = $self->createTool($tname, $defaults);
   return(0, 'createFailedReport: ' . $tname . " creation failed: $tool") unless($res);

   my $report;

   unless($file){
     $report = $self->getConfigValue('toolreport');
     return(0, 'createFailedReport: Toolreport not configured') unless($report);
   } else {
     $report = $file;
   }

   #We adding chunk thus the report file can be visible from the html page. Otherwise they are all
   #concatenated in the root session directory.
   ($res, $msg) = $self->addChunk("$chunk", \$report);
   return(0, "createFailedReport: $msg") unless($res);

   my $error;

   if(exists($h_chunk->{$chunk}->{jobs}->{$jobid}->{errors})){
     $error = $h_chunk->{$chunk}->{jobs}->{$jobid}->{errors};
   }elsif(-f $erfile){
     $error = $erfile;
   }

   if(-f $error){
     return(0, "createFailedReport: Could not open error file $error : $!") unless(open(ERR, "<$error"));
     local $INPUT_RECORD_SEPARATOR;
     $error = <ERR>;
     close(ERR);
     unlink($error);
   }

   unless(open(REP, ">>$report")){
     return(0, "createFailedReport: Could not open report file $report : $!");
   }

   print REP "========================================================================\n";
   print REP "Chunk: $chunk\n";
   print REP "Jobid: $jobid\n";
   print REP "Application: $name\n";
   print REP "Please fix error(s) if necessary or check '$name.conf' file.\n";
   print REP "Errors :\n\n$error\n\n" if($error);
   print REP "========================================================================\n\n";

  close(REP);

  return(1,'');
}


=head2 getRawEntryFromIprMatches

 Description: Queries a raw entry from IPRMATCHES. This method uses the sequence
              CRC64 checksum to query the entry.
 Arguments:   $seqid     Sequence ID.
              $seqcrc    CRC64 checksum.
              $seqlen    Sequence length.
              $appl      Applications (reference to an array). By default all
                         applications in configuration are used.
              $ipr       1: add IPR lookup info 
                         0: no (default)
              $go        1: add GO terms
                         0: no (default)
 Returns:     1, raw entry on success
              0, msg   on failure

=cut

sub getRawEntryFromIprMatches($$$;$$$) {
  my ($self, $seqid, $seqcrc, $seqlen, $appl, $ipr, $go) = @_;
  return (0, "getRawEntryFromIprMatches: query iprmatches failed: no sequence ID defined")     unless $seqid;
  return (0, "getRawEntryFromIprMatches: query iprmatches failed: no sequence CRC64 defined")  unless $seqcrc;
  return (0, "getRawEntryFromIprMatches: query iprmatches failed: no sequence length defined") unless $seqlen;

  if ($appl && ref($appl) ne 'ARRAY') {
    return (0, "getRawEntryFromIprMatches: query iprmatches failed: applications must be passed in a array reference");
  }
  elsif (!$appl || ($appl && @$appl == 0)) {
    my $sapplist = $self->getConfigValue('applications');
    return (0, "getRawEntryFromIprMatches: query iprmatches failed: configuration error, applications not configured") unless $sapplist;

    my @sapps = split(/\,/, $sapplist);
    return (0, "getRawEntryFromIprMatches: query iprmatches failed: configuration error, applications list is empty") unless @sapps > 0;

    $appl = \@sapps;
  }

  my $raw = '';

  eval {
      # suppress the "subroutine redefined" warnings
      local $WARNING = 0;

      my $ofh = new IO::String($raw);

      #### EQ
      my ($res, $msg,$iprmatches, $entries);
      ($res, $iprmatches) = new Index::IprMatches(undef, 1);
      die("Unable to create Index::IprMatches object : $iprmatches") unless($res);

      ($res, $entries) = $iprmatches->getEntry($seqcrc);
      die($entries) unless($res);
      ######
	
      if($entries && ref($entries) eq 'ARRAY') {
	  if(scalar(@$entries) > 1){
	      #I think it is better to launch the analysis and escape the write_raw subroutine in case we have
	      #multiple entries form IprMatches. As we can have same crc64 for different sequences with same length
	      #escaping write_raw avoids to provide wrong results to the user.
	      #Do nothing
	  } else {
	      foreach my $entry (@$entries){ #we only actually have one entry from IprMatches
		  ($res, $msg) = $iprmatches->parseFields(\$entry);
		  die $msg unless $res;
		  my $iprtaxores = { }; # functionality REMOVED

		  ($res, $msg) = $iprmatches->get_seqlen();
		  die $msg unless $res;

		  if($msg->[0] == $seqlen){
		      # CRC64 and sequence length matches so this is the same sequence (in theory)
		      ($res, $msg) = $iprmatches->write_raw($iprmatches->parse(\$entry, $appl, $iprtaxores), $seqid, $ofh, $ipr, $go);
		      die $msg unless $res;
		      last;
		  }
	      }
	  }
      }
  };

  if ($@) {
      return (0, "getRawEntryFromIprMatches: query iprmatches failed: $@");
  }
  return (1, $raw);
}


=head2 getInterProFields

 Description: Queries accession number and name fields from InterPro.
 Arguments:   $hitac    Hit accession number, e.g.
                        'PD019552'    (BlastProDom)
                        'PR00003'     (FingerPrintScan)
                        'PS00317'     (ScanRegExp)
                        'PS50311'     (ProfileScan)
                        'PF00095'     (HMMPfam)
                        'SM00217'     (HMMSmart)
                        'TIGR00010'   (HMMTigr)
                        'PIRSF000196' (HMMPIRSF)
                        'SSF46561'    (HMMSSF)

 Returns:     1, ac, name, goterms on success
              0, msg, undef, undef on failure

=cut

sub getInterProFields($) {
    my ($self, $hitac, $go) = @_; #EQ added $go
    return (0, "getInterProFields: hit accession number not defined") unless $hitac;

    my ($iprac, $iprname, $goterms, $goid, $godesc, $gocat);

    eval {
        # suppress the "subroutine redefined" warnings
        local $WARNING = 0;

	my($res, $entries, $interpro, $msg);
	($res, $interpro) = Index::InterPro->new(undef, 1);
	die("Index::InterPro : $interpro") unless($res);

	($res, $entries) = $interpro->getEntry($hitac);
	die($entries) unless($res);

	if($entries && ref($entries) eq 'ARRAY'){
	  foreach my $entry (@$entries){

	    ($res, $msg) = $interpro->parseFields(\$entry);
	    die $msg unless $res;

	    ($res, $iprac) = $interpro->get_ac();
	    die($iprac) unless($res);
	    $iprac = $iprac->[0];

	    ($res, $iprname) = $interpro->get_name();
	    die $iprname unless $res;
	    $iprname = $iprname->[0];

	    if($go){
	      ($res, $goid) = $interpro->get_goclass();
	      die $goid unless($res);
	    }

	    if($iprac){
	      if($goid && ref($goid) eq 'ARRAY'){
		$goterms = join(', ', @$goid);
	      } else {
		$goterms = '';
	      }
	    }
	  }
	}
	else {
	  $iprac   = $self->getConfigValue('iprscan.iprnullac')   || 'NULL';
	  $iprname = $self->getConfigValue('iprscan.iprnullname') || 'NULL';
	  $goterms = '';
	}

      };

    if ($@) {
        return (0, "getInterProFields: query interpro failed: $@");
    }

    return (1, $iprac, $iprname, $goterms);
  }


=head2 crc64

 Description: Calculates CRC64 checksum from a string.

              This implementation is from Swissknife-1.42 package SWISS:CRC64
              implemeted by Alexandre Gattiker (gattiker@isb-sib.ch).

              http://swissknife.sourceforge.net/

 Arguments:   $in    Input string.
 Returns:     checksum

=cut

sub crc64($) {
    my ($self, $in) = @_;
    return '' unless $in;

    # Initialisation
    # 32 first bits of generator polynomial for CRC64
    # the 32 lower bits are assumed to be zero
    my $POLY64REVh = 0xd8000000; 
    my @CRCTableh = 256;
    my @CRCTablel = 256;
    my $initialized;
    my $crcl = 0;
    my $crch = 0;
    if (!$initialized) {
        $initialized = 1;
        for (my $i=0; $i<256; $i++) {
            my $partl = $i;
            my $parth = 0;
            for (my $j=0; $j<8; $j++) {
                my $rflag = $partl & 1;
                $partl >>= 1;
                $partl |= (1 << 31) if $parth & 1;
                $parth >>= 1;
                $parth ^= $POLY64REVh if $rflag;
            }
            $CRCTableh[$i] = $parth;
            $CRCTablel[$i] = $partl;
        }
    }
  
    foreach (split '', $in) {
        my $shr = ($crch & 0xFF) << 24;
        my $temp1h = $crch >> 8;
        my $temp1l = ($crcl >> 8) | $shr;
        my $tableindex = ($crcl ^ (unpack "C", $_)) & 0xFF;
        $crch = $temp1h ^ $CRCTableh[$tableindex];
        $crcl = $temp1l ^ $CRCTablel[$tableindex];
    }
    return wantarray ? ($crch, $crcl) : sprintf("%08X%08X", $crch, $crcl);
}


=head2 byStartPosition

 Description: Sorts locations by start positions.
 Arguments:
 Returns:

=cut

sub byStartPosition() {
    my ($aa, $bb) = (0, 0);

    $aa = $a->{start};
    $bb = $b->{start};

    return ($aa <=> $bb);
}


1;
__END__
