=head1 NAME

Seg - Seg utilities.

=head1 SYNOPSIS

  # examples here
  use Dispatcher::Tool::Seg;

=head1 DESCRIPTION

=head1 VERSIONS

$Id: Seg.pm,v 1.2 2010/01/20 13:53:08 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

interhelp@ebi.ac.uk
Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut

package Dispatcher::Tool::Seg;

BEGIN {
  unless($ENV{IPRSCAN_HOME}){
    print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
    exit();
  }
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

use lib $ENV{IPRSCAN_LIB};
use Dispatcher::Tool;
@ISA = qw( Dispatcher::Tool );

use Data::Dumper;
use English;
use File::Spec::Functions;
use POSIX qw(strftime);
use Sys::Hostname;
use strict;


=head2 new

 Description: Constructor allocates an anonymous hash, which
              is tied to the class.
 Arguments:   $name        Seg tool name (optional). If given, sets
                           the Dispatcher::Config object automatically inside
                           the object. If not given, the Dispatcher::Config
                           must be set later with setConfig.
              $defaults    Default values for configuration, hash reference
                           (optional).
 Returns:     $self object

=cut

sub new(;$$) {
    my ($class, $name, $defaults) = @_;

    my $self = { };
    bless $self, $class;

    if ($name) {
        $self->setToolName($name);
        my ($res, $msg) = $self->parseConfig(1, $defaults);
        die "failed to parse $name configuration: $msg\n" unless $res;
    }

    return $self;
}


=head2 getCmdLine

 Description: Returns Seg command line.
 Arguments:   -
 Returns:     1, cmd on success
              0, msg on failure

=cut

sub getCmdLine() {
    my $self = shift;
    return $self->getConfigValue('cmdline');
}


=head2 parseResult

 Description: Reads a seg result file and creates a hash table:

              'nhits'   => '2',
              '1'       => { 'seqid'   => 'swall|Q92455|ABF1_TRIRE',
                             'seqdesc' => 'Alpha-L-arabinofuranosidase precursor (EC 3.2.1.55) (Arabinosidase).',
                             'start'   => '5',
                             'end'     => '16'
                           },
              '2'       => { 'seqid'   => 'swall|Q92455|ABF1_TRIRE',
                             'seqdesc' => 'Alpha-L-arabinofuranosidase precursor (EC 3.2.1.55) (Arabinosidase).',
                             'start'   => '369',
                             'end'     => '384'
                           }

 Arguments:   $href          Reference to a hash.
              $result        Path to a Seg result file.
 Returns:     1, ''  on success
              0, msg on failure

=cut

sub parseResult($$) {
    my ($self, $href, $result, $inx) = @_;
    return (0, "no seg input file defined") unless $result;

    $href = { } unless $href && ref($href) eq 'HASH';
    $inx = { } unless $inx && ref($inx) eq 'HASH';
    unless (open(FILE, "< $result")) {
        return (0, "could not read seg result file $result: $!");
    }

    my ($version, $seqid, $seqdesc);
    my $hit = 1;

    my $fpos = 0;

    while (<FILE>) {
        chomp;

        #
        # Example result:
        # 
        # >swall|Q92455|ABF1_TRIRE Alpha-L-arabinofuranosidase precursor (EC 3.2.1.55) (Arabinosidase).
        # 
        # 				    1-4    MLSN
        # 		    ariiaagciaag    5-16   
        # 				   17-368  SLVAAGPCDIYSSGGTPCVAAHSTTRALFS
        # 					   AYTGPLYQVKRGSDGATTAISPLSSGVANA
        # 					   AAQDAFCAGTTCLITIIYDQSGRGNHLTQA
        # 					   PPGGFSGPESNGYDNLASAIGAPVTLNGQK
        # 					   AYGVFVSPGTGYRNNAASGTAKGDAAEGMY
        # 					   AVLDGTHYNGACCFDYGNAETNSRDTGNGH
        # 					   MEAIYFGDSTVWGTGSGKGPWIMADLENGL
        # 					   FSGSSPGNNAGDPSISYRFVTAAIKGQPNQ
        # 					   WAIRGGNAASGSLSTFYSGARPQVSGYNPM
        # 					   SKEGAIILGIGGDNSNGAQGTFYEGVMTSG
        # 					   YPSDATENSVQANIVAARYAVAPLTSGPAL
        # 					   TVGSSISLRATTACCTTRYIAH
        # 		sgstvntqvvssssat  369-384  
        # 				  385-500  ALKQQASWTVRAGLANNACFSFESRDTSGS
        # 					   YIRHSNFGLVLNANDGSKLFAEDATFCTQA
        # 					   GINGQGSSIRSWSYPTRYFRHYNNTLYIAS
        # 					   NGGVHVFDATAAFNDDVSFVVSGGFA
        # 
        if ($_ =~ /^>(.*)/) {
	    $seqid = $self->trim($1);
	    $inx->{$1} = $fpos;
	    if ($seqid =~ /^([^\s]+)\s+(.+)/) {
		$seqid   = $1;
		$seqdesc = $2;
	    }
        }
	elsif ($_ =~ /^\s*[a-zA-Z]+\s+(\d+)-(\d+)\s+/) {
            $href->{$hit}->{seqid}   = $seqid   if $seqid;
            $href->{$hit}->{seqdesc} = $seqdesc if $seqdesc;
	    $href->{$hit}->{methid}  = "";
            $href->{$hit}->{start}   = $1;
            $href->{$hit}->{end}     = $2;
	    $hit++;
        }
	elsif ($_ =~ /^\s*$/){
	  $fpos = tell(FILE);
	}
    }

    $href->{nhits} = $hit - 1;

    close FILE;

    return (1, '');
}


=head2 getRawResult

 Description: Returns raw formatted result string.
 Arguments:   $href      Reference to a result hash.
              $seqs      Sequence hash (see InterProScan::checkSequences).
              $iprscan   Reference to Dispatcher::Tool::InterProScan object.
                         Required for IPR fields and GO terms.
              $ipr       1: add IPR fields
                         0: no (default)
              $go        1: add GO terms
                         0: no (default)
 Returns:     1, str on success
              0, msg on failure

=cut

sub getRawResult($$;$$$) {
    my ($self, $href, $seqs, $iprscan, $ipr, $go) = @_;
    return (0, "no result hash defined")     unless $href && ref($href) eq 'HASH';
    return (0, "no sequences hash defined") unless $seqs && ref($seqs) eq 'HASH';

    if (!$iprscan && ($ipr || $go)) {
	return (0, "no InterProScan object defined");
    }
    elsif ($iprscan && ref($iprscan) ne 'Dispatcher::Tool::InterProScan') {
	return (0, "InterProScan object has wrong class: " . ref($iprscan));
    }

    my $nhits = $href->{nhits};
    return (0, "number of hits not defined in result hash") unless defined $nhits;
    return (1, '') if $nhits == 0;

    my $appl = $self->getConfigValue('applname');
    my $date = POSIX::strftime("%d-%b-%Y", localtime);
    my ($hit, $raw, $seqid, $seqcrc, $seqlen, $res, $iprac, $iprname, $goterms);

    for (my $i = 1; $i <= $nhits; $i++) {
	$hit = $href->{$i};
	next unless $hit;  # should not happen

	$seqid = $hit->{seqid};
	next unless $seqid;  # should not happen

	$seqcrc = $seqs->{$seqid}->{crc64};
	$seqlen = $seqs->{$seqid}->{len};

	# Seg result is very minimal, there are no identifiers or evalues
	$raw .= sprintf("%s\t%s\t%s\t", $seqid, $seqcrc, $seqlen);
        $raw .= sprintf("%s\tseg\tseg\t", $appl);
	$raw .= sprintf("%d\t%d\t", $hit->{start}, $hit->{end});
	$raw .= sprintf("NA\t?\t");  # there is no evalue
	$raw .= sprintf("%s", $date);

	if ($ipr) {
	    # not possible to query Seg matches from InterPro (no ID)
	    $raw .= sprintf("\tNULL\tNULL") if $ipr;
        }

	$raw .= "\n";
    }

    return (1, $raw);
}



1;
__END__

