=head1 NAME

InterPro - InterPro index utilities.

=head1 SYNOPSIS

  use Index::InterPro;


=head1 DESCRIPTION

  This package is used to index interpro.xml file.
  It indexes the database members accession number (see below 'dbkey="foo"').

  # examples here
  use Index::InterPro;

  my($res, $msg, $index);
  ($res, $index) = new Index::InterPro($file); #xml file here for interpro
  die $index unless $res;

  ($res, $msg) = $index->setRecDel('dumper', '</interpro>');
  die($msg) unless($res);

  ($res, $r_inx) = $index->buildIndex(['ac', 'name']); #list of entries you want to index. $regexp in this module for allowing rules.
  die($r_inx) unless($res);                            #if no argument given, will build the index with all the rules describe in $regexp.

  #or you can build the index on your own key-value pairs based on regular expression. Only the first match is taken into account.
  #so if you want to index multiple things based on the same line, you need to create another key-value pair.
  ($res, $r_inx) = $index->builIndex({ 'id' => '>(\S+)', 'name' => '^\s+\w+\s+(\S+)' });
  die $r_inx unless $res;

  ($res, $mess) = $index->indexOut($r_inx);               #need a reference to hash table.
  die $mess unless $res;

  my $id = 'IPR000342';
  my ($res, $pos) = $index->getIndex($id); #return the position in the file of the interpro entry containing this $id.
  die $pos unless $res;

  #getEntry returns a reference to an array in case of multiple entries found.
  my ($res, $entry) = $index->getEntry($id); #return the complete interpro entry.
  die $entry unless $res;
  $entry = $entry->[0] if($entry);

  #Either (parsing once the whole entry)
  my($res, $msg) = $index->parseFields(\$entry);
  die $msg unless $res;

  my ($res, $name) = $index->getField('name'); #return the name of this interpro entry.
  $name = $name->[0] if $name;
  my ($res, $name) = $index->getField(['ac', 'name', 'desc']); #return the ac, name and description of this interpro entry.
  if($name){
      my $nm = $name->[0];
      my $ac = $name->[1];
      my $de = $name->[2];
  }
  
  .... # see below for fields you can retrieve.
  my ($res, $name) = $index->get_name(); #return the name of this interpro entry.
  $name = $name->[0] if $name;

  #or simplier
  my ($res, $name) = $index->getField('name', \$entry); #return the name of this interpro entry by parsing it on the fly
  $name = $name->[0] if $name;

  my ($res, $goterms) = $index->get_goclass(\$entry); #return the GO annotations for this interpro entry.
  die $goterms unless $res;

=head1 VERSIONS

$Id: InterPro.pm,v 1.4 2009/07/14 16:50:15 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut

package Index::InterPro;

BEGIN {
  unless($ENV{IPRSCAN_HOME}){
    print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
    exit();
  }
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

use lib $ENV{IPRSCAN_LIB};
use Index;
@ISA = qw(Index);
#use Dispatcher::Tool;
use strict;
use English;


#We define regular expression for parsing the entry
my $regexp = {
	      ac          => '\s*<interpro +id=.(IPR\d+).',
	      name        => '\s*<name>(.*?)<\/name>',
              short_name  => '\s*<interpro .+short_name=.([^\"]+)',
	      child       => '\s*<rel_ref +ipr_ref=.([^ \'\"]+).',
	      contains    => '\s*<rel_ref +ipr_ref=.([^ \'\"]+).',
	      found       => '\s*<rel_ref\ ipr_ref=.([^ \'\"]+).',
	      parent      => '\s*<rel_ref\ ipr_ref=.([^ \'\"]+).',
	      meth_ac     => '\s*<db_xref +protein_count.*?dbkey=.([^ \'\"]+).',
	      meth_name   => '\s*<db_xref +protein_count.*?name=.([^ \'\"]+).',
	      type        => '\s*<interpro.*?type=.([^ \'\">]+).',
	      class       => '\s*<classification +id=.(GO:[^ \'\"]+).',
	      category    => '\s*<category>(.*?)<\/category>',
	      description => '\s*<description>(.*?)<\/description>',
	      taxonomy    => '\s*<taxon_data +name=.([^\"\']+)'
	     };

#Used to retrieve fields in one go. If you need to put some delimeter in the parsing, use 'rule', 'start' and 'end'...
#In the new method you'll have to use setRules() method to put this hash table into memory
my $rules = {
	      ac          => '\s*<interpro +id=.(IPR\d+).',
	      name        => '\s*<name>(.*?)<\/name>',
              short_name  => '\s*<interpro .+short_name=.([^\"]+)',
	      child       => { rule  => '\s*<rel_ref +ipr_ref=.([^ \'\"]+).',
			       start => '\s*<child_list>',
			       end   => '\s*<\/child_list>'
			     },
	      contains    => { rule  => '\s*<rel_ref +ipr_ref=.([^ \'\"]+).',
			       start => '\s*<contains>',
			       end   => '\s*<\/contains>'
			     },
	      found       => { rule  => '\s*<rel_ref\ ipr_ref=.([^ \'\"]+).',
			       start => '\s*<found_in>',
			       end   => '\s*<\/found_in>'
			     },
	      parent      => { rule  => '\s*<rel_ref\ ipr_ref=.([^ \'\"]+).',
			       start => '\s*<parent_list>',
			       end   => '\s*<\/parent_list>'
			     },
	      meth_ac     => {rule  => '\s*<db_xref +protein_count.*?dbkey=.([^ \'\"]+).',
			      start => '\s*<member_list>',
			      end   => '\s*<\/member_list>'
			      },
	      meth_name   => {rule  => '\s*<db_xref +protein_count.*?name=.([^ \'\"]+).',
			      start => '\s*<member_list>',
			      end   => '\s*<\/member_list>'
			     },
	      type        => '\s*<interpro.*?type=.([^ \'\">]+).',
	      class       => {rule  => '\s*<classification +id=.(GO:[^ \'\"]+).',
			      start => '\s*<classification',
			      end   => '\s*<\/classification>'
			     },
	      category    => {rule  => '\s*<category>(.*?)<\/category>',
			      start => '\s*<category>',
			      end   => '<\/category>'
			     },
	      description => {rule  => '\s*<description>(.*?)<\/description>',
			      start => '\s*<description>',
			      end   => '<\/description>'
			     },
	     taxonomy     => {rule  => '\s*<taxon_data +name=.([^\"\']+)',
			      start => '\s*<taxonomy_distribution>',
			      end   => '\s*<\/taxonomy_distribution>'
			     }
	     };

#We define here correspondance between method and DBname
#The key name must be the same name than in the raw file(case sensitive)
my $databases = {
                 'FPrintScan'  => 'PRINTS',
                 'ProfileScan' => 'PROFILE',
                 'BlastProDom' => 'PRODOM',
		 'HAMAP'       => 'HAMAP',
                 'HMMSmart'    => 'SMART',
                 'HMMPfam'     => 'PFAM',
                 'HMMTigr'     => 'TIGRFAMs',
                 'ScanRegExp'  => 'PROSITE',
                 'PatternScan' => 'PROSITE',
                 'Coil'        => 'COIL',
                 'Seg'         => 'SEG',
                 'TMHMM'       => 'TMHMM',
                 'SignalPHMM'  => 'SIGNALP',
                 'HMMPIR'      => 'PIR',
                 'superfamily' => 'SUPERFAMILY',
		 'Gene3D'      => 'GENE3D',
		 'HMMPanther'  => 'PANTHER'
                };

=head2 new

 Description:  Create a new object Index::InterPro.

 Arguments:    $file a file to index
               $tool Do you want to use Dispatcher::Tool to use index.conf values? (optional)

 Returns:      1, $self on success
               0, msg on failure

=cut


sub new (;$$) {

    my($class, $file, $tool) = @_;

    my $self = bless({ } , $class);

    my($res, $msg);

    if($file){
	($res, $msg) = $self->checkFile($file);
	return(0, $msg) unless $res;
	
	($res, $msg) = $self->setFile($file);
	return(0, $msg) unless($res);
    }

    if($tool){
	($res, $msg) = $self->_init();
	return(0, $msg) unless $res;
    }

    ($res, $msg) = $self->setRegExp($regexp);
    return(0, $msg) unless $res;

    ($res, $msg) = $self->setRules($rules);
    return(0, $msg) unless $res;

    return (1, $self);
}

=head2 _init

       Description: Initialize record delimiter, file and parse configuration file from index configuration file.
       Argument:    
       Returns:     1, '' on success
	            0, msg on error

=cut

 sub _init {

     require Dispatcher::Tool;

     my ($self) = @_;

     my $tool = new Dispatcher::Tool();

     $tool->setToolName('index');

     my($res, $msg) = $tool->parseConfig(1);
     return(0, $msg) unless($res);

     $self->{tool} = $tool;

     my $recdel = $tool->getConfigValue('recdel.interpro');
     return(0, 'No tag input record delimeter [recdel.interpro] configured in index.conf') unless($recdel);

     ($res, $msg) = $self->setRecDel('dumper',$recdel);
     return(0, $msg) unless($res);

    #File already setted and passed as an argument to the new method
    unless($self->getFile()){
	($res, $msg) = $self->setFile($tool->getConfigValue('file.interpro'));
	return(0, $msg) unless($res);
    }

     return (1, '');
}


=head2 get_goclass

 Description:  Retrieve GO terms tags for an InterPro entry.

 Arguments:    

 Returns:      1, reference to an array containing the go informations.
               0, msg on failure

=cut

sub get_goclass {

  my($self) = @_;

  if($self->{parsed}){
      if($self->{fields} && ref($self->{fields}) eq 'HASH'){

	  my $hash = $self->{fields};
	  my ($class, $desc, $cat, $res);
	  $class = $hash->{'class'};
	  $cat   = $hash->{'category'};
	  $desc  = $hash->{'description'};
	  my $go = [ ];

	  if($class && ref($class) eq 'ARRAY' && $desc && ref($desc) eq 'ARRAY' && $cat && ref($cat) eq 'ARRAY'){
	      for(my $i = 0; $i <= $#$class; $i++){
		  push(@$go, "$cat->[$i]: $desc->[$i] ($class->[$i])"); 
	      }
	  }elsif($class && $cat && $desc){
	      push(@$go, "$cat: $desc ($class)");
	  }else{
	      return(1, '');
	  }
	  return(1, $go);

      }
  }else{
      return(0, "get_goclass: Entry is not parsed yet, use parseFields first.");
  }
}

=head2 getDBname

 Description:  Return the DB name corresponding to the Application
               given.

 Arguments:    $entry, reference to a scalar which is the complete entry
               returned by get_entry subroutine.

 Returns:      1, database name or unknown

=cut

sub getDBname {

  my($self, $method) = @_;
  return('unknown') unless($method);
  unless(exists($databases->{$method})){
    return('unknown');
  }

  return $databases->{$method};

}

=head2 getMethId

 Description:  Retrieve methods informations (acc & name) from an InterPro entry.
 Arguments:
 Returns:      1, reference to an array containing strings ('ac: name').
               0, msg on error

=cut

sub getMethId {

    my($self) = @_;

    unless($self->{parsed}){
	return(0, "getMethId: Entry is not parsed yet, use parseFields first.");
    }

    my($res, $ac, $name, $hash);
    $hash = $self->{fields};

    $ac = $hash->{'meth_ac'};
    $name = $hash->{'meth_name'};

    my $ra_meth = [ ];
    if(ref($ac) eq 'ARRAY' && ref($name) eq 'ARRAY'){
	for(my $i = 0; $i < scalar(@$ac); $i++){
	    push(@$ra_meth, "$ac->[$i]: $name->[$i]");
	}
    }else{
	push(@$ra_meth, "$ac: $name");
    }
    return(1, $ra_meth);
}

1;
