=head1 NAME

IprMatches - IprMatches index utilities.

=head1 SYNOPSIS

use Index::IprMatches;

=head1 DESCRIPTION

  This package is used to index match.xml file.
  It indexes the crc64 attribute and the id of the proteins (see below).
  It also creates a raw file if a crc64 of one of your sequence(s)
  matches the known crc64 in the match.xml file (which contain the known
  hit proteins in InterPro).

  # examples here
  use Index::IprMatches;

  my($res, $msg, $index);
  ($res, $index) = new Index::IprMatches($file); #xml file here for iprmatches
  die $index unless $res;

  ($res, $msg) = $index->setRecDel('dumper', '</protein>');
  die($msg) unless($res);

  ($res, $r_inx) = $index->buildIndex(['ac', 'name']); #list of entries you want to index. $regexp in this module for allowing rules.
  die($r_inx) unless($res);                            #if no argument given, will build the index with all the rules describe in $regexp.

  #or you can build the index on your own key-value pairs based on regular expression. Only the first match is taken into account.
  #so if you want to index multiple things based on the same line, you need to create another key-value pair.
  ($res, $r_inx) = $index->builIndex({ 'id' => '>(\S+)', 'name' => '^\s+\w+\s+(\S+)' });
  die $r_inx unless $res;

  ($res, $mess) = $index->indexOut($r_inx);               #need a reference to hash table.
  die $mess unless $res;

  my $id = 'RS16_ECOLI';
  my ($res, $pos) = $index->getIndex($id); #return the position in the file of the match.xml entry containing this $id.
  die $pos unless $res;

  #getEntry returns a reference to an array in case of multiple entries found.
  my ($res, $entry) = $index->getEntry($id); #return the complete entry.
  die $entry unless $res;
  $entry = $entry->[0] if($entry);

  #Either (parsing once the whole entry)
  my($res, $msg) = $index->parseFields(\$entry);
  die $msg unless $res;

  my ($res, $name) = $index->getField('name'); #return the name of this entry.
  $name = $name->[0] if $name;
  my ($res, $name) = $index->getField(['ac', 'name', 'desc']); #return the ac, name and description of this entry.
  if($name){
      my $nm = $name->[0];
      my $ac = $name->[1];
      my $de = $name->[2];
  }

  .... # see below for fields you can retrieve.
  my ($res, $name) = $index->get_name(); #return the name of this entry.
  $name = $name->[0] if $name;
  #or simplier
  my ($res, $name) = $index->getField('name', \$entry); #return the name of this entry by parsing it on the fly
  $name = $name->[0] if $name;

  #Specific function for this module.
  #==================================
  my $ofh = \*STDIN; #(File handle);
  my $appl = [qw(BlastProDom Coil Seg FingerPrintScan)];
  ($res, $msg) = $index->write_raw($index->parse(\$entry, $appl), $seqid, $ofh, $ipr, $go);
  die $msg  unless $res;


=head1 VERSIONS

$Id: IprMatches.pm,v 1.7 2008/09/03 14:26:11 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

Sarah Hunter <hunter@ebi.ac.uk>
Emmanuel Quevillon <tuco@ebi.ac.uk>

=cut

package Index::IprMatches;

BEGIN {
  unless($ENV{IPRSCAN_HOME}){
    print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
    exit();
  }
  unless($ENV{IPRSCAN_LIB}){
    $ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
  }
}

use lib $ENV{IPRSCAN_LIB};
use Index;
@ISA = qw(Index);
use strict;
use English;


#We define regular expression for parsing the entry
my $regexp = {
	seqlen    => '\s*<protein +.*?length=.([^\"\']+)',
	crc       => '\s*<protein +.*?crc64=.([^\"\']+)',
	id        => '\s*<protein +id=.([^\"\']+)',
	name      => '\s*<protein +.*?name=.([^\"\']+)',
	iprid     => '\s*<ipr +id=.([^\"\']+)',
	iprnm     => '\s*<ipr +.*name=.([^\"\']+)',
	iprtype   => '\s*<ipr +.type=.([^\"\']+)',
	matchid   => '\s*<match +id=.([^\"\']+)',
	matchnm   => '\s*<match +.*name=.([^\"\']+)',
	matchdbnm => '\s*<match +.*dbname=.([^\"\']+)',
	matchevd  => '\s*<match +.*evd=.([^\"\']+)',
	matchstat => '\s*<match +.*status=.([^\"\']+)',
	locstart  => '\s*<lcn +start=.([^\"\']+)',
	locend    => '\s*<lcn +.*end=.([^\"\']+)',
	locscore  => '\s*<lcn +.*score=.([^\"\']+)'
};

#Used to retrieve fields in one go. If you need to put some delimeter in the parsing, use 'rule', 'start' and 'end'...
#In the new method you'll have to use setRules() method to put this hash table into memory
my $rules = {
	seqlen     => '\s*<protein +.*?length=.([^\"\']+)',
	crc        => '\s*<protein +.*?crc64=.([^\"\']+)',
	id         => '\s*<protein +id=.([^\"\']+)',
	name       => '\s*<protein +.*?name=.([^\"\']+)',
	iprid      => '\s*<ipr +id=.([^\"\']+)',
	iprnm      => '\s*<ipr +.*name=.([^\"\']+)',
	iprtype    => '\s*<ipr +.*type=.([^\"\']+)',
	matchid    => { start => '\s*<match',
		end    => '\s*<\/match>',
		rule   => '\s*<match +id=.([^\"\']+)'
	},
	matchnm    => { start => '\s*<match',
		end    => '\s*<\/match>',
		rule   => '\s*<match +.*name=.([^\"\']+)'
	},
	matchdbnm  => { start => '\s*<match',
		end    => '\s*<\/match>',
		rule   => '\s*<match +.*dbname=.([^\"\']+)'
	},
	matchstat  => { start => '\s*<match',
		end    => '\s<\/match>',
		rule   => '\s<match +.*status=.([^\"\']+)'
	},
	matchevd   => { start   => '\s*<match',
		end    => '\s<\/match>',
		rule   => '\s<match +.*evd=.([^\"\']+)'
	},
	locstart   => { start => '\s*<lcn',
		end    => '\s*<lcn.*\/>',
		rule   => '\s*<lcn +start=.([^\"\']+)'
	},
	locend     => { start => '\s*<lcn',
		end    => '\s*<lcn.*\/>',
		rule   => '\s*<lcn +.*end=.([^\"\']+)'
	},
	locscore   => { start => '\s*<lcn',
		end    => '\s*<lcn.*\/>',
		rule   => '\s*<lcn +.*score=.([^\"\']+)'
	},
};

=head2 new

 Description:  Create a new object Index::IprMatches.

 Arguments:    $in a file to index
               $tool Do you want to use Dispatcher::Tool to use index.conf values? (optional)

 Returns:      1, $self on success
               0, msg on failure

=cut


sub new (;$$) {

    my($class, $file, $tool) = @_;

    my $self = bless({ } , $class);

    my($res, $msg);

    if($file){
	($res, $msg) = $self->checkFile($file);
	return(0, $msg) unless $res;

	($res, $msg) = $self->setFile($file);
	return(0, $msg) unless($res);
    }

    if($tool){
	($res, $msg) = $self->_init();
	return(0, $msg) unless $res;
    }

    ($res, $msg) = $self->setRegExp($regexp);
    return(0, $msg) unless $res;

    ($res, $msg) = $self->setRules($rules);
    return(0, $msg) unless $res;

    return (1, $self);
}

=head2 _init

 Description: Initialize record delimiter, file and parse configuration file from index configuration file.
 Argument:    
 Returns:     1, '' on success
              0, msg on error

=cut

sub _init {

    require Dispatcher::Tool;

    my ($self) = @_;

    my $tool = new Dispatcher::Tool();

    $tool->setToolName('index');

    my($res, $msg) = $tool->parseConfig(1);
    return(0, $msg) unless($res);

    $self->{tool} = $tool;

    my $recdel = $tool->getConfigValue('recdel.match');
    return(0, 'No tag input record delimeter [recdel.match] configured in index.conf') unless($recdel);

    ($res, $msg) = $self->setRecDel('dumper',$recdel);
    return(0, $msg) unless($res);

    #File already setted and passed as an argument to the new method
    unless($self->getFile()){
	($res, $msg) = $self->setFile($tool->getConfigValue('file.match_full'));
	return(0, $msg) unless($res);
    }

    return (1, '');
}


=head2 parse

 Description:  Parse the entry retrieved from the indexed file.

 Arguments:    $text, scalar reference to the complete entry.
               $r_appl, reference to an applications array.
	       $r_taxo, reference to a taxonomy list array (optional) when user filters output with taxonomy.
 Returns:      reference of three hash tables.
               0, msg on failure

=cut

sub parse ($$;$$$) {

    my ($self, $text, $r_appl, $r_taxos) = @_;

    if(!ref($text) or $$text eq ""){
	return(0, "parse: Unable to parse XML entry...! Empty entry or not a reference to a Scalar.");
    }
    if(!ref($r_appl) or ref($r_appl) ne "ARRAY"){
	return(0, "parse: You need to give a reference to an array...!");
    }
    if($r_taxos){
	if(!ref($r_taxos) or ref($r_taxos) ne 'HASH'){
	    return(0, "parse: You need to give a reference to a hash table... !");
	}
    }

    if(!scalar(@{$r_appl})){
	return(0, "parse: Array is empty. No applications selected...!");
    }
    my %prot = ();
    my %ipr = ();
    my %matches = ();
    my %convert = (
		   GENE3D   => 'Gene3D',
		   PANTHER  => 'HMMPanther',
		   PRODOM   => 'BlastProDom',
		   SSF      => 'superfamily',
		   PFAM     => 'HMMPfam',
		   PROSITE  => 'PatternScan',	# changed for prosite patterns (no longer scanregexp)
		   #PREFILE  => 'ProfileScan',	# no longer necessary
		   PRINTS   => 'FPrintScan',
		   SMART    => 'HMMSmart',
		   PIRSF    => 'HMMPIR',
		   PROFILE  => 'ProfileScan',
		   TIGRFAMs => 'HMMTigr'
		  );

    #Example here of what it looks like
    #
	# <protein id="O00060" name="CYPH_UROFA" length="163" crc64="A00A641CBA0DBD1D">
	#  <match id="G3D.2.40.100.10" name="G3D.2.40.100.10" dbname="GENE3D" status="T" evd="not_rel">
	#   <lcn start="1" end="163" score="4.5E-64" />
	#  </match>
	#   <match id="PF00160" name="Pro_isomerase" dbname="PFAM" status="T" evd="HMMPfam">
	#    <ipr id="IPR002130" name="Peptidyl-prolyl cis-trans isomerase, cyclophilin type" type="Domain" />
	#     <lcn start="3" end="163" score="1.1E-101" />
	#   </match>
	# </protein>
        # #matchless protein:
        # <protein id="A0A008" name="A0A008_9ACTO" length="427" crc64="B4CF4DF9B01E6A7D">
        # </protein>
        # #but N.B. this code expects a closing protein tag because of the recdel in iprscan.conf..!


    my $dbid = "";
    my $iprid = "";
    my $acc = "";
	my $evd = "";
    my $matchbool = 0;

    foreach my $line (split /\n/, $$text){
		# match the protein info
		if ($line =~ /\s*<protein\s+id=.([^\"]+).\s+name=.([^\"]+).\s+length=.([^\"]+).\s+crc64=.([^\"]+).+/){
			$prot{$1} = {id => "$1", name => "$2", length => "$3", crc64 => "$4"};
			$acc = $1;
		}
                                                                                                                                                             
		# match the erm... match info :-) (if there is any...)
		if($line =~ /^\s+<match\s+id=.([^\"]+).\s+name=.([^\"]+).\s+dbname=.([^\"]+).\s+status=.([^\"]+).\s+evd=.([^\"]+).+>/ && $acc) {
			$matchbool = 0;

			# we only want to display results for the requested applications
			next unless (grep {/$convert{$3}/i} @{$r_appl}); # translate using the hash and check "r_appl" for requested application

			# Store the right info (match id, name, dbname, status and evidence; initialise an anon array for location)
			# matches{"O00060"}{"PF00160"} = {id => "PF00160", name => "Pro_isomerase", dbname => "PFAM", status => "T", evd => "HMMPfam"..}
                        # N.B. We don't want to store False Positive (F) matches, so skip
                        next if ($4 eq "F");

                        $matches{$acc}{$1} = {  id => "$1",
                                                name => "$2",
                                                dbname => "$convert{$3}",
                                                status => "$4",
                                                evd => "$5",
                                                iprbool => 0,
                                                lcn => []
                                            } unless ($matches{$acc}{$1}); # unless statement checks for (technically illegal) repeated <match> elements in the entry
			$dbid = $1;
			$evd = $5;

			$matchbool = 1;
		}

		# see if the entry match has an ipr associated with it or not
		if ($line =~ /^\s+<ipr\s+id=.([^\"]+).\s+name=.([^\"]+)./) {
			next unless ($matchbool); # go to the next in the loop if no matchbool set (may have been skipped for various reasons)
			$matches{$acc}{$dbid}{iprbool} = 1;
			
			# Store the right info
			# ipr{"O00060"}{"PF00160"} = {id => "IPR002130", name => "Peptidyl-prolyl cis-trans isomerase, cyclophilin type"}
			$ipr{$acc}{$dbid} = {id => "$1", name => "$2"};
			$iprid = $1;
		}

		if ($line =~ /^\s+<lcn\s+start=.([^\"]+).\s+end=.([^\"]+).\s+score=.([^\"]+).\s/) {
			# If no score in the lcn line, then add score="N.A."

			next unless ($matchbool);

			# next if ($1 == 0); # Removed as it means that positionless matches won't be reported
			if ($3 ne "") {
				push (@{$matches{$acc}{$dbid}{lcn}}, {start => "$1", end => "$2", evalue => "$3"});
			} else {
				push (@{$matches{$acc}{$dbid}{lcn}}, {start => "$1", end => "$2", evalue => "N.A."});
			}
		}

		if ($line =~ /^\s+<lcn\s+status=.([^\"]+).\s+/){
			next unless($matchbool);
			push(@{$matches{$acc}{$dbid}{lcn}}, {start => "0", end => "0", status => "?", evidence => "ProfileScan"});
		}
	}

	return (\%prot, \%ipr, \%matches);
}


=head2 write_raw

 Description:  Creates raw file for entries retieved and parsed from match.xml

 Arguments:    $r_prot, reference to hash table containing protein name as key
               $r_ipr, reference to hash table containing ipr entries, protein name as key
               $r_matches, reference to hash table containing matches entries as key
               $seq_id, ID of the sequence returned this iprmatches entry.
               $fh, a reference to a file handle to write in.
               $ipr, iprlookup asked?
               $go, go terms asked?

 Returns:      1, '' on success.
               0, msg on failure

=cut

sub write_raw ($$$$$;$$$$$$$) {

	use Index::InterPro;


	my($res, $index) = new Index::InterPro(undef, 1);
	return(0, "write_raw: $index") unless $res;

	my($self, $r_prot, $r_ipr, $r_matches, $seq_id, $fh, $ipr, $go) = @_;
	if(ref($r_prot) ne "HASH" or ref($r_ipr) ne "HASH" or ref($r_matches) ne "HASH"){
		return(0, "IprMatches::write_raw: Can not write the raw file, wrong references passed to the function, need Hash references...\n");
	}

	my $i = 0;
	my $date = `date +"%d-%b-%Y"`;
	$date =~ s/\n//g;

	foreach my $protid (keys %{$r_prot}){
		foreach my $dbid (keys %{$r_matches->{$protid}}) {
			for($i = 0; $i <= $#{$$r_matches{$protid}{$dbid}{lcn}}; $i++){

				my $go_entry;
				#Perform the GO lookup.
				if($go && $$r_matches{$protid}{$dbid}{iprbool}){
					my $go_class;
					($res, $go_entry) = $index->getEntry($$r_ipr{$protid}{$dbid}{id});
					return(0, "write_raw: $go_entry") unless $res;

					if (ref($go_entry) && ref($go_entry) ne "ARRAY") {
						return (0, "write_raw: not an array");
					} elsif (ref($go_entry)) {
						# get_entry returns an array ref. 
						# A database member acc/id should be unique on all the interpro database.
						$go_entry = $go_entry->[0]; 

						($res, $go_entry) = $index->parseFields(\$go_entry);
						return(0, "write_raw: Cannot parse go_entry $go_entry") unless $res;

						($res, $go_class) = $index->get_goclass();
						return(0, "write_raw: $go_class") unless $res;

						$go_entry = (ref($go_class) eq 'ARRAY' ? join(', ', @{$go_class}) : '');
					}
				}

				my $line = sprintf("%s\t%s\t%s\t", $seq_id, $$r_prot{$protid}{crc64},$$r_prot{$protid}{length});
				$line   .= sprintf("%s\t%s\t%s\t", $$r_matches{$protid}{$dbid}{dbname}, $$r_matches{$protid}{$dbid}{id}, $$r_matches{$protid}{$dbid}{name});
				$line   .= sprintf("%s\t%s\t%s\t", $$r_matches{$protid}{$dbid}{lcn}[$i]{start}, $$r_matches{$protid}{$dbid}{lcn}[$i]{end}, $$r_matches{$protid}{$dbid}{lcn}[$i]{evalue});
				$line   .= sprintf("%s\t%s", $$r_matches{$protid}{$dbid}{status}, $date);

				if ($ipr) {
					if ($$r_matches{$protid}{$dbid}{iprbool}) {
						$line .= sprintf ("\t%s\t%s", $$r_ipr{$protid}{$dbid}{id}, $$r_ipr{$protid}{$dbid}{name});
					} else {
						$line   .= sprintf("\t%s\t%s", "NULL", "NULL");
					}
				}

				$line   .= sprintf("\t%s", $go_entry) if($go and $ipr);

				print $fh "$line\n";

			} #end for
		} #end $dbid
	} #end $protid

	return(1, '');
} #end write_raw()

1;
