=head1 NAME

SmartOverlaps - Index smart.overlaps file

=head1 SYNOPSIS

  use Index::SmartOverlaps;

=head1 DESCRIPTION

  This package is used to index smart.overlaps file (not public).
  It indexes the file by the domain name (e.g. DEXDc2)

	     #name     |    domain    |  typ  | number
	#--------------+--------------+-------+--------
	 #DEXDc        | DEXDc2       | merge |      2
	 #DEXDc        | DEXDc        | merge |      1
	 #DEXDc        | DEXDc3       | merge |      3
	 #EGF_like     | EGF_like     | split |      4
	 #EGF_like     | EGF          | split |      3
	 #EGF_like     | EGF_CA       | split |      1
	 #EGF_like     | EGF_Lam      | split |      2
	 #HELICc       | HELICc       | merge |      1
  # ...

  # examples here
  use Index::SmartOverlaps;


  my($res, $msg, $index);
  ($res, $index) = new Index::SmartOverlaps($file); #Smart file to index
  die $index unless $res;

  ($res, $msg) = $index->setRecDel('dumper', '\n');
  die($msg) unless($res);

  ($res, $r_inx) = $index->buildIndex(['famid','domain']); #list of entries you want to index. $regexp in this module for allowing rules.
  die($r_inx) unless($res);                            #if no argument given, will build the index with all the rules describe in $regexp.

  #or you can build the index on your own key-value pairs based on regular expression. Only the first match is taken into account.
  #so if you want to index multiple things based on the same line, you need to create another key-value pair.
  ($res, $r_inx) = $index->buildIndex({ 'famid' => '^\s+(\w+)', 'domain' => '^\s+\w+\s+\|\s+(\S+)' });
  die $r_inx unless $res;

  ($res, $mess) = $index->indexOut($r_inx);               #need a reference to hash table.
  die $mess unless $res;

  my $method_name = 'DEXDc';
  my ($res, $pos) = $index->getIndex($method_name); #return the position in the file for this $id.
  die $pos unless $res;

  #getEntry returns a reference to an array in case of multiple entries found.
  my ($res, $entry) = $index->getEntry($family_name); #return the complete entry.
  die $entry unless $res;
  $entry = $entry->[0] if($entry);

  #Either (parsing once the whole entry)
  my($res, $msg) = $index->parseFields(\$entry);
  die $msg unless $res;

  my ($res, $type) = $index->getField('type'); #return the type of this entry.
  $type = $type->[0] if $name;
  my ($res, $list) = $index->getField(['famiid', 'domain', 'type', 'priority']); #return the family name, domain name, type and number (priority) of this entry.
  if($list){
      my $fam = $name->[0];
      my $dom = $name->[1];
      my $typ = $name->[2];
      my $num = $name->[3];
  }

  #.... # see below for fields you can retrieve.
  #my ($res, $name) = $index->get_name(); #return the name of this smart entry.
  #$name = $name->[0] if $name;
  ##or simplier
  #my ($res, $name) = $index->getField('name', \$entry); #return the name of this smart entry by parsing it on the fly
  #$name = $name->[0] if $name;
#

=head1 VERSIONS

$Id: SmartOverlaps.pm,v 1.2 2009/07/28 12:51:49 hunter Exp $

Copyright (c) European Bioinformatics Institute 2002

=head1 AUTHORS / ACKNOWLEDGEMENTS

Sarah Hunter <hunter@ebi.ac.uk>
Robert Petryszak <rpetry@ebi.ac.uk>

=cut

package Index::SmartOverlaps;

BEGIN {
    unless($ENV{IPRSCAN_HOME}){
	print STDERR "$0 : IPRSCAN_HOME environment variable not set!\n";
	exit();
    }
    unless($ENV{IPRSCAN_LIB}){
	$ENV{IPRSCAN_LIB} = "$ENV{IPRSCAN_HOME}/lib";
    }
}

use lib $ENV{IPRSCAN_LIB};
use Index;
@ISA = qw(Index);
use strict;
use English;


#We define rule to parse the entry
my $regexp = {
	    famid       => '\s+(\w+)\s+\|',
	    domain      => '\s+\w+\s+\|\s+(\w+)\s+\|',
	    type        => '\s+\w+\s+\|\s+\w+\s+\|\s+(\w+)\s+\|',
	    priority    => '\s+\w+\s+\|\s+\w+\s+\|\s+\w+\s+\|\s+(\d+)',
};


=head2 new

 Description:	Create a new object Index::SmartOverlaps.

 Arguments:	$file a file to index
		$tool Do you want to use Dispatcher::Tool to use index.conf values? (optional)

 Returns:	1, $self on success
		0, msg on failure

=cut


sub new (;$$) {

    my($class, $file, $tool) = @_;

    my $self = bless({ } , $class);

    my($res, $msg);

    if($file){
	($res, $msg) = $self->checkFile($file);
	return(0, $msg) unless($res);

	($res, $msg) = $self->setFile($file);
	return(0, $msg) unless($res);
    }

    if($tool){
	($res, $msg) = $self->_init();
	return(0, $msg) unless($res);
    }

    ($res, $msg) = $self->setRegExp($regexp);
    return(0, $msg) unless $res;

    return (1, $self);
}

=head2 _init

 Description:	Initialize record delimiter, file and parse configuration file from index configuration file.
 Arguments:    
 Returns:	1, '' on success
		0, msg on error

=cut

sub _init {

    require Dispatcher::Tool;

    my ($self) = @_;

    my $tool = new Dispatcher::Tool();

    $tool->setToolName('index');

    my($res, $msg) = $tool->parseConfig(1);
    return(0, $msg) unless($res);

    $self->{tool} = $tool;

    my $recdel = $tool->getConfigValue('recdel.smart.overlaps');
    return(0, 'No tag input record delimeter [recdel.smart.overlaps] configured in index.conf') unless($recdel);

    ($res, $msg) = $self->setRecDel('dumper',$recdel);
    return(0, $msg) unless($res);

    #File already set and passed as an argument to the new method
    unless($self->getFile()){
	($res, $msg) = $self->setFile($tool->getConfigValue('file.smart.overlaps'));
	return(0, $msg) unless($res);
    }

    return (1, '');
}

1;
